Package: netresponse
Type: Package
Title: Functional Network Analysis
Version: 1.73.0
Date: 2023-04-03
Author: Leo Lahti, Olli-Pekka Huovilainen, Antonio Gusmao and Juuso
        Parkkinen
Maintainer: Leo Lahti <leo.lahti@iki.fi>
Description: Algorithms for functional network analysis. Includes an
        implementation of a variational Dirichlet process Gaussian
        mixture model for nonparametric mixture modeling.
License: GPL (>=2)
Depends: R (>= 2.15.1), BiocStyle, Rgraphviz, rmarkdown, methods,
        minet, mclust, reshape2
Imports: ggplot2, graph, igraph, parallel, plyr, qvalue, RColorBrewer
Suggests: knitr
URL: https://github.com/antagomir/netresponse
BugReports: https://github.com/antagomir/netresponse/issues
biocViews: CellBiology, Clustering, GeneExpression, Genetics, Network,
        GraphAndNetwork, DifferentialExpression, Microarray,
        NetworkInference, Transcription
VignetteBuilder: knitr
RoxygenNote: 7.2.3
Config/pak/sysreqs: cmake libglpk-dev make libicu-dev libuv1-dev
        libxml2-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:34:52 UTC
RemoteUrl: https://github.com/bioc/netresponse
RemoteRef: HEAD
RemoteSha: 8072ad494852aeac63b603fae099ccdd7fea300d
NeedsCompilation: yes
Packaged: 2026-05-04 09:13:21 UTC; root
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-04 09:25:18 UTC; unix

# Generated by roxygen2: do not edit by hand

export(centerData)
export(continuous.responses)
export(detect.responses)
export(enrichment.list.factor.minimal)
export(factor.responses)
export(factor.responses.minimal)
export(find.similar.features)
export(generate.toydata)
export(get.model.parameters)
export(independent.models)
export(list.responses.continuous.multi)
export(list.responses.continuous.single)
export(listify.groupings)
export(mixture.model)
export(model.stats)
export(order.responses)
export(pick.model.parameters)
export(plot_matrix)
export(plot_response)
export(plot_responses)
export(plot_subnet)
export(response2sample)
export(sample2response)
export(set.breaks)
export(vdp.mixt)
exportMethods(get.subnets)
import(RColorBrewer)
import(Rgraphviz)
import(graph)
import(mclust)
import(methods)
import(parallel)
importFrom(BiocStyle,pkg_ver)
importFrom(ggplot2,aes)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,qplot)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_set)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,gray)
importFrom(grDevices,palette)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(igraph,graph.data.frame)
importFrom(igraph,graph.edgelist)
importFrom(igraph,igraph.from.graphNEL)
importFrom(igraph,igraph.to.graphNEL)
importFrom(igraph,subgraph)
importFrom(minet,build.mim)
importFrom(plyr,ddply)
importFrom(qvalue,qvalue)
importFrom(reshape2,melt)
importFrom(rmarkdown,render)
importFrom(stats,cor)
importFrom(stats,dnorm)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,prcomp)
importFrom(stats,princomp)
importFrom(stats,qchisq)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(utils,read.csv)
useDynLib(netresponse)

[![Build Status](https://api.travis-ci.org/antagomir/netresponse.png)](https://travis-ci.org/antagomir/netresponse)
<!--[![Stories in Ready](https://badge.waffle.io/antagomir/netresponse.png?label=Ready)](http://waffle.io/antagomir/netresponse)-->
<!--[![DOI](https://zenodo.org/badge/4203/rOpenGov/sorvi.png)](https://github.com/rOpenGov/sorvi)-->

netresponse 
===========

Tools for functional network analysis (follow links for references):

 * [netresponse](http://bioinformatics.oxfordjournals.org/content/26/21/2713)
   discover context-specific activation patterns in interaction
   networks

 * [Interaction Component Model
   ICMg](http://www.biomedcentral.com/1752-0509/4/4) to discover
   network modules. Can be supervised by functional information of the
   network.

For installation and usage, check the [tutorial page](vignettes/NetResponse.md).

<!--_Bioinformatics_ 2011. {[Abstract](http://bioinformatics.oxfordjournals.org/content/26/21/2713) | [Preprint pdf](http://www.roihu.info/publications/preprints/Lahti10bioinf-preprint.pdf)}-->


### Support

You are welcome to:
  
  * [submit suggestions and bug reports](https://github.com/antagomir/netresponse/issues)
  * [send a pull request](https://github.com/antagomir/netresponse/)
  * ping antagomir at IRCnet or FreeNode


### Authors

Maintainer: [Leo Lahti](http://www.iki.fi/Leo.Lahti)

Contributors: Olli-Pekka Huovilainen, Antonio Gusmao, Juuso Parkkinen  

Acknowledgements: This program is partially based on the
[AIVGA](http://www.cis.hut.fi/projects/ivga/) Agglomerative
Independent Variable Group Analysis source code (v. 1.0) Copyright (C)
2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa,
Harri Valpola, and Paul Wagner.

[Matlab
implementation](http://www.cis.hut.fi/projects/mi/software/NetResponse)
is also available but not supported.

## ----setup, include=FALSE-----------------------------------------------------
library(knitr)

# Global options
opts_chunk$set(fig.path="fig/", eval=FALSE)

## ----NetResponse1a, warning=FALSE, message=FALSE------------------------------
# library(netresponse)
# res <- generate.toydata(Dim = 3, Nc = 3, Ns = 200, sd0 = 3, rgam.shape = 1, rgam.scale = 1)
# 
# D <- res$data
# component.means <- res$means
# component.sds   <- res$sds
# sample2comp     <- res$sample2comp
# 
# # Use fully connected network
# network <- matrix(rep(1, 9), nrow = 3)

## ----NetResponse1b, warning=FALSE, message=FALSE------------------------------
# # Various network formats are supported, see help(detect.responses) for
# # details. With large data sets, consider the 'speedup' option.
# set.seed(4243)
# res <- detect.responses(D, network, mixture.method = "vdp", pca.basis = TRUE)
# 
# # List subnets (each is a list of nodes)
# subnet.id <- names(get.subnets(res))[[1]]

## ----NetResponse2, fig.width=6, fig.height=6, warning=FALSE, message=FALSE, fig.show="hide", eval=FALSE----
# library(ggplot2)
# vis <- plot_responses(res, subnet.id, plot_mode = "pca")

## ----NetResponse2b, fig.width=6, fig.height=5, warning=FALSE, message=FALSE, eval=FALSE----
# # Modify the resulting ggplot2 object to enhance visualization
# p <- vis$p # Pick the ggplot2 object from results
# p <- p + geom_point(size = 3) # Modify point size
# print(p)

## ----NetResponse3, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE----
# vis <- plot_responses(res, subnet.id, plot_mode = "network")

## ----NetResponse4, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE----
# vis <- plot_responses(res, subnet.id, plot_mode = "heatmap")

## ----NetResponse5, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE----
# vis <- plot_responses(res, subnet.id, plot_mode = "boxplot_data")

## ----NetResponse7, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE----
# plot_scale(vis$breaks, vis$palette, two.sided = TRUE)

## ----NetResponse8, warning=FALSE, message=FALSE-------------------------------
# subnet.id <- 'Subnet-1'
# 
# # Sample - response probabilities (soft cluster assignment)
# response.probs <- sample2response(res, subnet.id)
# tail(round(response.probs, 6))
# 
# # Sample - response hard assignments
# hard.clusters <- response2sample(res, subnet.id)
# print(hard.clusters)

## ----NetResponse9, warning=FALSE, message=FALSE-------------------------------
# params <- get.model.parameters(res, subnet.id)
# names(params)

## ----vdp, warning=FALSE, message=FALSE----------------------------------------
# # Generate 2-dimensional simulated data with 3 clusters
# res <- generate.toydata(Dim = 2, Nc = 3, Ns = 200, sd0 = 3, rgam.shape = 1, rgam.scale = 1)
# 
# D <- res$data
# real.means <- res$means
# real.sds   <- res$sds
# real.sample2comp     <- res$sample2comp
# 
# # Infinite Gaussian mixture model with
# # Variational Dirichlet Process approximation
# mixt <- vdp.mixt( D )
# 
# # Centroids of the detected Gaussian components
# estimated.means <- mixt$posterior$centroids
# 
# # The colors denote the known clusters
# # The blue ball denotes the original (known) cluster centroids and
# # the triangle denotes the estimated centroids
# plot(D, col = real.sample2comp, pch = 1)
# points(real.means, col = "blue", pch = 16, cex = 2)
# points(estimated.means, col = "blue", pch = 17, cex = 2)
# 
# # Hard mixture component assignment for each sample
# estimated.sample2comp <- apply(mixt$posterior$qOFz, 1, which.max)
# 
# # Compare known and estimated mixture components
# # (note that cluster indices may have switched due to unidentifiability)
# # nearly all samples have one-to-one match between the real and estimated
# # clusters
# head(table(estimated.sample2comp, real.sample2comp))

## ----cite, warning=FALSE, message=FALSE---------------------------------------
# citation("netresponse")

## ----version, warning=FALSE, message=FALSE------------------------------------
# sessionInfo()

######################################################################

# Before I put a sketch on paper, the whole idea is worked out
# mentally. In my mind I change the construction, make improvements,
# and even operate the device. Without ever having drawn a sketch I
# can give the measurements of all parts to workmen, and when
# completed all these parts will fit, just as certainly as though I
# had made the actual drawings. It is immaterial to me whether I run
# my machine in my mind or test it in my shop. The inventions I have
# conceived in this way have always worked. In thirty years there
# has not been a single exception. My first electric motor, the
# vacuum wireless light, my turbine engine and many other devices
# have all been developed in exactly this way.
#
#                                                    - Nicola Tesla

######################################################################

#' @title detect.responses
#' 
#' @description Main function of the NetResponse algorithm. 
#' Detect condition-specific network responses, given
#' network and a set of measurements of node activity in a set of
#' conditions. Returns a set of subnetworks and their estimated
#' context-specific responses.
#'
#' @param datamatrix Matrix of samples x features. For example, gene expression
#'   matrix with conditions on the rows, and genes on the columns. The matrix
#'   contains same features than the 'network' object, characterizing the network
#'   states across the different samples.
#' @param network Binary network describing undirected pairwise interactions between
#'   features of 'datamatrix'. The following formats are supported: binary
#'   matrix, graphNEL, igraph, graphAM, Matrix, dgCMatrix, dgeMatrix
#' @param initial.responses Initial number of components for each subnetwork
#'   model. Used to initialize calculations.
#' @param max.responses Maximum number of responses for each subnetwork. Can be
#'   used to limit the potential number of network states.
#' @param max.subnet.size Numeric. Maximum allowed subnetwork size.
#' @param verbose Logical. Verbose parameter.
#' @param implicit.noise Implicit noise parameter. Add implicit noise to vdp
#'   mixture model. Can help to avoid overfitting to local optima, if this
#'   appears to be a problem.
#' @param update.hyperparams Logical. Indicate whether to update
#'   hyperparameters during modeling.
#' @param prior.alpha,prior.alphaKsi,prior.betaKsi Prior parameters for
#'   Gaussian mixture model that is calculated for each subnetwork
#'   (normal-inverse-Gamma prior). alpha tunes the mean; alphaKsi and betaKsi are
#'   the shape and scale parameters of the inverse Gamma function, respectively.
#' @param vdp.threshold Minimal free energy improvement after which the
#'   variational Gaussian mixture algorithm is deemed converged.
#' @param merging.threshold Minimal cost value improvement required for merging
#' two subnetworks.
#' @param ite Defines maximum number of iterations on posterior update
#' (updatePosterior). Increasing this can potentially lead to more accurate
#' results, but computation may take longer.
#' @param information.criterion Information criterion for model selection.
#' Default is BIC (Bayesian Information Criterion); other options include AIC
#' and AICc.
#' @param speedup Takes advantage of approximations to PCA, mutual information
#' etc in various places to speed up calculations. Particularly useful with
#' large and densely connected networks and/or large sample size.
#' @param speedup.max.edges Used if speedup = TRUE. Applies prefiltering of
#' edges for calculating new joint models between subnetwork pairs when
#' potential cost changes (delta) are updated for a newly merged subnetwork and
#' its neighborghs. Empirical mutual information between each such subnetwork
#' pair is calculated based on their first principal components, and joint
#' models will be calculated only for the top candidates up to the number
#' specified by speedup.max.edges. It is expected that the subnetwork pair that
#' will benefit most from joint modeling will be among the top mutual
#' infomation candidates. This way it is possible to avoid calculating
#' exhaustive many models on the network hubs.
#' @param positive.edges Consider only the edges with positive association. Currently measured with Spearman correlation.
#' @param mc.cores Number of cores to be used in parallelization. See
#' help(mclapply) for details.
#' @param mixture.method Specify the approach to use in mixture modeling.
#' Options. vdp (nonparametric Variational Dirichlet process mixture model);
#' bic (based on Gaussian mixture modeling with EM, using BIC to select the
#' optimal number of components)
#' @param bic.threshold BIC threshold which needs to be exceeded before a new mode is added to the mixture with mixture.method = "bic"
#' @param pca.basis Transform data first onto PCA basis to try to avoid problems with non-diagonal covariances.
#' @param ... Further optional arguments to be passed.
#' @return NetResponseModel object.
#' @author Maintainer: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation("netresponse").
#' @keywords methods iteration
#' @export
#' @examples
#' \dontrun{
#'   #data(toydata)        # Load toy data set
#'   #D    <- toydata$emat   # Response matrix (for example, gene expression)
#'   #netw <- toydata$netw   # Network
#' 
#'   # Run NetReponse algorithm
#'   # model <- detect.responses(D, netw, verbose = FALSE)
#' }
detect.responses <- function(datamatrix,
        network = NULL,
        initial.responses = 1,   # initial number of components. FIXME: is this used?
        max.responses = 10,      
        max.subnet.size = 10,    # max. subnetwork size
        verbose = TRUE,          # print intermediate messages
        prior.alpha    = 1,      # Prior parameters
        prior.alphaKsi = 0.01,   # for VDP mixture
        prior.betaKsi  = 0.01,   # scale parameter for inverse Gamma
        update.hyperparams = 0,  # update hyperparameters. FIXME: check if this is applicable.
        implicit.noise = 0,      # Add implicit noise in vdp.mk.log.lambda.so and vdp.mk.hp.posterior.so 
        vdp.threshold = 1.0e-5,  # min. free energy improvement that stops VDP
        merging.threshold = 0,   # min. cost improvement for merging
        ite = Inf,               # max. iterations in updatePosterior
        information.criterion = "BIC", # information criterion for node merging
        speedup = TRUE,          # speed up calculations by approximations
        speedup.max.edges = 10,  # max. new joint models to be calculated; MI-based prefiltering applied
        positive.edges = FALSE, # If TRUE, consider positive edges only     
        mc.cores = 1, # number of cores for parallelization
        mixture.method = "vdp", # Which approach to use for sample mixture estimation within given subnet. Options: bic/vdp
        bic.threshold = 0,
        pca.basis = FALSE,
        ... # Further arguments
)

    {

        # Check data matrix validity         
        datamatrix <- check.matrix(datamatrix)

        # Check network validity and polish
        tmp <- check.network(network, datamatrix, verbose = verbose)
        network <- tmp$formatted
        network.orig <- tmp$original
        delta <- tmp$delta
        network.nodes <- tmp$nodes
        rm(tmp)

        ### INITIALIZE ###  
        if (verbose) message("matching the features between network and datamatrix")  
        samples <- rownames(datamatrix)
        datamatrix   <- matrix(datamatrix[, network.nodes], nrow(datamatrix))
        colnames(datamatrix) <- network.nodes
        rownames(datamatrix) <- samples
        rm(samples)
                 
        # Store here all params used in the model (defined in function call)
        params <- list(initial.responses = initial.responses, 
        max.responses = max.responses,
        max.subnet.size = max.subnet.size,
        verbose = verbose, 
        prior.alpha = prior.alpha, 
        prior.alphaKsi = prior.alphaKsi,
            prior.betaKsi = prior.betaKsi, 
        update.hyperparams = update.hyperparams, 
        implicit.noise = implicit.noise,
            vdp.threshold = vdp.threshold,
            merging.threshold = merging.threshold,
        ite = ite, 
        information.criterion = information.criterion,
        speedup = speedup,
        speedup.max.edges = speedup.max.edges,
        Nlog = log( nrow( datamatrix ) ),
        nbins = floor(sqrt(nrow(datamatrix))),
        mc.cores = mc.cores,
        mixture.method = mixture.method,
        bic.threshold = bic.threshold,
        positive.edges = positive.edges, 
        pca.basis = pca.basis         
        )

    # Place each node in a singleton subnet
    G <- lapply(seq_len(ncol( datamatrix )), function( x ){ x }) 

    # Filter network
    tmp <- filter.netw(network, delta, datamatrix, params)
    network <- tmp$network      
    delta <- tmp$delta
    # FIXME: for more efficient memory usage, remove from the
    # datamatrix those nodes which are
    # not in the network. But check that the indices are not confused.

    ########################################################################

    ### INDEPENDENT MODEL FOR EACH VARIABLE ###
    tmp <- independent.models(datamatrix, params)
    node.models <- tmp$nodes # model parameters
    C <- sum(tmp$C)

    ### MERGE VARIABLES ###

    # Store agglomeration steps
    move.cost.hist  <- matrix(c(0, 0, C), nrow = 3)

    if (params$max.subnet.size > 1) {

    ### compute costs for combined (singleton) variable pairs  ###
    tmp <- pick.model.pairs(network, network.nodes, node.models, datamatrix, params)

    model.pairs <- tmp$model.pairs
    delta <- tmp$delta

    # if there are groups left sharing a link and improvement (there are
    # connected items that have delta<0) then continue merging
    # note that diag(network) has been set to 0
    while ( !is.null(network) && any( na.omit(-delta) > merging.threshold )){

        if ( verbose ) { message(paste('Combining groups, ', sum(!is.na(G)), ' group(s) left...\n'))} else{}
    
        # Identify the best neighbor pair in the network (also check that
        # the new merged pair would not exceed the max allowed subnetwork
        # size)

        tmp <- find.best.neighbor(G, params$max.subnet.size, network, delta)
        delta <- tmp$delta
        best.edge <- tmp$best.edge

        # If merging is still possible
        if (-tmp$mindelta > merging.threshold) {

            a <- tmp$a 
            b <- tmp$b
        C <- C + tmp$mindelta
        move.cost.hist <- cbind(move.cost.hist, matrix(c(a, b, C), 3))      

        # put the new group to a's place only for those variables for
        # which this is needed.  For others, put Inf on the a neighborgs,
        # combine a and b in the network, remove self-link a-a, 
        # remove b (row and col)
        tmp.join <- join.subnets(network, delta, best.edge)
        network <- tmp.join$network
        delta <- tmp.join$delta
        node.models[[a]] <- model.pairs[[best.edge]]
        node.models[[b]] <- NA
    
        # remove self-links
        keep <- !(network[1,] == network[2,])
        network <- network[, keep]
        delta <- delta[keep]
        model.pairs <- model.pairs[keep]    

        # Merge groups G[[a]], G[[b]]
        G[[a]] <- sort(c(G[[a]], G[[b]]))
        G[[b]] <- NA

        # Skip the first b-1 elements as we only apply lower triangle here
        if ( ncol(network) <= 1 ) {
            if ( verbose ) { message("All nodes have been merged.\n") }
            delta <- Inf # indicates no merging can be be done any more
        } else {
            # Compute new joint models for the new
	    # merged subnet and its neighborghs
            merge.edges <- which(is.na(delta))
                           
            # Remove edges that would exceed max.size
            # FIXME: include as part of cost function?

        if (length(merge.edges) > 0) {

            new.sizes <- apply(matrix(network[, merge.edges], 2), 2, function (x) {length(c(G[[x[[1]]]], G[[x[[2]]]]))})
            merge.edges <- merge.edges[new.sizes <= params$max.subnet.size]

            if (speedup && length(merge.edges) > speedup.max.edges) {

                # To speed up computation, pre-filter the edge set for which
                # new models are calculated.  Calculate empirical associations
                # between the first principal components of each
                # subnetwork pair. If number of new subnetwork pairs exceeds
                # the threshold, then calculate new model only for the
                # subnetwork pairs that have the highest associations.
                # It is expected that the subnetwork pair that will benefit
                # most from joint modeling will also be among the top 
                # candidates. This way we can avoid calculating
                # exhaustive many models on large network hubs at each
                # update.
                gmis <- get.mis(datamatrix, network, delta, network.nodes, G, params)
            merge.edges <- which(is.na(delta))[order(gmis, decreasing = TRUE)]
        
            # Remove edges that would exceed max.size
            new.sizes <- apply(matrix(network[, merge.edges], 2), 2, function (x) {length(c(G[[x[[1]]]], G[[x[[2]]]]))})
            keep <- (new.sizes <= params$max.subnet.size)
            merge.edges <- merge.edges[keep][seq_len(speedup.max.edges)]
        
            # Needs Inf: NAs would be confused with other merges later since
            # models to be calculated are taken from is.na(delta) at each step
                delta[setdiff(which(is.na(delta)), merge.edges)] <- Inf 

            } 

        }
                      
        # TODO: parallelize to speed up
        for (edge in merge.edges) {
            tmp <- update.model.pair(datamatrix, delta, network, edge, network.nodes, G, params, node.models, model.pairs)
            model.pairs <- tmp$model.pairs
            delta <- tmp$delta
        }

        }

    } else {
        if ( verbose ) { message(paste('Merging completed: no groups having links any more, or cost function improvement does not exceed the threshold.')) }
        break
    }
    } 
    }
    
    # Remove left-out nodes (from the merges)
    nainds <- is.na(node.models)
    node.models <- node.models[!nainds]
    G <- G[!nainds]

    # Form a list of subnetworks (no filters)
    # mclapply was slower here  
    subnet.list <- lapply(G, function(x) { network.nodes[unlist(x)] }) 

    # name the subnetworks
    names(node.models) <- names(subnet.list) <- names(G) <- paste("Subnet-", seq_len(length(G)), sep = "")  
    #gc()

    # Convert original network to graphNEL (not before, to save more memory for computation stage)
    network.orig <- igraph.to.graphNEL(graph.data.frame(as.data.frame(t(network.orig)), directed = FALSE, vertices = data.frame(cbind(seq_len(length(network.nodes)), network.nodes))))
    nodes(network.orig) <- network.nodes

    # For one-dimensional subnets, 
    # order the modes by magnitude to simplify interpretation
    for (mi in seq_len(length(node.models))) {
    if (ncol(node.models[[mi]]$mu) == 1 && length(node.models[[mi]]$w) > 1) {
        o <- order(node.models[[mi]]$mu[,1])
        node.models[[mi]]$mu <- matrix(node.models[[mi]]$mu[o,], nrow = length(o))
        node.models[[mi]]$sd <- matrix(node.models[[mi]]$sd[o,], nrow = length(o))
        node.models[[mi]]$w <- node.models[[mi]]$w[o]
        rownames(node.models[[mi]]$mu) <- rownames(node.models[[mi]]$sd) <- names(node.models[[mi]]$w) <- paste("Mode-", seq_len(length(node.models[[mi]]$w)), sep = "")
    }
    } 

    # FIXME: if all nodes will be combined (merging.threshold = -Inf),
    # there will be an error. Fix.
    #  costs: cost function values at each state
    #  moves: indices of groups joined at each state in its columns
    #  groupings: groupings at each level of the hierarchy
    #  models: compressed representations of the models from each step

    model <- new("NetResponseModel",
        moves = matrix(move.cost.hist, 3),
        last.grouping = G,        # network nodes in indices
        subnets = subnet.list,    # network nodes in feature names;
        # FIXME: remove available from models and G
        params = params,
        datamatrix = datamatrix,
        network = network.orig,
        models = node.models
    )
}

#' update.model.pair
#' 
#' Mainly for internal use. Calculate joint model for given node pair and
#' update delta accordingly.
#' 
#' 
#' @usage update.model.pair(datamatrix, delta, network, edge, network.nodes, G,
#' params, model.nodes, model.pairs)
#' @param datamatrix datamatrix
#' @param delta delta
#' @param network network
#' @param edge edge
#' @param network.nodes network.nodes
#' @param G G
#' @param params params
#' @param model.nodes model.nodes
#' @param model.pairs model.pairs
#'
#' Return:
#' @return \item{model.pairs }{model.pairs} \item{delta }{delta}
#' @author Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao.  Maintainer:
#' Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords internal
#' @examples #
update.model.pair <- function(datamatrix, delta, network, edge, network.nodes, G, 
    params, model.nodes, model.pairs) {
    
    # Pick node indices
    a <- network[1, edge]
    b <- network[2, edge]
    vars <- network.nodes[sort(c(G[[a]], G[[b]]))]
    
    dat <- matrix(datamatrix[, vars], nrow(datamatrix))
    
    model.params <- mixture.model(dat, mixture.method = params$mixture.method, max.responses = params$max.responses, 
        implicit.noise = params$implicit.noise, prior.alpha = params$prior.alpha, 
        prior.alphaKsi = params$prior.alphaKsi, prior.betaKsi = params$prior.betaKsi, 
        vdp.threshold = params$vdp.threshold, initial.responses = params$initial.responses, 
        ite = params$ite, params$speedup, bic.threshold = params$bic.threshold, pca.basis = params$pca.basis)
    
    # Negative free energy is (variational) lower bound for P(D|H) Use this to
    # approximate P(D|H)
    if (is.finite(model.params$free.energy)) {
        # Compute COST-value for two independent subnets vs. joint model Negative free
        # energy (-cost) is (variational) lower bound for P(D|H) Use it as an
        # approximation for P(D|H) Cost for the indpendent and joint models -cost is sum
        # of two independent models (cost: appr. log-likelihoods)
        cost.ind <- info.criterion(model.nodes[[a]]$Nparams + model.nodes[[b]]$Nparams, 
            params$Nlog, -(model.nodes[[a]]$free.energy + model.nodes[[b]]$free.energy), 
            criterion = params$information.criterion)
        cost.joint <- info.criterion(model.params$Nparams, params$Nlog, -model.params$free.energy, 
            criterion = params$information.criterion)
        # change (increase) of the total cost
        delta[[edge]] <- cost.joint - cost.ind
    } else {
        warning("No free energy obtained.")
        delta[[edge]] <- Inf
    }
    
    # Store the joint models / cost for two independent vs. joint model
    if (-delta[[edge]] > params$merging.threshold) {
        # Store joint model only if it would improve the cost
        model.pairs[[edge]] <- model.params
    } else {
        model.pairs[[edge]] <- 0
    }
    
    list(model.pairs = model.pairs, delta = delta)
    
}

#' @title List significant responses
#' @description List significantly associated responses for all factors and levels in the given annotation matrix
#' @param annotation.df annotation data.frame with discrete factor levels, rows named by the samples
#' @param models List of models. Each model should have a sample-cluster assignment matrix qofz, or a vector of cluster indices named by the samples.
#' @param method method for enrichment calculation
#' @param min.size minimum sample size for a response
#' @param qth q-value threshold
#' @param verbose verbose 
#' @param data data (samples x features; or a vector in univariate case) 
#' @param rounding rounding digits 
#' @return Table listing all associations between the factor levels and responses
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @importFrom qvalue qvalue
#' @keywords utilities
list.responses.factor <- function(annotation.df, models, method = "hypergeometric", 
    min.size = 2, qth = Inf, verbose = TRUE, data = NULL, rounding = NULL) {
    
    pth <- NULL
    
    # samples x features
    if (is.vector(data)) {
        data2 <- matrix(data)
        rownames(data2) <- names(data)
        data <- data2
    }
    
    if (!is.data.frame(annotation.df)) {
        stop("Provide data.frame for the annotation.df argument!")
    }
    
    # Collect the tables from all factors and levels here
    collected.table <- NULL
    
    for (fnam in colnames(annotation.df)) {
        
        if (verbose) {
            message(fnam)
        }
        
        annotation.vector <- annotation.df[[fnam]]
        names(annotation.vector) <- rownames(annotation.df)
        
        # Order responses for each level of this factor
        responses.per.level <- factor.responses(annotation.vector, models, method = method, 
            min.size = min.size, data = data)
        
        responses.per.level <- responses.per.level[na.omit(names(responses.per.level))]
        
        # Add factor/level information
        mat <- NULL
        if (length(responses.per.level) > 0 && !is.null(na.omit(names(responses.per.level)))) {
            for (level in na.omit(names(responses.per.level))) {
                
                responses.per.level[[level]] <- cbind(responses.per.level[[level]], 
                  fnam, level)
                tmp <- responses.per.level[[level]]
                colnames(responses.per.level[[level]]) <- c(colnames(tmp)[seq_len(ncol(tmp) - 
                  2)], "Factor", "Level")
                
            }
            
            # Combine the level-wise matrices
            mat <- responses.per.level[[1]]
            
            if (length(responses.per.level) > 1) {
                for (i in 2:length(responses.per.level)) {
                  mat <- rbind(mat, responses.per.level[[i]])
                }
            }
            
        }
        
        # FIXME: do.call(rbind, l) tai do.call(cbind, l) is a neater way to implement
        # this
        collected.table <- rbind(collected.table, mat)
        
    }
    
    # Order by pvalue
    collected.table <- collected.table[order(as.numeric(as.character(collected.table[, 
        "pvalue"]))), ]
    collected.table <- data.frame(collected.table)
    collected.table$pvalue <- as.numeric(as.character(collected.table$pvalue))
    
    if (!is.null(collected.table)) {
        
        if (nrow(collected.table) > 100) {
            collected.table$qvalue <- qvalue(collected.table$pvalue, gui = FALSE, 
                fdr.level = 0.25)$qvalue
        } else {
            collected.table$qvalue <- rep(NA, nrow(collected.table))
        }
        
        # Filtering based on qvalues
        if (!all(is.na(collected.table$qvalue))) {
            collected.table <- collected.table[collected.table$qvalue < qth, ]
        } else {
            collected.table <- collected.table[collected.table$pvalue < pth, ]
        }
    }
    
    collected.table$mode <- as.character(collected.table$mode)
    collected.table$annotated.in.subset <- as.numeric(as.character(collected.table$annotated.in.subset))
    collected.table$fraction.in.subset <- as.numeric(as.character(collected.table$fraction.in.subset))
    collected.table$fraction.in.data <- as.numeric(as.character(collected.table$fraction.in.data))
    collected.table$pvalue <- as.numeric(as.character(collected.table$pvalue))
    collected.table$qvalue <- as.numeric(as.character(collected.table$qvalue))
    
    if (!is.null(rounding)) {
        
        collected.table$fraction.in.subset <- round(collected.table$fraction.in.subset, 
            rounding)
        collected.table$fraction.in.data <- round(collected.table$fraction.in.data, 
            rounding)
        collected.table$pvalue <- round(collected.table$pvalue, rounding)
        collected.table$qvalue <- round(collected.table$qvalue, rounding)
        
    }
    
    collected.table
    
}

# Copyright (C) 2010-2016 Leo Lahti Contact: Leo Lahti <leo.lahti@iki.fi> This
# program is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2, or (at your option) any later version.  This
# program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.

#' @title Factor responses
#' @description List responses for each level of the given factor
#' @param annotation.vector annotation vector with discrete factor levels, and named by the samples
#' @param groupings List of groupings. Each model should have a sample-cluster assignment matrix qofz, or a vector of cluster indices named by the samples.
#' @param method method for enrichment calculation
#' @param min.size minimum sample size for a response 
#' @param data data (samples x features; or a vector in univariate case)
#' @return List with each element corresponding to one factor level and listing the responses according to association strength
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @export
#' @keywords utilities
#' @examples res <- factor.responses(annotation.vector = NULL, groupings = NULL)
factor.responses <- function(annotation.vector, groupings, method = "hypergeometric", 
    min.size = 2, data = NULL) {

    if (is.null(groupings)) {return(NULL)}

    responses <- list()
    
    annotation.vector <- factor(annotation.vector)
    
    levels <- as.character(na.omit(unique(droplevels(annotation.vector))))
    
    for (lev in levels) {
        
        level.samples <- names(annotation.vector)[which(annotation.vector == lev)]
        
        ors <- enrichment.list.factor(groupings, level.samples, method = method)
        
        if (is.null(ors)) {
            ors <- NA
            warning(paste("No significant responses for level", lev))
        }
        
        responses[[as.character(lev)]] <- ors
    }
    
    # Pick top responses for each factor level
    responses <- responses[!is.na(responses)]
    
    responses.per.level <- NULL
    if (length(responses) > 0) {
        
        responses.per.level <- lapply(responses, function(dr) {
            dr$ordered.responses
        })
        responses.per.level <- responses.per.level[sapply(responses.per.level, nrow) > 
            0]
        
    }
    
    responses.per.level
    
}

#' @title Add ellipse to an existing plot
#' @description Calculates and plots ellipse corresponding to specified
#' confidence interval in 2-dimensional plot
#' @param centroid Vector with two elements defining the ellipse centroid.
#' @param covmat Covariance matrix for the investigated data. Only diagonal
#' covariances supported.
#' @param confidence Confidence level determining the ellipse borders based on
#' the covariance matrix.
#' @param npoints Number of plotting points.
#' @param col Color.
#' @param ... Other arguments to be passed.
#' @return Used for plotting side effects.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords utilities
add.ellipse <- function(centroid, covmat, confidence = 0.95, npoints = 100, col = "black", ...) {

    # add ellipse to a plot
    el <- ellipse(centroid, covmat, confidence, npoints)
    points(el, type = "l", col = col, ...)

    el

}

#' @title Univariate BIC mixture
#' @description Latent class analysis based on (infinite) Gaussian mixture
#' model. If the input (dat) is data matrix, a multivariate model is fitted. If
#' the input is a vector or a 1-dimensional matrix, a univariate model is
#' fitted.
#' @param x  dat vector (for univariate analysis) or a matrix (for multivariate analysis)
#' @param max.modes Maximum number of modes to be checked for mixture model selection
#' @param bic.threshold BIC threshold which needs to be exceeded before a new mode is added to the mixture.
#' @param min.modes minimum number of modes
#' @param ... Further optional arguments to be passed
#' @return Fitted latent class model (parameters and free energy)
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords utilities
bic.mixture.univariate <- function(x, max.modes, bic.threshold = 0, min.modes = 1, 
    ...) {

    
    # x <- datamatrix[, node]; max.modes = params$max.responses; bic.threshold =
    # params$bic.threshold
    
    best.mode <- bic.select.best.mode(x, max.modes, bic.threshold, min.modes = min.modes)
    mcl <- Mclust(x, G = best.mode)
    
    means <- as.vector(mcl$parameters$mean)
    sds <- as.vector(sqrt(mcl$parameters$variance$sigmasq))
    if (length(sds) == 1) {
        sds <- rep(sds, length(means))
    }
    ws <- as.vector(mcl$parameters$pro)
    
    if (is.null(ws)) {
        warning("NULL weights, replacing with 1")
        ws <- 1
    }
    if (is.null(means)) {
        warning("NULL means, replacing with 1")
        means <- 1
    }
    if (is.null(sds)) {
        warning("NULL sds, replacing with 1")
        sds <- 1
    }
    
    Nparams <- length(means) + length(sds) + length(ws)
    
    means <- matrix(means, nrow = length(ws))
    sds <- matrix(sds, nrow = length(ws))
    
    # Determine the most likely mode for each sample (-> hard clusters) save(means,
    # sds, ws, x, file = '~/tmp/tmp.RData')
    qofz <- P.r.s(matrix(x, nrow = 1), list(mu = means, sd = sds, w = ws), log = FALSE)
    rownames(qofz) <- names(x)
    
    names(means) <- names(sds) <- names(ws) <- paste("Mode", seq_len(length(ws)), 
        sep = "-")
    
    list(means = means, sds = sds, ws = ws, Nparams = Nparams, free.energy = -mcl$loglik, 
        qofz = qofz)
    
}

#' @title Investigate association of a continuous variable and the modes
#' @description Investigate association of a continuous variable and the modes given a list of groupings
#' @param annotation.df annotation data.frame with discrete factor levels, rows
#' named by the samples
#' @param groupings Sample mode information. Each element corresponds to one grouping; each grouping lists samples for the modes within that grouping.
#' @param method method for quantifying the association
#' @param pth p-value threshold applied to adjusted p-values
#' @param verbose verbose 
#' @param rounding rounding digits
#' @return Table listing all associations between the factor levels and responses
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @export
#' @importFrom qvalue qvalue
#' @keywords utilities
#' @examples res <- list.responses.continuous.multi(annotation.df = NULL, groupings = NULL)
list.responses.continuous.multi <- function(annotation.df, groupings, method = "t-test", 
    pth = Inf, verbose = TRUE, rounding = NULL) {

    if (is.null(groupings)) {return(NULL)}

    # annotation.df <- annot[, continuous.vars]; groupings <- groupings.listing; pth
    # = pth; method <- 't-test'; verbose = TRUE; rounding = NULL
    
    tab <- NULL
    for (gn in names(groupings)) {
        
        gntab <- list.responses.continuous.single(annotation.df, groupings[[gn]], 
            method = method, pth = Inf, verbose = verbose, rounding = rounding, adjust.p = FALSE)
        
        gntab <- cbind(grouping = gn, gntab)
        tab <- rbind(tab, gntab)
        
    }
    
    tab$p.adj <- rep(NA, nrow(tab))
    if (nrow(tab) > 100) {
        qv <- qvalue(tab$pvalue, pi0.method = "bootstrap", fdr.level = 0.25)
        if (("qvalues" %in% names(qv))) {
            tab$p.adj <- qv$qvalues
        }
    } else {
        tab$p.adj <- p.adjust(tab$pvalue, method = "BH")
    }
    
    tab$pvalue <- NULL
    tab <- tab[tab$p.adj < pth, ]
    
    tab
    
}

#' @title enrichment.list.factor
#' @description Orders the responses by association strength (enrichment score) to a given sample set. For instance, if the samples correspond to a particular experimental factor, this function can be used to prioritize the responses according to their association strength to this factor.
#' @param groupings List of groupings. Each model should have a sample-cluster assignment matrix qofz.
#' @param method 'hypergeometric' measures enrichment of factor levels in this
#'   response; 'precision' measures response purity for each factor level;
#'   'dependency' measures logarithm of the joint density between response and
#'   factor level vs. their marginal densities: log(P(r,s)/(P(r)P(s)))
#' @param verbose Follow progress by intermediate messages.
#' @param annotation.vector annotation vector
#' @param level level
#' @return A data frame which gives a data
#'   frame of responses ordered by enrichment score for the investigated sample.
#'   The model, response id and enrichment score are shown. The method field
#'   indicates the enrichment calculation method. The sample field lists the
#'   samples et for which the enrichments were calculated. The info field lists
#'   additional information on enrichment statistics.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @importFrom qvalue qvalue
#' @export
#' @examples res <- enrichment.list.factor.minimal(groupings = NULL,
#'     method = NULL,
#'     annotation.vector = NULL,
#'     level = NULL)
#'
enrichment.list.factor.minimal <- function(groupings, method, verbose = FALSE, annotation.vector, 
    level) {

    if (is.null(method)) {return(NULL)}
    
    # groupings, level.samples, method = method
    level.samples <- names(annotation.vector)[annotation.vector %in% level]
    
    if (is.null(names(groupings))) {
        names(groupings) <- seq_len(length(groupings))
    }
    
    # Check enrichment in the selected responses
    enrichment.info <- list()
    cnt <- 0
    for (subnet.id in names(groupings)) {
        
        groupings.list <- listify.groupings(groupings[[subnet.id]])
        
        for (response in names(groupings.list)) {
            
            # In enrichment calculations use only those samples where annotations are
            # available
            total.samples <- intersect(unlist(groupings.list), names(which(!is.na(annotation.vector))))
            response.samples <- intersect(groupings.list[[response]], names(which(!is.na(annotation.vector))))
            
            enr <- response.enrichment(total.samples, response.samples, level.samples, 
                method)
            
            cnt <- cnt + 1
            enrichment.info[[cnt]] <- c(model = subnet.id, mode = response, enrichment.score = enr$score, 
                enr$info)
            
        }
    }
    
    if (verbose) {
        message("Groupings checked.")
    }
    
    if (length(enrichment.info) > 0) {
        
        enrichment.info <- enrichment.info[sapply(enrichment.info, function(ei) {
            length(ei) > 2
        })]
        
        enr <- as.data.frame(t(sapply(enrichment.info, identity)))
        
        if ("pvalue" %in% colnames(enr)) {
            
            if (length(enr$pvalue) > 100) {
                # calculate q-values
                enr$qvalue <- qvalue(as.numeric(as.character(enr$pvalue)))$qvalues
            } else if (length(enr$pvalue) > 10) {
                enr$qvalue <- qvalue(as.numeric(as.character(enr$pvalue)),
                pi0.method = "bootstrap", fdr.level = 0.25)$qvalues
            } else {
                
                warning("Not enough p-values for q-value estimation")
                enr$qvalue <- rep(NA, length(enr$pvalue))
                
            }
        }
        
        if (nrow(enr) == 0) {
            return(NULL)
        }
        
        enr[, 3:ncol(enr)] <- apply(enr[, 3:ncol(enr)], 2, as.numeric)
        
        if ("enrichment.score" %in% names(enr)) {
            
            enr <- enr[order(enr$enrichment.score, decreasing = TRUE), ]
            
            enr[["model"]] <- as.character(enr[["model"]])
            
            tmp <- list(ordered.responses = enr, method = method, sample = level.samples)
            
            return(tmp)
            
        } else {
            return(NULL)
        }
        
    } else {
        return(NULL)
    }
    
}

###################################################################

 #  "An article about computational science in a scientific publication
 #  is not the scholarship itself, it is merely advertising of the
 #  scholarship. The actual scholarship is the complete software
 #  development environment and the complete set of instructions
 #  which generate the figures." 
 #                                          - Jon Claerbout

###################################################################

#' Description: Probabiity of mode given multiple samples (ie. data matrix)
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.rS <- function (dat, pars, log = TRUE) {

  # Probability of a response, given sample (group)
  # P(r|S) = P(S|r)P(r)/P(S) = P(S, r)/(sum_r P(S, r))
  # P(r|S) = P(S|r)P(r)/sum_r(P(S|r)P(r)) = (a*P(S|r)P(r))/(a*sum_r(P(S|r)P(r)))
  # -> log P(r|S) = log((a*P(S|r)P(r))/(a*sum_r(P(S|r)P(r)))) = log(a) + log(P(S|r)P(r)) - log(a) - log(sum_r(P(S|r)P(r)))
  # = log(a*P(S, r)) - log(sum_r(a*P(S, r)))
  # = log(a*psr) - log(sum_r(a*psr)) = log(a) + log(psr) - log(sum_r(exp(log(a) + log(psr))))
  # NOTE: unstable due to overflows in particular when multiple samples are used
  # Log P(r|S) = logP(S, r) - log(sumr(P(S, r)))
  #logp <- psr.log - log(sum(exp(psr.log)))
  # psr <- get.P.rs.joint(sample, model, subnet.id, log = FALSE)  

  # joint density P(S, r) for each component r
  psr.log <- P.rs.joint(dat, pars, log = TRUE)
  
  # density P(r|S), avoiding numerical overflows with log.a trick
  log.a <- 10 - max(psr.log)
  logp <- log.a + psr.log - log(sum(exp(log.a + psr.log)))

  # Compared output in univariate case to the direct calculation:
  #ps <- pars$w * dnorm(4.467782, mean = pars$mu, sd = pars$sd); ps/sum(ps)
  #P.rS(matrix(4.467782), pars, log = FALSE)
  # -> OK

  if (log) {
    logp
  } else {
    exp(logp)
  }
  
}



#' Description: Probabiity of mode given a sample (a data vector)
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'  @param scaling Try to avoid floating errors. To be improved later.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.r.s <- function (dat, pars, log = TRUE, scaling = 0) {

  # FIXME rowmeans(qofz) is constant but not 1
  # P(r|s) for each response r and sample s
  if (length(pars$w) == 1) {
    # If there is only one mode, its likelihood is 1 for all samples
    qofz <- matrix(rep(1, ncol(dat)))
    rownames(qofz) <- colnames(dat)
  } else {
    logp <- P.rs.joint.individual(dat, pars, log = TRUE)
    # Try scaling to avoind floating errors that occur easily with small probs
    qofz <- t(apply(logp, 2, function (logx) {xs <- (scaling + logx); exp(xs)/sum(exp(xs))}))
  }

  if ( log ) { qofz <- log(qofz) }

  matrix(qofz, nrow = ncol(dat))

}


#' Description: Joint probabiity density for mode and sample
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.rs.joint.individual <- function (dat, pars, log = TRUE) {

  # P(r,s) = P(s|r)P(r) = P(r|s)P(s)
  # P(r, s) for all samples and responses. Should hold sum_r P(r, s) = P(s) -> OK
  #  Alternatively: logp.joint <- t(t(prs.log) + ps.log)

  # FIXME: merge with P.rs.joint and/or P.rS to avoid redundancy
  psr.log <- P.s.r(dat, pars, log = TRUE)
  pr.log <- as.vector(log(pars$w))

  logp.joint <- psr.log + pr.log
  colnames(logp.joint) <- colnames(dat)
  
  if (log) {
    logp.joint
  } else {
    exp(logp.joint)
  }
  
}



#' Description: Joint probabiity density for mode and sample group
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.rs.joint <- function (dat, pars, log = TRUE) {

  # Joint (Log) density of a given sample group: P(S, r) = P(S|r)P(r)
  # and each mixture component r.
  
  # First calculate
  # P(S|r) for each response; length of output equals to number of responses
  # i.e. logsum of the individual sample densities
  pSr.log <- P.Sr(dat, pars, log = TRUE) #get.P.Sr(sample, model, subnet.id, log = TRUE)  
  # Density for each mixture component
  pr.log <- log(pars$w)

  logp.joint <- pSr.log + pr.log

  # Alternative:
  # logp.joint <- rowSums(get.P.rs.joint.individual(sample, model, pars, subnet.id, log = TRUE))

  if (log) {
    return(logp.joint)
  } else {
    return(exp(logp.joint))
  }  
  
}





#' Description: Probabiity density for sample group given mode
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.Sr <- function (dat, pars, log = TRUE) {

  # dat is features x samples matrix
  psr <- P.s.r(dat, pars, log = TRUE)

  # Returns responses x samples matrix
  # for each response, calculate logsum over samples
  if (log) {
     rowSums(psr)
  } else {
     exp(rowSums(psr))
  } 
}



#' Description: Probabiity density for sample given mode
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.s.r <- function (dat, pars, log = TRUE) {

  # dat: features x samples matrix

  # Log probability density on each data point for each response
  # P(s|r)
    
  if (!nrow(dat) == ncol(pars$mu)) { stop("Dimensions in dat and pars do not match!") }

  # FIXME: in many cases density needs to be calculated just within a
  # single response, here calculated for all responses. Speedup by
  # having this for given response only.

  if ( is.vector(dat) ) { dat <- as.matrix(dat, nrow = length(dat)) }

  # responses x samples matrix P(s|r)
  psr <- matrix(NA, nrow = length(pars$w), ncol = ncol(dat)) 
  if (!is.null(colnames(dat))) { colnames(psr) <- colnames(dat) }

  for ( response in seq_len(length( pars$w ))) {
    # Given the diagonal covariances, the density is product (log-sum)
    # over the densities for individual features (on each data point)
    psr[response, ] <- colSums(dnorm(dat, mean = as.numeric(pars$mu[response, ]), sd = as.numeric(pars$sd[response, ]), log = TRUE))          
  }

  logp <- psr # responses x samples
  rownames(logp) <- names(pars$w)
  colnames(logp) <- colnames(dat)

  if (log) {
     logp
  } else {
    exp(logp)
  }

}




#' Description: Probabiity density for sample 
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.S <- function (dat, pars, log = TRUE) {

  # psi: individual sample densities P(s) given in _log domain_

  # FIXME: develop this s.t. calculates individual sample densities independently
  
  # Overall probability of sample s, given the Gaussian mixture model
  # P(s) = sum_r P(s, r)
  # ps <- log(sum(get.P.rs.joint(sample, model, subnet.id, log = FALSE)))

  # FIXME: numerically does not hold tightly that P(S) = sumr P(S, r) = sumr P(S|r)P(r)
  # the latter equality holds, P(S) is problematic. Differences are not big for examples
  # I checked, but they are still notable. Check in more detail this one.
  #sum(get.P.rs.joint(s, model, subnet.id, log = FALSE))
  #sum(get.P.Sr(s, model, pars = NULL, subnet.id, log = FALSE) * pars$w)

  ps <- sum(P.s.individual(dat, pars, log = TRUE)) # log sum
  
  if (log) {
    ps
  } else {
    exp(ps)
  }
}




#' Description: Probabiity density for individual sample 
#' Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.
#'
#' Arguments:
#'  @param dat features x samples data matrix for mixture modeling
#'  @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
#'  @param log Logical. Return densities in log domain.
#'
#' Returns:
#'   @return Probability density
#'            
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords internal utilities

P.s.individual <- function (dat, pars, log = TRUE) {

  # FIXME: merge with P.S (?)
  
  # Overall probability of sample s, given the model. 
  # individually for each sample

  # responses x samples
  # for each sample (column), density mass is the sum over joint densities on individual responses
  # P(s) = sum_r P(s, r) = sum_r P(s,r) = sum_r P(s|r)P(r)

  ps <- colSums(P.rs.joint.individual(dat, pars, log = FALSE))

  # two alternatives to calculate P(s)
  # log(colSums(get.P.rs.joint.individual(rsample, model, subnet.id, log = FALSE)))
  # log(sum(get.P.Sr(rsample, model, subnet.id, log = FALSE) * get.P.r(model, subnet.id, log = FALSE)))
  # get.P.s(rsample, model, subnet.id, log = TRUE) 

  # log only after summation:
  if (log) {
    log(ps)
  } else {
    ps
  }
}

#' @title model.stats
#' @description Subnetwork statistics: size and number of distinct responses for each subnet.
#' @param models NetResponse object or list of models
#' @return
#' A 'subnetworks x properties' data frame containing the following elements.
#' \item{subnet.size: }{ Vector of subnetwork sizes. } \item{subnet.responses:
#' }{ Vector giving the number of responses in each subnetwork. }
#' @author Leo Lahti <leo.lahti@@iki.fi>
#' @references Leo Lahti et al.: Global modeling of transcriptional responses
#' in interaction networks. Bioinformatics (2010).  See citation('netresponse')
#' for reference details.
#' @keywords utilities
#' @export
#' @examples
#' # Load a pre-calculated netresponse model obtained with 
#' # model <- detect.responses(toydata$emat, toydata$netw, verbose = FALSE)
#' data(toydata)        
#' # Calculate summary statistics for the model
#' stat <- model.stats(toydata$model)
#' 
#' 
model.stats <- function(models) {
    
    # Check statistics for subnetworks subnetwork size number of responses
    
    if (is(models) == "NetResponseModel") {
        models <- models@models
    }
    
    if (is.null(names(models))) {
        names(models) <- seq_len(length(models))
    }
    
    Ncomps <- c()
    for (subnet.id in names(models)) {
        Ncomps[[subnet.id]] <- ncol(models[[subnet.id]]$qofz)
    }
    
    tab <- cbind(sapply(models, function(x) {
        ncol(x$mu)
    }), Ncomps)
    colnames(tab) <- c("nodes", "responses")
    rownames(tab) <- names(models)
    
    as.data.frame(tab)
    
}

#' Reading network files
#' 
#' Function to read network files.
#' 
#' Read in SIF network file, return R graph object in igraph or graphNEL
#' format.
#' 
#' @aliases read.sif
#' @usage read.sif(sif.file, format = 'graphNEL', directed = FALSE, header =
#' TRUE, sep = '\t', ...)
#' @param sif.file Name of network file in SIF format.
#' @param format Output format: igraph or graphNEL
#' @param directed Logical. Directed/undirected graph. Not used in the current
#' model.
#' @param header Logical. Indicate whether the SIF file has header or not.
#' @param sep Field separator.
#' @param ... Further optional arguments to be passed for file reading.
#' @return R graph object in igraph or graphNEL format.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords utilities
#' @examples
#' 
#' #net <- read.sif('network.sif')
#' 
read.sif <- function(sif.file, format = "graphNEL", directed = FALSE, header = TRUE, 
    sep = "\t", ...) {
    
    net <- read.csv(file = sif.file, sep = sep, colClasses = "character", header = header, 
        ...)
    
    # Assume form: node1 linktype node2 side.info..
    if (ncol(net) > 2) {
        
        # remove NA nodes
        nas <- apply(net, 1, function(x) {
            any(is.na(x[c(1, 3)]))
        })
        if (any(nas)) {
            net <- net[!nas, ]
            warning("NAs removed from network node list, ", sum(nas), " edges removed.")
        }
        
        net <- graph.edgelist(as.matrix(net[, -2]), directed = directed)
        
    } else if (ncol(net) == 2) {
        # assume form: node1 node2
        
        # remove NA nodes
        nas <- apply(net, 1, function(x) {
            any(is.na(x))
        })
        if (any(nas)) {
            net <- net[!nas, ]
            warning("NAs removed from network node list, ", sum(nas), " edges removed.")
        }
        
        net <- graph.edgelist(cbind(net[, 1], net[, 2]), directed = directed)
    }
    
    if (format == "graphNEL") {
        net <- igraph.to.graphNEL(net)
    }
    # if (format == 'igraph') { net <- igraph.from.graphNEL(igraph.to.graphNEL(net))
    # }
    
    net
}

# g <- read.graph(file, format = c('edgelist', 'pajek', 'ncol', 'lgl', 'graphml',
# 'dimacs', 'graphdb', 'gml')) FIXME: check also these; then convert graph to
# graphNEL

# Fixme: finish this later order.samples <- function (subnet.id, model,
# phenodata, which.factor, response, method = 'hypergeometric') { # - for given
# response, order factor levels by association strength (enrichment score) #
# P(s|r) = P(s,r)/P(r) total sample density and/or average sample density (both
# for individuals and groups s/S) # P(s|r)/P(s) = P(s,r)/P(r)P(s) # P(s|r)/P(s)
# -> OK: method = 'dependency' # P(s|r) = P(s,r)/P(r) -> OK, normalized version
# P(s|r)/P(S|r) available: method = 'precision' enr <-
# response.enrichments(subnet.id, model, phenodata, which.factor, response,
# method) # For the given response, return levels of the given factor (decreasing
# ordering by enrichement score) sort(enr, decreasing = TRUE) }


#' @title order.responses
#' @description Orders the responses by association strength (enrichment score) to a given sample set. For instance, if the samples correspond to a particular experimental factor, this function can be used to prioritize the responses according to their association strength to this factor.
#' @param models List of models. Each model should have a sample-cluster assignment matrix qofz.
#' @param sample Measure enrichment of this sample (set) across the observed
#'   responses.
#' @param method 'hypergeometric' measures enrichment of factor levels in this
#'   response; 'precision' measures response purity for each factor level;
#'   'dependency' measures logarithm of the joint density between response and
#'   factor level vs. their marginal densities: log(P(r,s)/(P(r)P(s)))
#' @param min.size,max.size,min.responses Optional parameters to filter the
#'   results based on subnet size and number of responses.
#' @param subnet.ids Specify subnets for which the responses shall be ordered.
#'   By default, use all subnets.
#' @param verbose Follow progress by intermediate messages.
#' @param data data (samples x features; or a vector in univariate case)
#' @return A data frame with elements 'ordered.responses' which gives a data
#'   frame of responses ordered by enrichment score for the investigated sample.
#'   The subnetwork, response id and enrichment score are shown. 
#'   The method field indicates the enrichment calculation method. 
#'   The sample field lists the samples et for which the enrichments 
#'   were calculated. 
#'   The info field lists additional information on enrichment statistics.
#' @note Tools for analyzing end results of the model.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @importFrom qvalue qvalue
#' @export
#' @examples res <- order.responses(models = NULL, sample = NULL)
#' # - for given sample/s (factor level), 
#' #     order responses (across all subnets) by association strength 
#' #     (enrichment score); overrepresentation
#' # order.responses(model, sample, method  = 'hypergeometric') 
order.responses <- function(models, sample, method = "hypergeometric", min.size = 2, 
    max.size = Inf, min.responses = 2, subnet.ids = NULL, verbose = FALSE, data = NULL) {
    
    # Given sample (for instance set of samples associated with a given factor level)
    # order the responses across all subnetworks based on their association strength

    if (is.null(models)) {return(NULL)}

    # For a given sample, calculate enrichment values in each response
    subnets <- responses <- scores <- pvals <- c()
    enrichment.info <- list()
    cnt <- 0
    
    # Get model statistics
    stat <- model.stats(models)
    
    # Filter the results
    sn <- get.subnets(models, get.names = TRUE, min.size, max.size, min.responses)
    stat <- stat[names(sn), ]
    if (is.null(subnet.ids)) {
        subnet.ids <- rownames(stat)
    }
    
    enr <- enrichment.list.factor(models, sample, method)
    
    enr
    
}

#' @title Listing significant responses
#' @description List responses with significant associations to a given sample group.
#' @param model NetResponseModel object.
#' @param sample User-specified samples group for which the enrichments are
#' calculated. For instance, an annotation category.
#' @param qth q-value threshold for enrichments
#' @param method Enrichment method.
#' @return Statistics of the significantly associated responses.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @seealso response.enrichment
#' @references See citation('netresponse')
#' @keywords utilities
#' @examples # 
list.significant.responses <- function(model, sample, qth = 1, method = "hypergeometric") {
    
    # Order responses according to their association with the given sample group
    o <- order.responses(model, sample = sample, method = "hypergeometric")$ordered.responses
    o[which(o$qvalue < qth), ]
    
}

enrichment.list <- function(groupings, annotation.vector) {
    
    annotated.samples <- names(which(!is.na(annotation.vector)))
    annotation.data <- annotation.vector[annotated.samples]
    names(annotation.data) <- annotated.samples
    
    associations <- NULL
    for (sn in names(groupings)) {
        
        mode.samples <- groupings[[sn]]
        
        # annotated samples in the mode
        s <- intersect(mode.samples, annotated.samples)
        
        # annotated samples in other modes
        sc <- intersect(unlist(groupings[setdiff(names(groupings), sn)]), annotated.samples)
        
        if (length(na.omit(s)) > 1 && length(na.omit(sc)) > 1) {
            pvals <- t.test(annotation.data[s], annotation.data[sc])$p.value
            fold.change <- mean(annotation.data[s]) - mean(annotation.data[sc])
        } else {
            warning(paste("Not enough annotated observations for response", sn))
            pvals <- NA
            fold.change <- NA
        }
        
        associations <- rbind(associations, cbind(mode = sn, pvalue = pvals, fold.change = fold.change))
        
    }
    
    associations <- data.frame(list(mode = as.character(associations[, "mode"]), 
        pvalue = as.numeric(associations[, "pvalue"]), fold.change = as.numeric(associations[, 
            "fold.change"])), stringsAsFactors = FALSE)
    
    
    associations
    
}

# system('~/local/R/R-2.13.0/bin/R CMD SHLIB ../src/netresponse.c');
# dyn.load('../src/netresponse.so')

#' vdp.mixt
#' 
#' Accelerated variational Dirichlet process Gaussian mixture.
#' 
#' Implementation of the Accelerated variational Dirichlet process Gaussian
#' mixture model algorithm by Kenichi Kurihara et al., 2007.
#'
#' ALGORITHM SUMMARY 
#' This code implements Gaussian mixture models with diagonal covariance matrices. 
#' The following greedy iterative approach is taken in order to obtain the number
#' of mixture models and their corresponding parameters:
#'
#' 1. Start from one cluster, $T = 1$.
#' 2. Select a number of candidate clusters according to their values of 
#'    'Nc' = \\sum_{n=1}^N q_{z_n} (z_n = c) (larger is better).
#' 3. For each of the candidate clusters, c: 
#'     3a. Split c into two clusters, c1 and c2, through the bisector of its 
#'         principal component. Initialise the responsibilities 
#'         q_{z_n}(z_n = c_1) and q_{z_n}(z_n = c_2). 
#'     3b. Update only the parameters of c1 and c2 using the observations that
#'         belonged to c, and determine the new value for the free energy, F{T+1}.
#'     3c. Reassign cluster labels so that cluster 1 corresponds to the largest 
#'         cluster, cluster 2 to the second largest, and so on.
#' 4. Select the split that lead to the maximal reduction of free energy, F{T+1}.
#' 5. Update the posterior using the newly split data.
#' 6. If FT - F{T+1} < \\epsilon then halt, else set T := T +1 and go to step 2.
#'
#' The loop is implemented in the function greedy(...)
#'
#' @param dat Data matrix (samples x features).
#' @param prior.alpha,prior.alphaKsi,prior.betaKsi Prior parameters for
#' Gaussian mixture model (normal-inverse-Gamma prior). alpha tunes the mean;
#' alphaKsi and betaKsi are the shape and scale parameters of the inverse Gamma
#' function, respectively.
#' @param do.sort When true, qOFz will be sorted in decreasing fashion by
#' component size, based on colSums(qOFz). The qOFz matrix describes the
#' sample-component assigments in the mixture model.
#' @param threshold Defines the minimal free energy improvement that stops the
#' algorithm: used to define convergence limit.
#' @param initial.K Initial number of mixture components.
#' @param ite Defines maximum number of iterations on posterior update
#' (updatePosterior). Increasing this can potentially lead to more accurate
#' results, but computation may take longer.
#' @param implicit.noise Adds implicit noise; used by vdp.mk.log.lambda.so and
#' vdp.mk.hp.posterior.so. By adding noise (positive values), one can avoid
#' overfitting to local optima in some cases, if this happens to be a problem.
#' @param c.max Maximum number of candidates to consider in
#' find.best.splitting. During mixture model calculations new mixture
#' components can be created until this upper limit has been reached. Defines
#' the level of truncation for a truncated stick-breaking process.
#' @param speedup When learning the number of components, each component is
#' splitted based on its first PCA component. To speed up, approximate by using
#' only subset of data to calculate PCA.
#' @param min.size Minimum size for a component required for potential
#'   splitting during mixture estimation.
#'
#' @return \item{ prior }{Prior parameters of the vdp-gm model (qofz: priors on observation lables; Mu: centroids; S2: variance).} 
#'         \item{ posterior }{Posterior estimates for the model parameters and statistics.} 
#'         \item{ weights }{Mixture proportions, or weights, for the Gaussian mixture components.}
#'  \item{ centroids }{Centroids of the mixture components.} 
#'  \item{ sds }{ Standard deviations for the mixture model components (posterior modes of the covariance diagonals square root). Calculated as sqrt(invgam.scale/(invgam.shape + 1)). } 
#'  \item{ qOFz }{ Sample-to-cluster assigments (soft probabilistic associations).} 
#' \item{ Nc }{Component sizes}
#'  \item{ invgam.shape }{ Shape parameter (alpha) of the inverse Gamma distribution } 
#'  \item{ invgam.scale }{ Scale parameter (beta) of the inverse Gamma distribution } 
#'  \item{ Nparams }{ Number of model parameters }
#' \item{ K }{ Number of components in the mixture model } 
#'  \item{ opts }{Model parameters that were used.} 
#'  \item{ free.energy }{Free energy of the model.}
#'
#' @note This implementation is based on the Variational Dirichlet Process
#'   Gaussian Mixture Model implementation, Copyright (C) 2007 Kenichi Kurihara
#'   (all rights reserved) and the Agglomerative Independent Variable Group
#'   Analysis package (in Matlab): Copyright (C) 2001-2007 Esa Alhoniemi, Antti
#'   Honkela, Krista Lagus, Jeremias Seppa, Harri Valpola, and Paul Wagner.
#' @author Maintainer: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references Kenichi Kurihara, Max Welling and Nikos Vlassis: Accelerated
#'   Variational Dirichlet Process Mixtures.  In B. Sch\'olkopf and J. Platt and
#'   T. Hoffman (eds.), Advances in Neural Information Processing Systems 19,
#'   761--768. MIT Press, Cambridge, MA 2007.
#' @keywords methods iteration
#' @export
#' @examples
#' 
#'   set.seed(123)
#' 
#'   # Generate toy data with two Gaussian components
#'   dat <- rbind(array(rnorm(400), dim = c(200,2)) + 5,
#'                array(rnorm(400), dim = c(200,2)))
#' 
#'   # Infinite Gaussian mixture model with 
#'   # Variational Dirichlet Process approximation
#'   mixt <- vdp.mixt( dat )
#' 
#'   # Centroids of the detected Gaussian components
#'   mixt$posterior$centroids
#' 
#'   # Hard mixture component assignments for the samples
#'   apply(mixt$posterior$qOFz, 1, which.max)
#' 
#' 
vdp.mixt <- function(dat, prior.alpha = 1, prior.alphaKsi = 0.01, prior.betaKsi = 0.01, 
    do.sort = TRUE, threshold = 1e-05, initial.K = 1, ite = Inf, implicit.noise = 0, 
    c.max = 10, speedup = TRUE, min.size = 5) {
    
    # qOFz sorted in decreasing fashion based on colSums(qOFz) threshold: minimal
    # free energy improvement that stops the algorithm initial.K initial number of
    # components ite # used on updatePosterior: maximum number of iterations
    # implicit.noise # Adds implicit noise in vdp.mk.log.lambda.so and
    # vdp.mk.hp.posterior.so
    
    # c.max max. candidates to consider in Candidates are chosen based on their Nc
    # value (larger = better). Nc = colSums(qOFz) find.best.splitting. i.e.
    # truncation parameter speedup: during DP, components are splitted based on their
    # first PCA component. To speed up, approximate by using only subset data to
    # calculate PCA.  min.size # min size for a component to be splitted
    
    # Prior parameters
    opts <- list(prior.alpha = prior.alpha, prior.alphaKsi = prior.alphaKsi, prior.betaKsi = prior.betaKsi, 
        speedup = speedup, do.sort = do.sort, threshold = threshold, initial.K = initial.K, 
        ite = ite, implicitnoisevar = implicit.noise, c.max = c.max)
    
    # alphaKsi smaller -> less clusters (and big!) -> quite sensitive betaKsi larger
    # -> less clusters (and big!)
    
    data <- list()
    data[["given.data"]] <- list()
    data[["given.data"]][["X1"]] <- dat
    
    # sample-component assignments
    qOFz <- rand.qOFz(nrow(dat), initial.K)
    
    # The hyperparameters of priors
    hp.prior <- mk.hp.prior(data, opts)
    
    # Posterior
    hp.posterior <- mk.hp.posterior(data, qOFz, hp.prior, opts)
    
    # Note: greedy gives components in decreasing order by size
    templist <- greedy(data, hp.posterior, hp.prior, opts, min.size)
    templist$hp.prior <- c(templist$hp.prior, list(qOFz = qOFz))
    qOFz <- matrix(templist$hp.posterior$qOFz, nrow(dat))
    
    ############################################### 
    
    # Retrieve model parameters
    
    # number of mixture components (nonempty components only!)  response must have
    # non-negligible' probability mass!  i.e. at least some points associated with it
    Kreal <- max(apply(qOFz, 1, which.max))  #sum(colSums(qOFz) > 1e-3)
    qOFz <- matrix(qOFz[, seq_len(Kreal)], nrow(dat))
    rownames(qOFz) <- rownames(dat)
    
    # Calculate mixture model parameters FIXME: move this outside from this vdp.mixt
    # function
    
    # Negative free energy is (variational) lower bound for P(D|H) Use it to
    # approximate P(D|HClist <- list(C))
    
    # number of parameters (d-dim. centroid + diag. cov. matrix + component weight
    # for each Kreal)
    Nparams <- Kreal * (2 * ncol(dat) + 1)
    
    # Calculate map estimates of model parameters from the posterior
    
    # variances are assumed inverse Gamma distributed and here beta/alpha gives the
    # expectation)
    
    # Parameters of the inverse Gamma function for component variances
    invgam.shape <- matrix(templist$hp.posterior$KsiAlpha[seq_len(Kreal), ], Kreal)
    invgam.scale <- matrix(templist$hp.posterior$KsiBeta[seq_len(Kreal), ], Kreal)
    
    # Calculate variances (mean and mode of the invgam distr.) from scale and shape
    # FIXME: beta/alpha used in C code var.update <-
    # matrix(invgam.scale/invgam.shape, Kreal) var.mean <-
    # matrix(invgam.scale/(invgam.shape - 1), Kreal)
    var.mode <- matrix(invgam.scale/(invgam.shape + 1), Kreal)
    variances <- var.mode  # select mean, mode, or their average for update
    
    # Ignore empty components assuming that the components have been ordered in
    # decreasing order by size component centroids
    centroids <- matrix(templist$hp.posterior$Mubar[seq_len(Kreal), ], Kreal)
    
    ############################################# 
    
    # Estimate weights directly from mass on each component; with large sample size
    # the solution should converge there
    w <- colSums(qOFz)
    w <- w/sum(w)  # normalize
    # FIXME: improve later, or retrieve weight directly from vdp code or the
    # densities to make more robust also for small sample size NOTE: the way we
    # calculate weights here is not in exact agreement with the real weights in the
    # model that would give qOFz but it is an approximation, and needed for end
    # analysis
    
    posterior <- list(weights = w, centroids = centroids, sds = sqrt(variances), 
        qOFz = qOFz, Nc = colSums(qOFz), invgam.shape = invgam.shape, invgam.scale = invgam.scale, 
        Nparams = Nparams, K = Kreal)
    
    # centroids: mubar sds alpha, beta Nc: # component sizes invgam shape # KsiAlpha
    # invgam.scale # KsiBeta Nparams = Nparams, # number of model parameters K =
    # Kreal # number of components
    
    # Later: include these from hp.posterior to the output later if needed.
    # 'Mutilde[1:Kreal,]' 'gamma[,1:Kreal]' 'Uhat'
    
    list(prior = templist$hp.prior, posterior = posterior, opts = opts, free.energy = templist$free.energy)
}

#' @title enrichment.list.factor
#' @description Orders the responses by association strength (enrichment score) to a given sample set. For instance, if the samples correspond to a particular experimental factor, this function can be used to prioritize the responses according to their association strength to this factor.
#' @param models List of models. Each model should have a sample-cluster assignment matrix qofz.
#' @param level.samples Measure enrichment of this sample (set) across the observed responses.
#' @param method 'hypergeometric' measures enrichment of factor levels in this
#'   response; 'precision' measures response purity for each factor level;
#'   'dependency' measures logarithm of the joint density between response and
#'   factor level vs. their marginal densities: log(P(r,s)/(P(r)P(s)))
#' @param verbose Follow progress by intermediate messages.
#' @return A data frame which gives a data
#'   frame of responses ordered by enrichment score for the investigated sample.
#'   The model, response id and enrichment score are shown. The method field
#'   indicates the enrichment calculation method. The sample field lists the
#'   samples et for which the enrichments were calculated. The info field lists
#'   additional information on enrichment statistics.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @importFrom qvalue qvalue
#' @examples #
enrichment.list.factor <- function(models, level.samples, method, verbose = FALSE) {
    
    # models; level.samples <- level.samples; method = method
    if (is.null(names(models))) {
        names(models) <- seq_len(length(models))
    }
    
    # Get model statistics
    stat <- model.stats(models)
    
    # Check enrichment in the selected responses
    enrichment.info <- list()
    for (subnet.id in names(models)) {
        
        if (verbose) {
            message(subnet.id)
        }
        
        qofz <- models[[subnet.id]]$qofz
        
        for (response in seq_len(ncol(qofz))) {
            
            enr <- response.enrichment(qofz, level.samples, response, method)
            
            # add further info about enrichments
            cnt <- cnt + 1
            
            enrichment.info[[cnt]] <- c(model = subnet.id, mode = response, enrichment.score = enr$score, 
                enr$info)
            
        }
    }
    
    if (verbose) {
        message("Models checked.")
    }
    
    if (length(enrichment.info) > 0) {
        
        enrichment.info <- enrichment.info[sapply(enrichment.info, function(ei) {
            length(ei) > 2
        })]
        
        enr <- as.data.frame(t(sapply(enrichment.info, identity)))
        
        if ("pvalue" %in% colnames(enr)) {
            
            if (length(enr$pvalue) > 100) {
                # calculate q-values
                enr$qvalue <- qvalue(as.numeric(as.character(enr$pvalue)))$qvalues
            } else if (length(enr$pvalue) > 10) {
                enr$qvalue <- qvalue(as.numeric(as.character(enr$pvalue)),
                pi0.method = "bootstrap", fdr.level = 0.25)$qvalues
            } else {
                
                warning("Not enough p-values for q-value estimation")
                enr$qvalue <- rep(NA, length(enr$pvalue))
                
            }
        }
        
        enr[, 3:ncol(enr)] <- apply(enr[, 3:ncol(enr)], 2, as.numeric)
        
        if ("enrichment.score" %in% names(enr)) {
            
            enr <- enr[order(enr$enrichment.score, decreasing = TRUE), ]
            
            enr[["model"]] <- as.character(enr[["model"]])
            
            # Add subnet info in the result table
            enr <- cbind(enr, stat[enr$model, ])
            
            tmp <- list(ordered.responses = enr, method = method, sample = level.samples)
            
            return(tmp)
            
        } else {
            return(NULL)
        }
        
    } else {
        return(NULL)
    }
    
}

# Save the other half of yourselves and your lives for pleasure and
# adventure. It is not enough to fight for natural land and the west;
# it is even more important to enjoy it. While you can. While it's
# still there... Enjoy yourselves, keep your brain in your head and
# your head firmly attached to the body, the body active and alive,
# and I promise you this much: I promise you this one sweet victory
# over our enemies, over those desk-bound men with their hearts in a
# safe deposit box, and their eyes hypnotized by desk calculators. I
# promise you this: you will outlive the bastards. --Ed Abbey.

#' @title Mixture model
#' @description Fit Gaussian mixture model
#' @param x data matrix (samples x features, for multivariate analysis) or a vector (for univariate analysis)
#' @param mixture.method Specify the approach to use in mixture modeling. Options. vdp (nonparametric Variational Dirichlet process mixture model); bic (based on Gaussian mixture modeling with EM, using BIC to select the optimal number of components)
#' @param max.responses Maximum number of responses for each subnetwork. Can be used to limit the potential number of network states.
#' @param implicit.noise Implicit noise parameter. Add implicit noise to vdp
#'        mixture model. Can help to avoid overfitting to local optima, if this
#'        appears to be a problem.
#' @param prior.alpha,prior.alphaKsi,prior.betaKsi Prior parameters for
#'        Gaussian mixture model that is calculated for each subnetwork
#'        (normal-inverse-Gamma prior). alpha tunes the mean; alphaKsi and betaKsi are
#'        the shape and scale parameters of the inverse Gamma function, respectively.
#' @param vdp.threshold Minimal free energy improvement after which the
#'        variational Gaussian mixture algorithm is deemed converged.
#' @param initial.responses Initial number of components for each subnetwork
#'         model. Used to initialize calculations.
#' @param ite Maximum number of iterations on posterior update
#'        (updatePosterior). Increasing this can potentially lead to more accurate results, but computation may take longer.
#' @param speedup Takes advantage of approximations to PCA, mutual information
#'         etc in various places to speed up calculations. Particularly useful with
#'        large and densely connected networks and/or large sample size.
#' @param bic.threshold BIC threshold which needs to be exceeded before a new mode is added to the mixture with mixture.method = "bic"
#' @param pca.basis pca.basis
#' @param min.responses minimum number of responses
#' @param ... Further optional arguments to be passed.
#' @return List with two elements: model: fitted mixture model (parameters and free energy); model.params: model parameters
#' @export
#' @references See citation("netresponse") 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords utilities
#' @examples res <- mixture.model(NULL)
mixture.model <- function (x, mixture.method = "vdp", max.responses = 10, implicit.noise = 0, prior.alpha = 1, prior.alphaKsi = 0.01, prior.betaKsi = 0.01, vdp.threshold = 1.0e-5, initial.responses = 1, ite = Inf, speedup = TRUE, bic.threshold = 0, pca.basis = FALSE, min.responses = 1, ...) {

    if (is.null(x)) {return(NULL)}

    # Present data in PCA space to cope with diagonality of the covariances
    if (pca.basis) {    
        if (nrow(x) > ncol(x)) {
          x <- princomp(x)$scores
        } else {
          # FIXME: implement sparse PCA here to gain more generality?
          warning("Less samples than features, not applying PCA basis")
          x <- x
        }
    }

    if ( is.vector(x) ) {
        xm <- matrix(x, nrow = length(x)); rownames(xm) <- names(x); x <- xm
    }

    if (mixture.method == "vdp") {

        model <- vdp.mixt(x,
                    implicit.noise = implicit.noise,
                prior.alpha = prior.alpha,
                    prior.alphaKsi = prior.alphaKsi,
                prior.betaKsi = prior.betaKsi,
                    threshold = vdp.threshold,
                # FIXME: move initial.K into initial.responses for clarity
            initial.K = initial.responses, 
            ite = ite,
            c.max = max.responses,
            speedup = speedup)

    model.params <- pick.model.parameters(model, colnames(x))      

  } else if (mixture.method == "bic") {     

    model <- bic.mixture(x, max.modes = max.responses,
    bic.threshold = bic.threshold, min.modes = min.responses)  

    mu <- matrix(model$means, nrow = length(model$ws))
    sd <- matrix(model$sds, nrow = length(model$ws))
    ws <- matrix(model$ws)
    qofz <- matrix(model$qofz, ncol = length(ws))
    bic <- model$bic

    rownames(qofz) <- rownames(x)

    rownames(mu) <- rownames(sd) <- names(ws)
    colnames(mu) <- colnames(sd) <- colnames(x)        
   
    model.params <- list(mu = mu,
                    sd = sd,
                    w = ws, 
                    qofz = qofz,
                    free.energy = model$free.energy, 
                    Nparams = model$Nparams, 
                    bic = bic)

    } else {
        stop("Provide proper mixture.method argument.")
    }    

    # FIXME: perhaps the 'model' is not needed any
    # more when model.params is given? Check and remove to save space.
    # How about pca.basis = TRUE case?
    model.params

}

#' @title Dna damage data set (PPI and expression) 
#' @description A combined yeast data set with protein-protein interactions and gene expression (dna damage). Gene expression profiles are transformed into links by computing a Pearson correlation for all pairs of genes and treating all correlations above 0.85 as additional links. Number of genes: 1823, number of interactions: 12382, number of gene expression observations: 52, number of total links with PPI and expression links: 15547.
#' @name dna
#' @docType data
#' @usage data(dna)
#' @format List of following objects: \describe{ \item{ppi}{PPI data matrix}
#' \item{exp}{gene expression profiles data matrix} \item{gids}{Vector of gene
#' ids corresponding to indices used in data matrices} \item{obs}{Gene
#' expression observation details} \item{combined.links}{pooled matrix of PPI
#' and expression links} }
#' @references Ulitsky, I. and Shamir, R. \emph{Identification of functional
#' modules using network topology and high-throughput data.} BMC Systems
#' Biology 2007, 1:8.
#' 
#' Nariai, N., Kolaczyk, E. D. and Kasif, S. \emph{Probabilistic Protein
#' Function Predition from Heterogenous Genome-Wide Data}. PLoS ONE 2007,
#' 2(3):e337.
#' 
#' Gasch, A., Huang, M., Metzner, S., Botstein, D. and Elledge, S.
#' \emph{Genomic expression responses to DNA-damaging agents and the regulatory
#' role of the yeast ATR homolog Mex1p.} Molecular Biology of the Cell 2001,
#' 12:2987-3003.
#' @source PPI data pooled from yeast data sets of [1] and [2]. Dna damage
#' expression set of [3].
#' @keywords datasets
#' @examples data(dna)
NULL


#' @title Class 'NetResponseModel'
#' @description A NetResponse model.
#' @name NetResponseModel-class
#' @aliases NetResponseModel-class [[,NetResponseModel-method show,NetResponseModel-method 
#' @docType class
#' @section Objects from the Class: Returned by \code{\link{detect.responses}}
#' function.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords classes
#' @examples showClass('NetResponseModel')
NULL


#' @title NetResponse: Global modeling of transcriptional responses in interaction networks
#' @description Global modeling of transcriptional responses in interaction networks.
#' \tabular{ll}{ Package: \tab netresponse\cr Type: \tab Package\cr Version:
#' \tab See sessionInfo() or DESCRIPTION file\cr Date: \tab 2011-02-03\cr
#' License: \tab GNU GPL >=2\cr LazyLoad: \tab yes\cr }
#' @name netresponse-package
#' @aliases netresponse-package netresponse
#' @docType package
#' @useDynLib netresponse
#' @author Leo Lahti, Olli-Pekka Huovilainen, Antonio Gusmao and Juuso
#' Parkkinen. Maintainer: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references Leo Lahti et al.: Global modeling of transcriptional responses
#' in interaction networks. Bioinformatics (2010). See citation('netresponse')
#' for details.
#' @keywords package
#' @examples
#' \dontrun{
#' # Define parameters for toy data
#' Ns <- 200  # number of samples (conditions)
#' Nf <- 10   # number of features (nodes)
#' feature.names <- paste('feat', seq(Nf), sep='')
#' sample.names  <- paste('sample', seq(Ns), sep='') 
#' # random seed
#' set.seed( 123 )
#' # Random network
#' netw <- pmax(array(sign(rnorm(Nf^2)), dim = c(Nf, Nf)), 0)
#' # in pathway analysis nodes correspond to genes
#' rownames(netw) <- colnames(netw) <- feature.names
#' # Random responses of the nodes across conditions 
#' D <- array(rnorm(Ns*Nf), dim = c(Ns,Nf), dimnames = list(sample.names, feature.names))
#' D[1:100, 4:6]  <- t(sapply(1:(Ns/2),function(x){rnorm(3, mean = 1:3)}))
#' D[101:Ns, 4:6] <- t(sapply(1:(Ns/2),function(x){rnorm(3, mean = 7:9)}))
#' # Calculate the model
#' #model <- detect.responses(D, netw)
#' ## Subnets (each is a list of nodes)
#' #get.subnets( model )
#' }
NULL

#' @title Osmoshock data set (PPI and expression)
#' @description A combined yeast data set with protein-protein interactions and gene expression (osmotick shock response). Gene expression profiles are transformed into links by computing a Pearson correlation for all pairs of genes and treating all correlations above 0.85 as additional links. Number of genes: 1711, number of interactions: 10250, number of gene expression observations: 133, number of total links with PPI and expression links: 14256.
#' @name osmo
#' @docType data
#' @usage data(osmo)
#' @format List of following objects: \describe{ \item{ppi}{PPI data matrix}
#' \item{exp}{gene expression profiles data matrix} \item{gids}{Vector of gene
#' ids corresponding to indices used in data matrices} \item{obs}{Gene
#' expression observation details} \item{combined.links}{pooled matrix of PPI
#' and expression links} }
#' @references Ulitsky, I. and Shamir, R. \emph{Identification of functional
#' modules using network topology and high-throughput data.} BMC Systems
#' Biology 2007, 1:8.
#' 
#' Nariai, N., Kolaczyk, E. D. and Kasif, S. \emph{Probabilistic Protein
#' Function Predition from Heterogenous Genome-Wide Data}. PLoS ONE 2007,
#' 2(3):e337.
#' 
#' O'Rourke, S. and Herskowitz, I. \emph{Unique and redundant roles for Hog
#' MAPK pathway components as revealed by whole-genome expression analysis.}
#' Molecular Biology of the Cell 2004, 15:532-42.
#' @source PPI data pooled from yeast data sets of [1] and [2]. Dna damage
#' expression set of [3].
#' @keywords datasets
#' @examples data(osmo)
NULL



#' @title toydata
#' @description Toy data for NetResponse examples.
#' @name toydata
#' @docType data
#' @usage data(toydata)
#' @format
#' Toy data: a list with three elements:
#' 
#' emat: Data matrix (samples x features). This contains the same features that
#' are provided in the network (toydata$netw). The matrix characterizes
#' measurements of network states across different conditions.
#' 
#' netw: Binary matrix that describes pairwise interactions between features.
#' This defines an undirected network over the features. A link between two
#' nodes is denoted by 1.
#' 
#' model: A pre-calculated model. Object of NetResponseModel class, resulting
#' from applying the netresponse algorithm on the toydata with model <-
#' detect.responses(D, netw).
#' @references Leo Lahti et al.: Global modeling of transcriptional responses
#' in interaction networks. Bioinformatics (2010).
#' @keywords misc
#' @examples
#'   data(toydata)
#'   D    <- toydata$emat   # Response matrix (samples x features)
#'   netw <- toydata$netw   # Network between the features
#'   model <- toydata$model # Pre-calculated NetResponseModel obtained with
#'                          # model <- detect.responses(D, netw)
NULL

# Validate mixture models

# Generate random data from five Gaussians. 
# Detect modes 
# Plot data points and detected clusters 

library(netresponse)

#fs <- list.files("~/Rpackages/netresponse/netresponse/R/", full.names = TRUE); for (f in fs) {source(f)}; dyn.load("/home/tuli/Rpackages/netresponse/netresponse/src/netresponse.so")

#########  Generate DATA #######################

res <- generate.toydata()
D <- res$data
component.means <- res$means
component.sds   <- res$sds
sample2comp     <- res$sample2comp

######################################################################

par(mfrow = c(2,1))

for (mm in c("vdp", "bic")) {

  # Fit nonparametric Gaussian mixture model
  #source("~/Rpackages/netresponse/netresponse/R/vdp.mixt.R")
  out <- mixture.model(D, mixture.method = mm, max.responses = 10, pca.basis = FALSE)

  ############################################################

  # Compare input data and results

  ord.out <- order(out$mu[,1])
  ord.in <- order(component.means[,1])

  means.out <- out$mu[ord.out,]
  means.in <- component.means[ord.in,]

  # Cluster stds and variances
  sds.out <- out$sd[ord.out,]
  vars.out <- sds.out^2

  sds.in  <- component.sds[ord.in,]
  vars.in <- sds.in^2

  # Check correspondence between input and output
  if (length(means.in) == length(means.out)) {
    cm <- cor(as.vector(means.in), as.vector(means.out))
    csd <- cor(as.vector(sds.in), as.vector(sds.out))
  }

  # Plot results (assuming 2D)
  ran <- range(c(as.vector(means.in - 2*vars.in), 
               as.vector(means.in + 2*vars.in), 
	       as.vector(means.out + 2*vars.out), 
	       as.vector(means.out - 2*vars.out)))

  real.modes <- sample2comp
  obs.modes <- apply(out$qofz, 1, which.max)

  # plot(D, pch = 20, main = paste(mm, "/ cor.means:", round(cm,6), "/ Cor.sds:", round(csd,6)), xlim = ran, ylim = ran)
  
  # plot(D, pch = real.modes, col = obs.modes, main = paste(mm, "/ cor.means:", round(cm,6), "/ Cor.sds:", round(csd,6)), xlim = ran, ylim = ran)
  
  # for (ci in 1:nrow(means.out))  { add.ellipse(centroid = means.out[ci,], covmat = diag(vars.out[ci,]), col = "red") }
  # for (ci in 1:nrow(means.in))  { add.ellipse(centroid = means.in[ci,], covmat = diag(vars.in[ci,]), col = "blue") }

}

library(netresponse)

# Three MODES

# set.seed(34884)
set.seed(3488400)

Ns <- 200
Nd <- 2

D3 <- rbind(matrix(rnorm(Ns*Nd, mean = 0), ncol = Nd), 
      	    matrix(rnorm(Ns*Nd, mean = 3), ncol = Nd),
      	    cbind(rnorm(Ns, mean = -3), rnorm(Ns, mean = 3))
	    )

#X11()
par(mfrow = c(2,2))
for (mm in c("vdp", "bic")) {
  for (pp in c(FALSE, TRUE)) {

    # Fit nonparametric Gaussian mixture model
    out <- mixture.model(D3, mixture.method = mm, pca.basis = pp)
    plot(D3, col = apply(out$qofz, 1, which.max), main = paste(mm, "/ pca:",  pp)) 

  }
}

# VDP is less sensitive than BIC in detecting Gaussian modes (more
# separation between the clusters needed)

# pca.basis option is less important for sensitive detection but
# it will help to avoid overfitting to unimodal features that
# are not parallel to the axes (unimodal distribution often becomes
# splitted in two or more clusters in these cases)

#' @title get.mis
#' @description Estimate mutual information for node pairs based on
#' the first principal components.
#' @usage get.mis(datamatrix, network, delta, network.nodes, G, params)
#' @param datamatrix datamatrix
#' @param network network
#' @param delta delta
#' @param network.nodes network.nodes
#' @param G G
#' @param params params
#' @return mutual information matrix
#' @author Maintainer: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
get.mis <- function(datamatrix, network, delta, network.nodes, G, params) {
    
    mis <- c()
    mi.cnt <- 0
    for (edge in which(is.na(delta))) {
        mi.cnt <- mi.cnt + 1
        # Pick node indices
        a <- network[1, edge]
        i <- network[2, edge]
        dat <- cbind(prcomp(matrix(datamatrix[, network.nodes[G[[a]]]], nrow(datamatrix)), 
            center = TRUE)$x[, 1], prcomp(matrix(datamatrix[, network.nodes[G[[i]]]], 
            nrow(datamatrix)), center = TRUE)$x[, 1])
        
        # mis[[mi.cnt]] <- build.mim(dat, estimator='mi.empirical', disc = 'equalwidth',
        # nbins = params$nbins)[1, 2]
        mis[[mi.cnt]] <- build.mim(dat, estimator = "spearman")[1, 2]
        
    }
    
    as.vector(unlist(mis))
    
}




build.mim <- function(dataset, estimator = "spearman", disc = "none", nbins = sqrt(NROW(dataset))) {
    # This function is licensed under cc-by-sa 3.0 Modified from minet 3.6.0 to use
    # discretize correctly (not exported in build.mim which may cause occasional
    # function name conflicts)
    
    # if (disc == 'equalfreq' || disc == 'equalwidth' || disc == 'globalequalwidth')
    # dataset <- infotheo::discretize(dataset, disc, nbins)
    
    if (estimator == "pearson" || estimator == "spearman" || estimator == "kendall") {
        mim <- cor(dataset, method = estimator, use = "complete.obs")^2
        diag(mim) <- 0
        maxi <- 0.999999
        mim[which(mim > maxi)] <- maxi
        mim <- -0.5 * log(1 - mim)
    } else {
        stop("unknown estimator")
    }
    # else if (estimator == 'mi.mm') estimator <- 'mm' else if (estimator ==
    # 'mi.empirical') estimator <- 'emp' else if (estimator == 'mi.sg') estimator <-
    # 'sg' else if (estimator == 'mi.shrink') estimator = 'shrink' if (estimator ==
    # 'mm' || estimator == 'emp' || estimator == 'sg' || estimator == 'shrink') { mim
    # <- mutinformation(dataset, method = estimator) diag(mim) <- 0 }
    mim[mim < 0] <- 0
    mim
}

# Copyright (C) 2010-2012 Leo Lahti Contact: Leo Lahti <leo.lahti@iki.fi> This
# program is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2, or (at your option) any later version.  This
# program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.
setMethod(f = "show", signature("NetResponseModel"), function(object) {
    cat("NetResponseModel object\n")
})

# Copyright (C) 2010-2013 Leo Lahti Contact: Leo Lahti <leo.lahti@iki.fi> This
# program is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2, or (at your option) any later version.  This
# program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.

#' @title Investigate association of a continuous variable and the modes
#' @description Investigate association of a continuous variable and the modes.
#' @param annotation.df annotation data.frame with discrete factor levels, rows
#' named by the samples
#' @param groupings Sample mode information. Each element corresponds to one of the modes and lists the samples assignment matrix qofz. Alternatively, a vector of mode indices named by the samples can be given.
#' @param method method for quantifying the association
#' @param pth p-value threshold (for adjusted p-values)
#' @param verbose verbose 
#' @param rounding rounding digits
#' @param adjust.p Adjust p-values (this will add p.adj column and remove pvalue column in the output table)
#' @return Table listing all associations between the factor levels and responses
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @importFrom qvalue qvalue
#' @export
#' @keywords utilities
#' @examples res <- list.responses.continuous.single(annotation.df = NULL, groupings = NULL)
list.responses.continuous.single <- function(annotation.df, groupings, method = "t-test", 
    pth = Inf, verbose = TRUE, rounding = NULL, adjust.p = TRUE) {

    if (is.null(groupings)) {return(NULL)}

    # Collect the tables from all factors and levels here
    collected.table <- NULL
    
    # Quantify association to each response for the continuous variable
    if (is.null(names(groupings))) {
        names(groupings) <- seq_len(length(groupings))
    }
    
    if (verbose) {
        message("Check mode lists")
    }
    
    # Convert grouping info to a list
    groupings.list <- listify.groupings(groupings)
    
    if (verbose) {
        message("Go through annotations")
    }
    for (fnam in colnames(annotation.df)) {
        
        if (verbose) {
            message(fnam)
        }
        
        annotation.vector <- annotation.df[[fnam]]
        names(annotation.vector) <- rownames(annotation.df)
        
        responses.per.cont <- enrichment.list(groupings.list, annotation.vector)
        
        responses.per.cont$annotation <- rep(fnam, nrow(responses.per.cont))
        
        collected.table <- rbind(collected.table, responses.per.cont)
        
    }
    
    if (nrow(collected.table) > 0) {
        
        nainds <- is.na(collected.table$pvalue)
        
        if (sum(nainds) > 0) {
            warning("Removing entries where p-values could not be calculated due to small sample size and/or missing values")
            collected.table <- collected.table[!nainds, ]
        }
        
        if (adjust.p) {
            
            collected.table$p.adj <- rep(NA, nrow(collected.table))
            if (nrow(collected.table) > 100) {
                if (verbose) {
                  message("Adjusting p with q")
                }
                qv <- qvalue(collected.table$pvalue, pi0.method = "bootstrap", fdr.level = 0.25)
                if (("qvalues" %in% names(qv))) {
                  collected.table$p.adj <- qv$qvalues
                }
            } else {
                if (verbose) {
                  message("Adjusting p with BH")
                }
                collected.table$p.adj <- p.adjust(collected.table$pvalue, method = "BH")
            }
        }
        
        # Order by pvalues
        collected.table <- collected.table[order(collected.table$pvalue), ]
        
        # Filtering based on p.adjs, if p.adjs are available
        if (adjust.p && (any(!is.na(collected.table$p.adj)) && !is.null(pth))) {
            collected.table <- collected.table[collected.table$p.adj < pth, ]
        }
    } else {
        collected.table <- NULL
    }
    if (length(collected.table) == 0) {
        collected.table <- NULL
    }
    
    if (!is.null(rounding)) {
        collected.table$p.adj <- round(collected.table$p.adj, rounding)
        collected.table$pvalue <- round(collected.table$pvalue, rounding)
        collected.table$fold.change <- round(collected.table$fold.change, rounding)
    }
    
    collected.table
    
}

# Generate Nc components from normal-inverseGamma prior

set.seed(12346)

Ns <- 300
Nd <- 2

# Isotropic cloud
D1 <- matrix(rnorm(Ns*Nd), ncol = Nd) 

# Single diagonal mode
D2 <- matrix(rnorm(Ns*Nd), ncol = Nd) %*% rbind(c(1,2), c(2,1)) 

# Two isotropic modes
D3 <- rbind(matrix(rnorm(Ns/2*Nd), ncol = Nd), matrix(rnorm(Ns/2*Nd, mean = 3), ncol = Nd))
D <- cbind(D1, D2, D3)

colnames(D) <- paste("Feature-",  1:ncol(D), sep = "")
rownames(D) <- paste("Sample-", 1:nrow(D), sep = "")

# Play with different options and check their effect on  running times for bic and vdp 

skip <- TRUE

if (!skip) {

  Ns <- 100
  Nd <- 2

  set.seed(3488400)

  D <- cbind(

     	rbind(matrix(rnorm(Ns*Nd, mean = 0), ncol = Nd), 
       	      matrix(rnorm(Ns*Nd, mean = 2), ncol = Nd),
      	      cbind(rnorm(Ns, mean = -1), rnorm(Ns, mean = 3))
 	    ), 

     	rbind(matrix(rnorm(Ns*Nd, mean = 0), ncol = Nd), 
       	      matrix(rnorm(Ns*Nd, mean = 2), ncol = Nd),
      	      cbind(rnorm(Ns, mean = -1), rnorm(Ns, mean = 3))
 	    )
	    )

  rownames(D) <- paste("R", 1:nrow(D), sep = "-")
  colnames(D) <- paste("C", 1:ncol(D), sep = "-")

  ts <- c()
  for (mm in c("bic", "vdp")) {


    # NOTE: no PCA basis needed with mixture.method = "bic"
    tt <- system.time(detect.responses(D, verbose = TRUE, max.responses = 5, 
	   		       mixture.method = mm, information.criterion = "BIC", 
			       merging.threshold = 0, bic.threshold = 0, pca.basis = TRUE))

    print(paste(mm, ":", round(tt[["elapsed"]], 3)))
    ts[[mm]] <- tt[["elapsed"]]
  }

   print(paste(names(ts)[[1]], "/", names(ts)[[2]], ": ", round(ts[[1]]/ts[[2]], 3)))

}

# -> VDP is much faster when sample sizes increase 
# 1000 samples -> 25-fold speedup with VDP

#' @title BIC mixture
#' @description Latent class analysis based on (infinite) Gaussian mixture model. If the input is data matrix,
#' a multivariate model is fitted; if the input is a vector, a univariate model is fitted
#' @param x samples x features matrix for multivariate analysis, or a vector for univariate analysis 
#' @param max.modes Maximum number of modes to be checked for mixture model selection
#' @param bic.threshold BIC threshold which needs to be exceeded before a new mode is added to the mixture.
#' @param min.modes minimum number of modes
#' @param ... Further optional arguments to be passed
#' @return Fitted latent class model (parameters and free energy)
#' @references See citation('netresponse') 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords utilities
bic.mixture <- function(x, max.modes, bic.threshold = 0, min.modes = 1, ...) {

    # x; max.modes = max.responses; bic.threshold = bic.threshold; min.modes =
    # min.responses
    
    if (!is.vector(x) && ncol(x) == 1) {
        x <- x[, 1]
    }
    
    if (is.vector(x)) {
        bic.mixture.univariate(x, max.modes, bic.threshold, min.modes = min.modes, 
            ...)
    } else {
        bic.mixture.multivariate(x, max.modes, bic.threshold, min.modes = min.modes, 
            ...)
    }
    
}

# 'The language of science is the language of probability, and not of p-values.'
# -- Luis Pericchi

#' @title Set breaks
#' @description Set breakpoints for two-way color palette.
#' @usage set.breaks(mat, interval = 0.1)
#' @param mat Matrix to visualize.
#' @param interval Density of color breakpoints.
#' @return A vector listing the color breakpoints.
#' @author Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao. Maintainer:
#' Leo Lahti <leo.lahti@@iki.fi>
#' @references L. Lahti et al.: Global modeling of transcriptional responses in
#' interaction networks. Submitted.
#' @keywords utilities
#' @export
#' @examples
#'  set.breaks(array(rnorm(100), dim = c(10, 10)), interval = .1) 
set.breaks <- function(mat, interval = 0.1) {
    if (max(abs(mat)) > 1) {
        m <- floor(max(abs(mat)))
    } else {
        m <- round(max(abs(mat)), nchar(1/interval) - 1)
    }
    
    mm <- m + interval/2
    vals <- seq(interval/2, mm, interval)
    # Note: the first and last values mimic infinity
    mybreaks <- c(-(m + 1e+06), c(-rev(vals), vals), m + 1e+06)
    mybreaks
}



ellipse <- function(centroid, covmat, confidence = 0.95, npoints = 100) {
    
    # Centroid: x0, y0 axes: a, b
    x0 <- centroid[[1]]
    y0 <- centroid[[2]]
    
    # Pick axis-wise stds
    a <- sqrt(diag(covmat))[[1]]
    b <- sqrt(diag(covmat))[[2]]
    
    theta <- seq(0, 2 * pi, length = npoints)
    
    # Confidence intervals (df=1: 1.96; df=2: 2.45)
    cint <- sqrt(qchisq(confidence, 2))
    
    # Determine point on the ellipse
    x <- x0 + cint * a * cos(theta)
    y <- y0 + cint * b * sin(theta)
    
    # Rotate alpha degrees from the x-axis: x <- x0 + a * cos(theta) * cos(alpha) - b
    # * sin(theta) * sin(alpha) y <- y0 + a * cos(theta) * sin(alpha) + b *
    # sin(theta) * cos(alpha)
    
    # Output ellipse points
    cbind(x, y)
    
}

# Copyright (C) 2008-2012 Olli-Pekka Huovilainen and Leo Lahti Contact: Leo Lahti
# <leo.lahti@iki.fi> This program is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your option) any
# later version.  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.  Acknowledgements: This program is based on the AIVGA
# Agglomerative Independent Variable Group Analysis package (v. 1.0) Copyright
# (C) 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
# Valpola, and Paul Wagner.

#' independent.models
#' 
#' Mainly for internal use. Provide independent models for each node.
#'
#' @param datamatrix datamatrix
#' @param params parameters
#' @return 
#'   \item{nodes }{Model for each node} 
#'   \item{C }{Costs for individual models}
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @export
#' @examples res <- independent.models(datamatrix = NULL, params = NULL)
#' @keywords internal
independent.models <- function(datamatrix, params) {

    if (is.null(datamatrix)) {return(NULL)}

    # Storage list for calculated models
    model.nodes <- vector(length = ncol(datamatrix), mode = "list")  # individual nodes
    
    if (params$verbose) {
        message("Compute cost for each variable")
    }
    
    C <- vector(length = ncol(datamatrix), mode = "numeric")
    
    # FIXME parallelize?
    for (k in seq_len(ncol(datamatrix))) {
        
        node <- colnames(datamatrix)[[k]]
        
        if (params$verbose) {
            message(paste("Computing model for node", k, "/", ncol(datamatrix)))
        }
        
        Nparams <- NULL
        
        model.params <- mixture.model(matrix(datamatrix[, node], nrow(datamatrix)), 
            mixture.method = params$mixture.method, max.responses = params$max.responses, 
            implicit.noise = params$implicit.noise, prior.alpha = params$prior.alpha, 
            prior.alphaKsi = params$prior.alphaKsi, prior.betaKsi = params$prior.betaKsi, 
            vdp.threshold = params$vdp.threshold, initial.responses = params$initial.responses, 
            ite = params$ite, speedup = params$speedup, bic.threshold = params$bic.threshold, 
            pca.basis = params$pca.basis)
        
        # Cost for model
        C[[k]] <- info.criterion(model.params$Nparams, params$Nlog, -model.params$free.energy, 
            criterion = params$information.criterion)
        
        model.nodes[[k]] <- model.params
        
    }
    
    gc()
    
    if (params$verbose) {
        message("independent models done")
    }
    
    list(nodes = model.nodes, C = C)
    
}

# 1. vdp.mixt: moodien loytyminen eri dimensiolla, naytemaarilla ja komponenteilla
#   -> ainakin nopea check

#######################################################################

# Generate random data from five Gaussians. 
# Detect modes with vdp-gm. 
# Plot data points and detected clusters with variance ellipses

#######################################################################

library(netresponse)
#source("~/Rpackages/netresponse/netresponse/R/detect.responses.R")
#source("~/Rpackages/netresponse/netresponse/R/internals.R")
#source("~/Rpackages/netresponse/netresponse/R/vdp.mixt.R")
#dyn.load("/home/tuli/Rpackages/netresponse/netresponse/src/netresponse.so")


#########  Generate DATA #############################################

res <- generate.toydata()
D <- res$data
component.means <- res$means
component.sds   <- res$sds
sample2comp     <- res$sample2comp

######################################################################

# Fit nonparametric Gaussian mixture model
out <- vdp.mixt(D)
# out <- vdp.mixt(D, c.max = 3) # try with limited number of components -> OK

############################################################

# Compare input data and results

ord.out <- order(out$posterior$centroids[,1])
ord.in <- order(component.means[,1])

means.out <- out$posterior$centroids[ord.out,]
means.in <- component.means[ord.in,]

# Cluster stds and variances
sds.out <- out$posterior$sds[ord.out,]
sds.in  <- component.sds[ord.in,]
vars.out <- sds.out^2
vars.in <- sds.in^2

# Check correspondence between input and output
if (length(means.in) == length(means.out)) {
   cm <- cor(as.vector(means.in), as.vector(means.out))
   csd <- cor(as.vector(sds.in), as.vector(sds.out))
}

# Plot results (assuming 2D)

ran <- range(c(as.vector(means.in - 2*vars.in), 
               as.vector(means.in + 2*vars.in), 
	       as.vector(means.out + 2*vars.out), 
	       as.vector(means.out - 2*vars.out)))

#plot(D, pch = 20, main = paste("Cor.means:", round(cm,3), "/ Cor.sds:", round(csd,3)), xlim = ran, ylim = ran) 
#for (ci in 1:nrow(means.out))  { add.ellipse(centroid = means.out[ci,], covmat = diag(vars.out[ci,]), col = "red") }
#for (ci in 1:nrow(means.in))  { add.ellipse(centroid = means.in[ci,], covmat = diag(vars.in[ci,]), col = "blue") }

# Copyright (C) 2010-2013 Leo Lahti Contact: Leo Lahti <leo.lahti@iki.fi> This
# program is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2, or (at your option) any later version.  This
# program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.

#' Enrichment for a specified sample group in the given response.
#' 
#' Calculate enrichment values for a specified sample group in the given
#' response.
#'
#' @param total.samples All samples in the data
#' @param response.samples Samples in the investigated subset
#' @param annotated.samples Samples at the investigated annotation level for enrichment calculation
#' @param method Enrichment method.
#'
#' @return List with enrichment statistics, depending on enrichment method.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @seealso order.responses
#' @references See citation('netresponse')
#' @keywords utilities
#' @examples #enr <- response.enrichment(subnet.id, models, sample, response, method)

response.enrichment <- function(total.samples, response.samples, annotated.samples, 
    method = "hypergeometric") {
    
    if (length(response.samples) == 0) {
        warning("No samples in response")
        return(NULL)
    }
    # Fixme: minor stochasticity here, perhaps due to numerical limitations?  Method
    # indicates which test will be used; the higher the better score
    if (method == "hypergeometric") {
        enr <- enrichment.score(total.samples, response.samples, annotated.samples, 
            method = method)
    }
    
    # This could be implemented, not sure how useful it would be P(r|S) = P(S,r)/P(S)
    # this assumes that all samples in S come from exactly one of the responses prS
    # <- P.rS(samples, model, pars = NULL, subnet.id, log = FALSE) Or normalized
    # version of the above: P(r|S)/P(r)
    
    if (method == "dependency") {
        
        # log(P(s,r)/P(s)P(r))
        
        # now with actual sample density (not density mass as above) P(S) = sum_r P(S,r)
        
        # ps.log <- log(sum(get.P.rs.joint(s, models, subnet.id, log = FALSE)))
        
        # this requires features x samples matrix psr.log <- P.s.r(t(dat), pars, log =
        # TRUE)
        
        # log(P(s,r)/P(s)P(r)) = log(P(s|r)/P(s)) enr <- list(score = psr.log - ps.log,
        # info = NULL)
        
    }
    
    if (method == "precision") {
        
        # This differs from 'hypergeometric' and 'dependency' in that they compare
        # proportion of factor level in response to factor level in overall model now we
        # compare proportion of factor level in response to overall response (all samples
        # in the response) This does not necessarily correlate with the two other
        # measures.  In a way, this measures the purity of the response w.r.t. given
        # factor level
        
        # precision: TP/(TP + FP) = TP/n fraction of true posivites in response -> here:
        # density mass associated with this sample in each response additionally
        # normalize by the analogous fraction in overall model density mass is the sum of
        # individual sample densities
        
        # P(s|r) / P(S|r)
        
        # density for each data point dens <- sample.densities(s.ann, model, subnet.id,
        # log = FALSE, summarize = FALSE)[response, s.ann]
        
        # P(s,r)/P(s)P(r) = P(s|r)/P(s) for factor level samples Fraction of total
        # density mass of factor level sample compared to all samples within the response
        # and w.r.t. overall density mass of the sample
        
        # relative density of sample enr <- list(score = sum(dens[s])/sum(dens), info =
        # NULL)
        
    }
    
    # recall: TP/(TP + FN) = TP/P fraction of all true positives included in the
    # response -> Add later
    
    # later utilize probabilistic interpretation of precision/recall?
    
    enr
    
}

#' Convert grouping info into a list; each element corresponds to a
#' group and lists samples in that group. 
#' 
#' @param groupings a list, a vector, or a samplesxmodes assignment matrix
#' @param verbose verbose
#'
#' @return Group list
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords utilities
#' @export
#' @examples res <- listify.groupings(groupings = NULL)
listify.groupings <- function(groupings, verbose = FALSE) {

    if (is.null(groupings)) {return(NULL)}

    if (is.matrix(groupings[[1]])) {
        if (verbose) {
            message("Convert mode matrix to list")
        }
        groupings.list <- list()
        for (sn in names(groupings)) {
            # samples in each mode (hard assignment)
            groupings[[sn]] <- response2sample(groupings[[sn]])
        }
    } else if (is.vector(groupings) && !is.list(groupings)) {
        if (verbose) {
            message("Convert mode vector to list")
        }
        groupings.list <- split(names(groupings), groupings)
    } else if (is.list(groupings)) {
        if (verbose) {
            message("Mode list ok")
        }
        groupings.list <- groupings
    }
    
    groupings.list
}


#' Convert grouping info into a vector; each element corresponds to a
#' group and lists samples in that group.
#' 
#' @param groupings a list, a vector, or a samplesxmodes assignment matrix
#' @param verbose verbose
#'
#' @return Indicator vector
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords utilities
#' @examples #

vectorize.groupings <- function(groupings, verbose = FALSE) {
    
    if (is.matrix(groupings[[1]])) {
        if (verbose) {
            message("Convert mode matrix to vector")
        }
        gv <- colnames(groupings)[apply(groupings, 1, which.max)]
    } else if (is.vector(groupings) && !is.list(groupings)) {
        if (verbose) {
            message("Convert mode vector to list")
        }
        gv <- groupings
    } else if (is.list(groupings)) {
        gv <- unlist(sapply(seq_len(length(groupings)), function(i) {
            rep(names(groupings)[[i]], length(groupings[[i]]))
        }))
        names(gv) <- unlist(groupings)
    }
    
    gv
}

setClass("NetResponseModel", representation(moves = "matrix",
    last.grouping = "list", 
    subnets = "list", params = "list", datamatrix = "matrix",
    network = "graphNEL", 
    models = "list"))

# Copyright (C) 2010-2012 Leo Lahti Contact: Leo Lahti <leo.lahti@iki.fi> This
# program is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2, or (at your option) any later version.  This
# program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License for more details.


info.criterion <- function(nparams, nlog, logp, criterion = "BIC") {
    
    # Calculate various information criteria
    
    if (criterion == "AIC") {
        # Akaike IC
        return(AIC(nparams, nlog, logp))
    } else if (criterion == "BIC") {
        # Bayesian IC
        return(BIC(nparams, nlog, logp))
    } else if (criterion == "AICc") {
        # Akaike for linear models, finite sample
        return(AICc(nparams, nlog, logp))
    }
    
}




BIC <- function(nparams, nlog, logp) {
    
    # Calculate Bayesian Information Criterion (BIC)
    
    # NOTE: Original formulation (Schwartz) assumed that data is iid and likelihood
    # is in exponential family.  However, BIC is later derived with considerably
    # loosened assumptions, see e.g.  Cavanaugh et al.: Generalizing the Derivation
    # of the Schwarz Information Criterion it seems that the assumptions listed in
    # Section 3 will quarantee the validity of BIC for mixtures of exponential family
    # distributions; confirm.
    
    # negative free energy is lower bound for log(P(D|H)) logp = -cost
    
    nparams * nlog - 2 * logp
    
}

AIC <- function(nparams, nlog, logp) {
    
    # Calculate Akaike Information Criterion (AIC) Note: nlog not used here but
    # included for compatibility
    
    # negative free energy is lower bound for log(P(D|H)) logp = -cost
    
    2 * (nparams - logp)
    
}

AICc <- function(nparams, nlog, logp) {
    
    # Calculate Akaike Information Criterion correction for finite sample size (AICc)
    
    # AICc is AIC with a correction for finite sample sizes, giving a greater penalty
    # for extra parameters. Burnham & Anderson (2002) strongly recommend using AICc,
    # rather than AIC, if n is small or k is large. Since AICc converges to AIC as n
    # gets large, AICc generally should be employed. Using AIC, instead of AICc, when
    # n is not many times larger than k2, increases the probability of selecting
    # models that have too many parameters, i.e. of overfitting. The probability of
    # AIC overfitting can be substantial, in some cases. Brockwell & Davis (p. 273)
    # advise using AICc as the primary criterion in selecting the orders of an ARMA
    # model for time series. McQuarrie & Tsai ground their high opinion of AICc on
    # extensive simulation work with regression and time series. AICc was first
    # proposed by Hurvich & Tsai (1989). Different derivations of it are given by
    # Brockwell & Davis, Burnham & Anderson, and Cavanaugh. All the derivations
    # assume a univariate linear model with normally-distributed errors (conditional
    # upon regressors); if that assumption does not hold, then the formula for AICc
    # will usually change. Further discussion of this, with examples of other
    # assumptions, is given by Burnham & Anderson (2002, ch.7). Note that when all
    # the models in the candidate set have the same k, then AICc and AIC will give
    # identical (relative) valuations. In that situation, then, AIC can always be
    # used.
    
    # negative free energy is lower bound for log(P(D|H)) logp = -cost
    
    n <- exp(nlog)
    
    AIC(nparams, nlog, logp) + 2 * nparams(nparams + 1)/(n - nparams - 1)
    
}




# AIC.c <- cmpfun(AIC) AICc.c <- cmpfun(AICc) BIC.c <- cmpfun(BIC)

get.subnet <- function(res, subnet.id) {
    
    if (is.numeric(subnet.id)) {
        subnet.id <- paste("Subnet", subnet.id, sep = "-")
        warning("subnet.id given as numeric; converting to character: ", subnet.id, 
            sep = "")
    }
    
    # Nodes for a given subnet
    get.subnets(res)[[subnet.id]]
    
}


compute.weight <- function(pt, mu, vars, xt) {
    
    # pt <- qOFz[t,] # P(c|t) for all c xt <- dat[t, ] # data point
    
    # Initial weights are zero
    w <- rep.int(0, nrow(mu))
    sds <- sqrt(vars)
    
    # Avoid overflows: no probability can be 0 exactly. Add negligible constant in
    # these cases
    pt[pt < 9.99988867182683e-321] <- 9.99988867182683e-321
    pt <- pt/sum(pt)  # renormalize
    
    # set arbitrary weigh for the first cluster (only relations between weights
    # matter) normalize later w[[1]] <- 1 # added below operate on log domain to
    # avoid floating errors
    
    logdens <- sum(dnorm(xt, mu[1, ], sds[1, ], log = TRUE))
    
    if (nrow(mu) > 1) {
        logw <- sapply(2:nrow(mu), function(i) {
            # print(prod(dnorm(xt, mu[i, ], sds[i, ])))
            
            # dens * pt[[i]]/(pt[[1]] * prod(dnorm(xt, mu[i, ], sds[i, ])))
            
            logdens + log(pt[[i]]) - log(pt[[1]]) - sum(dnorm(xt, mu[i, ], sds[i, 
                ], log = TRUE))
            
        })
        w <- exp(logw)
        
        # Densities in original domain, standardized s.t. first weight is 1
        w <- c(1, w)
        
        # normalize to unity and return
        return(w/sum(w))
    } else {
        return(1)  #w <- 1
    }
    
}



############################################ 


retrieve.model <- function(model, subnet.id) {
    
    # Recalculate the model for a given subnet Note: the algorithm has some
    # stochasticity in initialization etc.  so the results may not be exactly same
    # each time
    
    # Former: get.model
    
    # model: output from run.netresponse function subnet.id: id/index of the subnet
    # to check level: which agglomeration step datamatrix for which the model was
    # calculated
    
    # Copyright (C) 2008-2012 Leo Lahti Licence: GPL >=2
    
    if (is.numeric(subnet.id)) {
        subnet.id <- paste("Subnet", subnet.id, sep = "-")
        warning("subnet.id given as numeric; converting to character: ", subnet.id, 
            sep = "")
    }
    
    # Get subnet nodes
    nodes <- model@subnets[[subnet.id]]
    
    # Compute the model
    x <- matrix(model@datamatrix[, nodes], nrow(model@datamatrix))
    
    pars <- mixture.model(x, mixture.method = model$params$mixture.method, max.responses = model$params$max.responses, 
        implicit.noise = model$params$implicit.noise, prior.alpha = model$params$prior.alpha, 
        prior.alphaKsi = model$params$prior.alphaKsi, prior.betaKsi = model$params$prior.betaKsi, 
        vdp.threshold = model$params$vdp.threshold, initial.responses = model$params$initial.responses, 
        ite = model$params$ite, speedup = model$params$speedup, bic.threshold = model$params$bic.threshold, 
        pca.basis = model$params$pca.basis)
    
    
    pars
}



# INPUT: data, hp_posterior, hp_prior, opts OUTPUT:
# list(free_energy,hp_posterior,data,c) * free_energy: free energy of the best
# split found * hp_posterior: posterior info of the best split found * c: index
# of the cluster that resulted in the best split found DESCRIPTION: Implements
# the VDP algorithm steps 2 to 4.

find.best.splitting <- function(data, hp.posterior, hp.prior, opts, min.size = 5) {
    
    # min.size: minimum size of a component required for splitting
    
    # Copyright (C) 2008-2012 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    dat <- data$given.data$X1
    epsilon <- 1e-10  # FIXME: should this be a tunable function parameter?
    c.max <- opts$c.max
    
    # ALGORITHM STEP 2
    
    # Sort clusters by size, use at most c.max candidates and ensure cluster size is
    # > 2
    candidates <- order(hp.posterior$Nc, decreasing = TRUE)
    candidates <- candidates[hp.posterior$Nc[candidates] > 2]
    # candidates <- which(hp.posterior$Nc > 2)
    if (length(candidates) == 0) {
        c <- 1
    }
    
    qOFz <- mk.qOFz(data, hp.posterior, hp.prior, opts)
    fc <- mk.E.log.q.p.eta(data, hp.posterior, hp.prior, opts)
    log.lambda <- mk.log.lambda(data, hp.posterior, hp.prior, opts)
    sub.data <- data
    
    # ALGORITHM STEP 3 (3a,3b,3c) check which split gives best improvements in cost
    
    new.qOFz.list <- list()  #Initialize
    new.free.energy <- rep(Inf, min(c.max, length(candidates)))
    
    for (c in candidates[seq_len(min(c.max, length(candidates)))]) {
        
        # relating.n has the indexes of data points that belonged to the candidate
        # cluster prior to splitting (that's why it is the sum over the now 2 clusters
        # (after splitting).  REMARK: Is 0.5 ok? - when there are lots of clusters it is
        # natural to assume some points will have less than 0.5 for any cluster.
        
        relating.n <- which(qOFz[, c] > 0.5)
        if (length(relating.n) == 0) {
            next
        } else {
        }
        
        # ALGORITHM STEP 3a. split the candidate cluster Split cluster c in qOFz into two
        # smaller ones
        
        new.c <- ncol(qOFz)
        new.qOFz <- split.qofz(qOFz, c, new.c, dat, opts$speedup, min.size)
        
        new.K <- ncol(new.qOFz)
        sub.qOFz <- new.qOFz[relating.n, unique(c(c, new.c, new.K))]
        
        # Ensure it remains a matrix
        sub.data$given.data$data <- sub.data$given.data$X1 <- array(dat[relating.n, 
            ], dim = c(length(relating.n), ncol(dat)))
        
        # ALGORITHM STEP 3b and 3c update the posterior of the split clusters for a small
        # number of iter.  update_posterior sorts clusters by size
        
        sub.hp.posterior <- mk.hp.posterior(sub.data, sub.qOFz, hp.prior, opts)
        dummylist <- updatePosterior(sub.data, sub.hp.posterior, hp.prior, opts, 
            10, 0)
        sub.hp.posterior <- dummylist$hp.posterior
        sub.qOFz <- dummylist$qOFz
        
        # FIXME: check this already previously for c == new.c?
        if (ncol(sub.qOFz) < 3) {
            next
        } else {
        }
        
        # If there are more than 1 empty components then go to next step
        if (sum(colSums(sub.qOFz) < epsilon) > 1) {
            next
        } else {
        }
        
        sub.log.lambda <- mk.log.lambda(data, sub.hp.posterior, hp.prior, opts)
        insert.indices <- c(c, new.c, new.K:(new.K + ncol(sub.qOFz) - 3))
        
        if (max(insert.indices) > ncol(log.lambda)) {
            new.log.lambda <- cbind(log.lambda, array(0, dim = c(nrow(log.lambda), 
                max(insert.indices) - ncol(log.lambda))))
        } else {
            new.log.lambda <- log.lambda
        }
        
        
        new.log.lambda[, insert.indices] <- sub.log.lambda
        new.fc <- fc
        new.fc[insert.indices] <- mk.E.log.q.p.eta(sub.data, sub.hp.posterior, hp.prior, 
            opts)
        new.free.energy[[c]] <- mk.free.energy(data, sub.hp.posterior, hp.prior, 
            opts, new.fc, new.log.lambda)$free.energy
        
        # if new.qOFz is not large enough to accommodate update from sub.qOFz then add
        # columns
        if (ncol(new.qOFz) < max(insert.indices)) {
            new.qOFz <- cbind(new.qOFz, array(0, dim = c(nrow(new.qOFz), max(insert.indices) - 
                ncol(new.qOFz))))
        }
        
        new.qOFz[relating.n, ] <- 0
        new.qOFz[relating.n, insert.indices] <- sub.qOFz
        new.qOFz.list[[c]] <- new.qOFz
    }
    
    # Select cluster split that minimizes free energy
    c <- which.min(new.free.energy)
    free.energy <- new.free.energy[[c]]
    
    
    if (is.infinite(free.energy)) {
        c <- -1
    } else {
        hp.posterior <- mk.hp.posterior(data, new.qOFz.list[[c]], hp.prior, opts)
    }
    
    list(free.energy = free.energy, hp.posterior = hp.posterior, data = data, c = c)
}




find.best.neighbor <- function(G, max.subnet.size, network, delta) {
    
    # Order edges by delta values. The two subnets with the smallest delta are joined
    # unless the merged subnet exceeds max size.
    o <- order(delta)
    
    # Check size of the resulting merged subnet one-by-one, starting from the
    # smallest
    best.found <- FALSE
    cnt <- 0
    best.edge <- a <- b <- NULL
    mindelta <- Inf
    
    while (!best.found) {
        cnt <- cnt + 1
        ind <- o[[cnt]]  # FIXME: could use the filtering for top neighborghs here as well
        z <- network[1, ind]
        i <- network[2, ind]
        
        # Finish when the new merged subnetwork, which does not exceed max size, is found
        if (length(c(G[[z]], G[[i]])) <= max.subnet.size) {
            # Sort a and b
            a <- min(c(z, i))
            b <- max(c(z, i))
            best.edge <- ind
            mindelta <- delta[[best.edge]]
            best.found <- TRUE
        } else {
            # Infinite cost for merges that exceed maximum size
            delta[[ind]] <- Inf
        }
    }
    
    list(a = a, b = b, mindelta = mindelta, best.edge = best.edge, delta = delta)
    
}


join.subnets <- function(network, delta, best.edge) {
    
    # Pick the nodes to merge
    a <- network[1, best.edge]
    b <- network[2, best.edge]
    
    # replace b nodes by a: this replaces edges from other nodes pointing to b to
    # point into a
    network[network == b] <- a
    
    # remove delta values for nodes associated with node a since this node now has
    # different (merged) set of features and the model needs to be recalculated
    inds <- apply(network == a, 2, any)
    delta[inds] <- NA
    
    # sort entries (row1 < row2)
    network <- matrix(apply(network, 2, sort), 2)
    
    list(network = network, delta = delta)
    
}






# INPUT: matrix (q_of_z) OUTPUT: matrix

# DESCRIPTION: Sorted matrix in decreasing fashion based on the value of colSums.
# Remark: The last column of the matrix is kept in place (it is not sorted).

sortqofz <- function(qOFz) {
    
    # Copyright (C) 2008-2010 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    Nc <- colSums(qOFz)
    I <- order(Nc[-length(Nc)], decreasing = TRUE)
    I <- c(I, ncol(qOFz))  # add last column element separately (outside of ordering)
    
    # Order the cols and ensure qOFz remains a matrix
    array(qOFz[, I], dim = dim(qOFz))
    
}


# INPUT: data - matrix with data vectors K - number of clusters OUTPUT: q_of_z -
# matrix of size N*(K+1).  DESCRIPTION: This function assigns data randomly to K
# clusters by drawing cluster membership values from a uniform distribution.

rand.qOFz <- function(N, K) {
    
    # Copyright (C) 2008-2012 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    qOFz <- matrix(runif(N * (K + 1)), N, K + 1)
    qOFz[, K + 1] <- 0
    
    # normalize and return (each row should sum up to 1)
    qOFz/rowSums(qOFz)
    
}




# INPUT: 'old' free_energy value, 'new' free_energy value, options OUTPUT: bool:
# 0 if the there was no significant improvement.  1 if new_free_energy is smaller
# than free_energy (more than opts$threshold).



free.energy.improved <- function(free.energy, new.free.energy, warn.when.increasing, 
    threshold) {
    
    # INPUT: 'old' free.energy value, 'new' free.energy value, options OUTPUT: bool:
    # 0 if the there was no significant improvement.  1 if new.free.energy is smaller
    # than free.energy (more than opts$threshold).
    
    # Copyright (C) 2008-2010 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    diff <- new.free.energy - free.energy
    
    v <- abs(diff/free.energy)
    
    if (is.nan(v) || v < threshold) {
        bool <- 0
    } else {
        if (diff > 0) {
            if (warn.when.increasing) {
                if (v > 0.001) {
                    stop(c("the free energy increased. The diff is ", toString(diff)))
                } else {
                    warning(c("the free energy increased. The diff is ", toString(diff)))
                }
            }
            bool <- 0
        } else {
            bool <- ifelse(diff == 0, 0, 1)
        }
    }
    
    bool
}




# INPUT: data: structure with data matrix hp_posterior: posterior information for
# the current mixture model hp_prior: prior information opts: options list.
# OUTPUT: list(free_energy, hp_posterior, hp_prior, data); DESCRIPTION: Read the
# main description on the beginning of the file.


greedy <- function(data, hp.posterior, hp.prior, opts, min.size) {
    
    # Copyright (C) 2008-2012 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    free.energy <- mk.free.energy(data, hp.posterior, hp.prior, opts)$free.energy
    
    while (1) {
        
        # ALGORITHM STEP 2-4
        
        templist <- find.best.splitting(data, hp.posterior, hp.prior, opts, min.size)
        new.free.energy <- templist$free.energy
        new.hp.posterior <- templist$hp.posterior
        c <- templist$c
        if (c == (-1)) 
            {
                break
            }  # infinite free energy -> break splitting
        
        # ALGORITHM STEP 5
        dummylist <- updatePosterior(data, new.hp.posterior, hp.prior, opts, ite = opts$ite, 
            do.sort = 1)
        
        new.free.energy <- dummylist$free.energy
        new.hp.posterior <- dummylist$hp.posterior
        
        # ALGORITHM STEP 6
        if (free.energy.improved(free.energy, new.free.energy, 0, opts$threshold) == 
            0) {
            break  #free.energy didn't improve, greedy search is over
        }
        
        free.energy <- new.free.energy
        hp.posterior <- new.hp.posterior
        
    }
    
    list(free.energy = free.energy, hp.posterior = hp.posterior, hp.prior = hp.prior, 
        data = data)
}

############################################################################### 


# INPUT: data, hp_posterior, hp_prior, opts, ite, do_sort * do_sort: TRUE/FALSE:
# indicates whether the clusters should be sorted by size or not.  * ite: number
# of update iterations. OUTPUT: Updated parameters: list(free_energy,
# hp_posterior, q_of_z) DESCRIPTION: Updates the posterior of the mixture model.
# if do_sort=true it also sorts the cluster labels by size. (i.e. cluster 1 =
# largest cluster)


updatePosterior <- function(data, hp.posterior, hp.prior, opts, ite = Inf, do.sort = 1) {
    
    # Copyright (C) 2008-2012 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    epsilon <- 1e-10
    free.energy <- Inf
    i <- last.Nc <- start.sort <- 0
    opts.internal <- opts
    break.loop <- FALSE
    
    while (1) {
        i <- i + 1
        
        new.free.energy <- Inf
        cnt <- 0
        while (is.infinite(new.free.energy) && cnt <= 10) {
            
            templist <- mk.free.energy(data, hp.posterior, hp.prior, opts.internal)
            new.free.energy <- templist$free.energy
            log.lambda <- templist$log.lambda
            if (is.infinite(new.free.energy)) {
                warning("Free energy not finite: adding implicit noise.")
                opts.internal$implicitnoisevar <- opts.internal$implicitnoisevar + 
                0.1
            }
            cnt <- cnt + 1
        }
        
        if ((is.finite(ite) && i >= ite) || (is.infinite(ite) && free.energy.improved(free.energy, 
            new.free.energy, 0, opts.internal$threshold) == 0)) {
            free.energy <- new.free.energy
            if (do.sort && opts.internal$do.sort && (!start.sort) && is.finite(free.energy)) {
                start.sort <- 1
            } else break  # this will break the while loop
        }
        
        last.Nc <- hp.posterior$Nc
        free.energy <- new.free.energy
        qOFz <- mk.qOFz(data, hp.posterior, hp.prior, opts.internal, log.lambda)
        
        # if the last component is not 'empty' and max number components not reached add
        # a new empty component
        if (sum(qOFz[, ncol(qOFz)]) >= epsilon && ncol(qOFz) < opts$c.max) {
            qOFz <- cbind(qOFz, 0)
        }
        
        # Sort components by size (note: last component kept in its place)
        if (start.sort) {
            qOFz <- sortqofz(qOFz)
        }
        
        # Pick at most c.max+1 components, remove the smallest one, except the one added
        # in this iteration (ie. c.max + 1) FIXME: consider how to improve the
        # implementation regarding to this!  (3/2012) if (ncol(qOFz) == opts$c.max + 1) {
        # qOFz <- qOFz[, setdiff(seq_len(ncol(qOFz)), opts$c.max), drop = FALSE] }
        
        # If the smallest of the previous components (excluding the one added in this
        # interation) is empty then remove it (if new component was not added this is ok
        # to have as well)
        if (sum(qOFz[, ncol(qOFz) - 1]) < epsilon) {
            qOFz <- matrix(qOFz[, -(ncol(qOFz) - 1)], nrow(qOFz))
        }
        
        qOFz <- matrix(qOFz/rowSums(qOFz), nrow(qOFz))  # probabilities sum to one      
        hp.posterior <- mk.hp.posterior(data, qOFz, hp.prior, opts.internal)
        
    }
    
    list(free.energy = free.energy, hp.posterior = hp.posterior, qOFz = qOFz)
}


sumlogsumexp <- function(log.lambda) {
    .Call("vdpSumlogsumexp", log.lambda, PACKAGE = "netresponse")
}


softmax <- function(A) {
    qOFz <- .Call("vdpSoftmax", A, PACKAGE = "netresponse")
    
    # Ensure that this remains a matrix even if it has 1-dimension on rows or cols
    qOFz <- array(qOFz, dim = dim(A))
    
    # In rare (< 1 / 1e6?) cases, and particularly when sample size is large (>1500)
    # the vdpSoftmax function seems to produce NaNs. This occurred for particularly
    # low values of A: A[,1] in the range < -800 and below while other values in
    # A[,1] were at least -400 and mostly in range -100 ... 0; for A[,2] typical
    # range around -800, and for the NaN outlier: -1000. In both cases it was
    # A[209,1] and A[209,2] i.e. the same sample.  This is so rare that the effect on
    # the results is negligible, but this should be diagnosed and fixed asap. As this
    # is part of iteration, simply replace the defected sample with equal probability
    # in all groups.
    if (sum(!is.na(qOFz)) > 0) {
        # FIXME: optimize with apply!  Detect empty components and ignore
        inds <- c()
        for (i in seq_len(ncol(qOFz))) {
            if (sum(na.omit(qOFz[, i])) == 0) {
                inds <- c(inds, i)
                qOFz[, i] <- 0
            }
        }
        
        for (i in seq_len(nrow(qOFz))) {
            if (sum(is.na(qOFz[i, ])) > 0) {
                inds2 <- setdiff(seq(ncol(qOFz)), inds)
                qOFz[i, inds2] <- rep.int(1/length(inds2), length(inds2))
            }
        }
    }
    
    qOFz
    
}

mk.hp.posterior <- function(data, qOFz, hp.prior, opts) {
    
    # Copyright (C) 2008-2012 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    dat <- data$given.data$X1
    
    # Ensure that qOFz is a matrix
    qOFz <- matrix(qOFz, nrow(dat))
    
    # If qOFz exceeds max cluster size then remove one cluster (the second last one,
    # which assumes clusters are sorted and the last is a new one) FIXME 3/2012 quick
    # hack - consider better implementations regarding this
    if (ncol(qOFz) > opts$c.max + 1) {
        inds <- setdiff(seq_len(ncol(qOFz)), ncol(qOFz) - 1)
        qOFz <- matrix(qOFz[, inds], nrow(dat))
        qOFz <- matrix(qOFz/rowSums(qOFz), nrow(dat))
    }
    
    # Compatibility variables not needed for the current functionality
    tmp.realS <- X2 <- dimX2 <- 0
    
    out <- .Call("mHPpost", dat, ncol(dat), nrow(dat), X2, dimX2, tmp.realS, opts$implicitnoisevar, 
        hp.prior$Mumu, hp.prior$S2mu, hp.prior$AlphaKsi, hp.prior$BetaKsi, hp.prior$U.p, 
        hp.prior$alpha, qOFz, ncol(qOFz), PACKAGE = "netresponse")
    
    qOFz <- matrix(out$qOFz, nrow(qOFz))
    
    # if (ncol(qOFz) > opts$c.max) { }
    
    hp.posterior <- list(Mubar = matrix(out$Mubar, ncol(qOFz)), Mutilde = matrix(out$Mutilde, 
        ncol(qOFz)), KsiAlpha = matrix(out$KsiAlpha, ncol(qOFz)), KsiBeta = matrix(out$KsiBeta, 
        ncol(qOFz)), gamma = matrix(out$gamma, 2), Nc = out$Nc, qOFz = qOFz, Uhat = out$Uhat)
    
    hp.posterior
}


mk.hp.prior <- function(data, opts) {
    
    # Copyright (C) 2008-2012 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    # INPUT: data - matrix with data vectors opts - list with algorithm options
    # OUTPUT: hp_prior - list with prior information DESCRIPTION: Prior for the mean
    # = mean of data Prior for the variance = variance of data
    
    
    dat <- data$given.data$X1  # real-valued. Data to be clustered.
    
    Mean <- colMeans(dat)  # mean of each dimension
    Var <- colVariances(dat, Mean)  # Variance of each dimension
    # colSums((dat - rep(Mean, each = nrow(dat)))^2)/nrow(dat)
    
    # priors for distribution of codebook vectors Mu ~ N(MuMu, S2.Mu).. list(Mumu =
    # Mean, S2mu = Var, U.p = Inf) priors for data variance Ksi ~ invgam(AlphaKsi,
    # BetaKsi) variance is modeled with inverse Gamma distribution FIXME: some of
    # these are redundant, remove to save memory
    list(Mumu = Mean, S2mu = Var, U.p = Inf, AlphaKsi = rep(opts$prior.alphaKsi, 
        ncol(dat)), BetaKsi = rep(opts$prior.betaKsi, ncol(dat)), alpha = opts$prior.alpha)
    
}


######################################################################################## 


# INPUT: data, hp_posterior, hp_prior, opts OUTPUT: free_energy: value of mixture
# model's free energy log_lambda: Used for posterior of labels q_of_z <-
# softmax(log_lambda); DESCRIPTION: ...


mk.free.energy <- function(data, hp.posterior, hp.prior, opts, fc = NULL, log.lambda = NULL) {
    
    # Copyright (C) 2008-2010 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    if (is.null(fc) || is.null(log.lambda)) {
        fc <- mk.E.log.q.p.eta(data, hp.posterior, hp.prior, opts)  # 1*K
        log.lambda <- mk.log.lambda(data, hp.posterior, hp.prior, opts)  # N*K
    }
    
    hpgsum <- colSums(hp.posterior$gamma)
    dig <- digamma(hpgsum)
    
    E.log.p.of.V <- lgamma(hpgsum) - lgamma(1 + hp.prior$alpha) - colSums(lgamma(hp.posterior$gamma)) + 
        lgamma(hp.prior$alpha) + ((hp.posterior$gamma[1, ] - 1) * (digamma(hp.posterior$gamma[1, 
        ]) - dig)) + ((hp.posterior$gamma[2, ] - hp.prior$alpha) * (digamma(hp.posterior$gamma[2, 
        ]) - dig))
    
    extra.term <- sum(E.log.p.of.V)
    free.energy <- extra.term + sum(fc) - sumlogsumexp(log.lambda)
    
    # Return
    list(free.energy = free.energy, log.lambda = log.lambda)
    
}

# mk.free.energy.c <- cmpfun(mk.free.energy)

mk.qOFz <- function(data, hp.posterior, hp.prior, opts, log.lambda = NULL) {
    
    # Copyright (C) 2008-2010 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    if (is.null(log.lambda)) {
        log.lambda <- mk.log.lambda(data, hp.posterior, hp.prior, opts)
    }
    
    qOFz <- softmax(log.lambda)
    
    # Do not allow empty clusters
    as.matrix(qOFz[, !colSums(qOFz) == 0], nrow(log.lambda))
    
}



mk.log.lambda <- function(data, hp.posterior, hp.prior, opts) {
    
    
    # Copyright (C) 2008-2010 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    dat <- data$given.data$X1
    
    # Ensure matrix for data
    if (!is.matrix(dat)) {
        stop("Error in mk.log.lambda: dat should be a matrix!")
    }
    
    # Compatibility variables, not needed for current functionality
    tmp.realS <- X2 <- dimX2 <- 0
    
    out <- .Call("mLogLambda", dat, ncol(dat), nrow(dat), X2, dimX2, tmp.realS, opts$implicitnoisevar, 
        hp.prior, hp.posterior, PACKAGE = "netresponse")
    
    matrix(out, nrow(dat))
    
}


########################################################################### 


# INPUT: data, hp_posterior, hp_prior, opts OUTPUT: matrix [1xk]: used to compute
# the free_energy formula.  DESCRIPTION: Regards the gaussian model's parameters.



mk.E.log.q.p.eta <- function(data, hp.posterior, hp.prior, opts) {
    
    # Copyright (C) 2008-2010 Antonio Gusmao and Leo Lahti Licence: GPL >=2 This
    # function is based on the Variational Dirichlet Process Gaussian Mixture Model
    # implementation, Copyright (C) 2007 Kenichi Kurihara (all rights reserved) and
    # the Agglomerative Independent Variable Group Analysis package: Copyright (C)
    # 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
    # Valpola, and Paul Wagner
    
    # returns E [ log q(eta)/p(eta) ].q fc: 1 by k
    
    dat <- data$given.data$X1
    
    # Ensure matrix for data
    if (!is.matrix(dat)) {
        stop("Error in mk.E.log.q.p.eta: dat is not a matrix!")
    }
    
    N <- nrow(dat)
    M1 <- ncol(dat)
    K <- nrow(hp.posterior$Mubar)
    l.codebook <- (-M1/2) * matrix(1, K)
    Ksi.log <- (digamma(hp.posterior$KsiAlpha) - log(hp.posterior$KsiBeta))
    
    for (j in seq_len(M1)) {
        l.codebook <- l.codebook + 0.5 * (log(hp.prior$S2mu[[j]]/hp.posterior$Mutilde[, 
            j]) + ((hp.posterior$Mubar[, j] - hp.prior$Mumu[[j]])^2 + hp.posterior$Mutilde[, 
            j])/hp.prior$S2mu[[j]]) + lgamma(hp.prior$AlphaKsi[[j]]) - lgamma(hp.posterior$KsiAlpha[, 
            j]) + hp.posterior$KsiAlpha[, j] * log(hp.posterior$KsiBeta[, j]) - hp.prior$AlphaKsi[[j]] * 
            log(hp.prior$BetaKsi[[j]]) + (hp.posterior$KsiAlpha[, j] - hp.prior$AlphaKsi[[j]]) * 
            Ksi.log[, j] + (hp.prior$BetaKsi[[j]] - hp.posterior$KsiBeta[, j]) * 
            (hp.posterior$KsiAlpha[, j]/hp.posterior$KsiBeta[, j])
    }
    
    t(l.codebook)
}

# mk.E.log.q.p.eta.c <- cmpfun(mk.E.log.q.p.eta)

################################################# 


colVariances <- function(dat, Mean) {
    # This is about 5x faster than apply(dat, 2, var) max(abs(colVariances(dat) -
    # apply(dat,2,var)))
    colSums((dat - rep(Mean, each = nrow(dat)))^2)/(nrow(dat) - 1)
}

# colVariances.c <- cmpfun(colVariances)


#' @title split.qofz
#' @description Split q of z.
#' @details INPUT: data, qOFz, hp_posterior, hp_prior, opts OUTPUT: list(new.qOFz, new.c);
#' * new.qOFz: posterior over labels including the split clusters.  * new.c: index
#'  of the newly created cluster.  DESCRIPTION: Implements the VDP algorithm step
#'  3a.
#' @description Main function of the NetResponse algorithm. 
#' Detect condition-specific network responses, given
#' network and a set of measurements of node activity in a set of
#' conditions. Returns a set of subnetworks and their estimated
#' context-specific responses.
#' @param qOFz qOFz
#' @param c c
#' @param new.c new.c
#' @param dat dat
#' @param speedup speedup
#' @param min.size min.size
#' @return object
split.qofz <- function(qOFz, c, new.c, dat, speedup = TRUE, min.size = 4) {
    
    # compute the first principal component of the candidate cluster, not the whole
    # data set.
    
    # min.size option sets the required minimum size of a cluster for splitting;
    # smaller clusters are not splitted
    
    # Pick sample indices and samples corresponding to cluster c
    cluster_assignments <- apply(qOFz, 1, which.max)
    indices <- which(cluster_assignments == c)
    if (length(indices) < min.size) {
        #'Component must have at least min.size samples to be splitted.'
        # -> no splitting
        new.qOFz <- qOFz
    } else {
        
        component.data <- matrix(dat[indices, ], length(indices))
        
        # If the number of samples is high calculating PCA might take long but can be
        # approximated by using less samples:
        
        pcadata <- component.data
        
        if (speedup) {
            
            # when a candidate cluster, C, is split to generate two new clusters, it is split
            # by mapping the data onto the first principal component of the data in C and
            # then splitting that in half. To speed up, one can compute an approximate first
            # principal component by considering a randomly selected subset of the data
            # belonging to C, and computing its first principal component.
            
            # number of samples in this component
            ns <- nrow(component.data)
            nd <- ncol(component.data)
            
            # If component size exceeds cmax, use only a random subset of data to calculate
            # PCA size of the random subset increases slowly (linearly) with component size.
            cmax <- 20  #take at least this many random samples    
            nr <- min(ns, cmax + floor(sqrt(ns)))  # do not take more samples than are available
            rinds <- sample(ns, nr)
            
            # Pick random subset of the component data and accompanying indices to speed up
            # PCA calculations
            pcadata <- matrix(component.data[rinds, ], nrow = nr)
            indices <- indices[rinds]
        }
        
        # Split the cluster based on the first PCA component FIXME: compare speed with
        # other PCA implementations and select fastest
        dir <- prcomp(pcadata)$x[, 1]
        I1 <- indices[dir >= 0]
        I2 <- indices[dir < 0]
        
        # Initialize split by adding a zero column on qOFz
        
        # If one of qOFz clusters is empty, then do not create new clusters but instead
        # fill in the empty cluster during cluster split.  FIXME: ensure already in
        # creating qOFz-matrices that no zero columns are allowed. This will avoid the
        # need to address the issue here.  -> OK, done this. w remove this unnecessary
        # check here and test if the code works ok
        empty.cols <- (colSums(qOFz) == 0)
        if (!any(empty.cols)) {
            # no empty columns -> add an empty cluster
            new.qOFz <- array(0, dim = c(nrow(qOFz), ncol(qOFz) + 1))
            new.qOFz[, -new.c] <- qOFz
        } else {
            # an empty column -> no need to add new clusters
            new.qOFz <- qOFz
            new.c <- which(empty.cols)[[1]]
        }
        
        # Split this component (samples given in I1, I2) into two smaller components
        new.qOFz[I1, c] <- qOFz[I1, c]
        new.qOFz[I2, c] <- 0  # Remove entries from cluster c
        new.qOFz[I2, new.c] <- qOFz[I2, c]  # Add same entries to cluster new.c
    }
    
    new.qOFz
    
}

# split.qofz.c <- cmpfun(split.qofz)

############################################################################### 


# fsort <- function (df, sortvar, decreasing = FALSE) {
fsort <- function(df, sortvar) {
    
    o <- order(df[[sortvar]])
    # if (decreasing) {o <- rev(o)}
    df[o, ]
    
}

# fsort.c <- cmpfun(fsort)

################################################################## 

#' Write NetResponse results summary into a file.
#' 
#' Experimental version.
#' 
#' @usage write.netresponse.results(x, subnet.ids = NULL, filename)
#' @param x NetResponseModel
#' @param subnet.ids List of subnet ids to consider. By default, all subnets.
#' @param filename Output file name.
#' @return Used for side effects.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords utilities
write.netresponse.results <- function(x, subnet.ids = NULL, filename) {
    
    write("NetResponse - subnetworks", file = filename, append = FALSE)
    write("==========================\n", file = filename, append = TRUE)
    
    if (is.null(subnet.ids)) {
        subnet.ids <- names(x@subnets)
    }
    
    for (nam in subnet.ids) {
        write(nam, file = filename, append = TRUE)
        write(x@subnets[[nam]], file = filename, append = TRUE)
        write("----------------", file = filename, append = TRUE)
    }
    
}





#' get.model.parameters
#' 
#' Retrieve the mixture model parameters of the NetResponse algorithm for a
#' given subnetwork.
#' 
#' Only the non-empty components are returned. Note: the original data matrix
#' needs to be provided for function call separately.
#' 
#' @usage get.model.parameters(model, subnet.id = NULL)
#' @param model Result from NetResponse (detect.responses function).
#' @param subnet.id Subnet identifier. A natural number which specifies one of
#' the subnetworks within the 'model' object.
#' @return A list with the following elements: \item{mu}{ Centroids for the
#' mixture components. Components x nodes.} \item{sd}{ Standard deviations for
#' the mixture components. A vector over the nodes for each component, implying
#' the diagonal covariance matrix of the model (i.e. diag(std^2)). Components x
#' nodes} \item{w}{Vector of component weights.} \item{nodes}{List of nodes in
#' the subnetwork.} \item{K}{Number of mixture components.}
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references Leo Lahti et al.: Global modeling of transcriptional responses
#' in interaction networks. Bioinformatics (2010). See citation('netresponse')
#' for details.
#' @keywords utilities
#' @export
#' @examples
#' 
#' # Load toy data
#' data( toydata )          # Load toy data set
#' D     <- toydata$emat    # Response matrix (for example, gene expression)
#' model <- toydata$model   # Pre-calculated model
#' 
#' # Get model parameters for a given subnet
#' # (Gaussian mixture: mean, covariance diagonal, mixture proportions)
#' get.model.parameters(model, subnet.id = 1)
#' 
get.model.parameters <- function(model, subnet.id = NULL) {
    
    if (is(model) == "NetResponseModel") {
        
        if (is.numeric(subnet.id)) {
            warning("subnet.id given as numeric; converting to character: ", "Subnet-", 
                subnet.id, sep = "")
            subnet.id <- paste("Subnet", subnet.id, sep = "-")
        }
        
        pars <- model@models[[subnet.id]]
        pars[["nodes"]] <- model@subnets[[subnet.id]]
        
    } else {
        
        pars <- list()
        pars$mu <- model$model$posterior$centroids
        pars$sd <- model$model$posterior$sds
        pars$w <- model$model$posterior$weights
        
    }
    
    pars
    
}

#' @title Center data matrix.
#' @description Center data matrix to 0 for each variable by removing the means.
#' @param X The data set: samples x features. Each feature will be centered.
#' @param rm.na Ignore NAs.
#' @param meanvalue Can be used to set a desired center value. The default is 0.
#' @return Centered data matrix.
#' @note Note that the model assumes samples x features matrix, and centers
#' each feature.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords utilities maths
#' @examples centerData(matrix(rnorm(100), 10, 10))
#' @export
centerData <- function(X, rm.na = TRUE, meanvalue = NULL) {
    
    # Shift data matrix (columns) to zero, or given 'meanvalue'
    
    if (!rm.na) {
        xcenter <- colMeans(X)
        X2 <- X - rep(xcenter, rep.int(nrow(X), ncol(X)))
    } else {
        X2 <- array(NA, dim = c(nrow(X), ncol(X)), dimnames = dimnames(X))
        for (i in seq_len(ncol(X))) {
            x <- X[, i]
            nainds <- is.na(x)
            xmean <- mean(x[!nainds])
            X2[!nainds, i] <- x[!nainds] - xmean
        }
        dimnames(X2) <- dimnames(X)
    }
    
    if (!is.null(meanvalue)) {
        # Shift the data so that mean gets a specified value
        X2 <- X2 + meanvalue
    }
        
    X2
    
}

# The best academic advice I ever got was: 'Spend at least an hour every day on
# the manuscript closest to publication'.

#' @title generate.toydata
#' @description Generate toy data.
#' @usage D <- generate.toydata()
#' @param Dim Dimensionality of data
#' @param Nc Number of modes
#' @param Ns Number of data points
#' @param sd0 Component spread
#' @param rgam.shape Shape parameter for Gamma distribution 
#' @param rgam.scale Scale parameter for Gamma distribution
#' @return Simulated data matrix (samples x features)
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords internal
#' @examples D <- generate.toydata()
#' @export
generate.toydata <- function(Dim = 3, Nc = 3, Ns = 200, sd0 = 3, rgam.shape = 2, 
    rgam.scale = 2) {
    
    # Generate means and variances (covariance diagonals) for the components
    component.means <- matrix(rnorm(Nc * Dim, mean = 0, sd = sd0), nrow = Nc, ncol = Dim)
    component.vars <- matrix(1/rgamma(Nc * Dim, shape = rgam.shape, scale = rgam.scale), 
        nrow = Nc, ncol = Dim)
    component.sds <- sqrt(component.vars)
    
    # Size for each component -> sample randomly for each data point from uniform
    # distr.  i.e. cluster assignments
    sample2comp <- sample.int(Nc, Ns, replace = TRUE)
    D <- array(NA, dim = c(Ns, Dim))
    for (i in seq_len(Ns)) {
        # component identity of this sample
        ci <- sample2comp[[i]]
        cm <- component.means[ci, ]
        csd <- component.sds[ci, ]
        D[i, ] <- rnorm(Dim, mean = cm, sd = csd)
    }
    
    colnames(D) <- paste("Feature-", seq_len(ncol(D)), sep = "")
    rownames(D) <- paste("Sample-", seq_len(nrow(D)), sep = "")
    
    list(data = D, means = component.means, sds = component.sds, sample2comp = sample2comp)
}

#' @title response2sample
#' @description List the most strongly associated response of a given subnetwork for each sample.
#' @param model A NetResponseModel object or list. 
#' @param subnet.id Subnet id. A natural number which specifies one of the
#' subnetworks within the 'model' object.
#' @param component.list List samples separately for each mixture component
#' (TRUE). Else list the most strongly associated component for each sample
#' (FALSE).
#' @param verbose Follow progress by intermediate messages.
#' @param data Data (features x samples; or a vector for univariate case) to predict response for given data points (currently implemented only for mixture.model output)
#'
#' Return:
#' @return A list. Each element corresponds to one subnetwork response, and
#' contains a list of samples that are associated with the response (samples
#' for which this response has the highest probability P(response | sample)).
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references Leo Lahti et al.: Global modeling of transcriptional responses
#' in interaction networks. Bioinformatics (2010).  See citation('netresponse')
#' for citation details.
#' @keywords utilities
#' @export
#' @examples
#' # Load example data
#' data( toydata )         # Load toy data set
#' D    <- toydata$emat    # Response matrix (for example, gene expression)
#' model <- toydata$model  # Pre-calculated model
#' 
#' # Find the samples for each response (for a given subnetwork)
#' response2sample(model, subnet.id = 1)
#' 
response2sample <- function(model, subnet.id = NULL, component.list = TRUE, verbose = FALSE, 
    data = NULL) {
    
    if (is(model) == "NetResponseModel") {
        
        if (is.numeric(subnet.id)) {
            subnet.id <- paste("Subnet", subnet.id, sep = "-")
            warning("subnet.id given as numeric; converting to character: ", subnet.id, 
                sep = "")
        }
        
        response.probabilities <- model[[subnet.id]]$qofz  # sample2response(model, subnet.id)
        
        rownames(response.probabilities) <- rownames(model@datamatrix)
        
    } else if (is(model) == "list") {
        
        if (!is.null(model$qofz)) {
            # Pick response probabilities from the model object
            response.probabilities <- model$qofz
        } else {
            # Otherwise, retrieve the response probabilities assuming the input data and
            # parameters are presented on the same basis For mixture.model output
            if (is.vector(data)) {
                data2 <- t(matrix(data))
                colnames(data2) <- names(data)
                rownames(data2) <- "Mode-1"
                data <- data2
                data2 <- NULL
            }
            
            # Find cluster for each sample
            response.probabilities <- P.r.s(data, model$params, log = TRUE)
            # rownames(response.probabilities) <- colnames(data)
            colnames(response.probabilities) <- paste("Mode", seq_len(ncol(response.probabilities)), 
                sep = "-")
        }
    }
    
    # For each sample, list the most strongly associated response (highest P(r|s))
    clusters <- apply(response.probabilities, 1, which.max)
    
    if (length(clusters) == 0) {
        warning(paste("Error with response2sample in subnet", subnet.id))
    } else if (component.list) {
        # list samples separately for each cluster
        clusters <- lapply(seq(max(clusters)), function(i) {
            if (is.null(names(clusters))) {
                which(clusters == i)
            } else {
                names(which(clusters == i))
            }
        })
        
        names(clusters) <- paste("Mode-", seq_len(length(clusters)), sep = "")
        if (length(clusters) < ncol(response.probabilities)) {
            n <- ncol(response.probabilities) - length(clusters)
            clusters <- c(clusters, vector(n, mode = "list"))
        }
    }
    
    clusters
    
}

# 'To invent, you need a good imagination and a pile of junk.'  -- Thomas Edison

#' @title Continuous responses
#' @description Quantify association between modes and continuous variable
#' @param annotation.vector annotation vector with discrete factor levels, and named by the samples
#' @param model NetResponse model object
#' @param method method for enrichment calculation
#' @param min.size minimum sample size for a response 
#' @param data data matrix (samples x features)
#' @return List with each element corresponding to one variable and listing the responses according to association strength
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @export
#' @keywords utilities
#' @examples res <- continuous.responses(annotation.vector = NULL, model = NULL)
continuous.responses <- function(annotation.vector, model, method = "t-test", min.size = 2,
    data = NULL) {

    if (is.null(model)) {return(NULL)}
    
    # method = 't-test'; min.size = 2; data = t(dat[, gpt])
    
    if (is.null(data) && is(model) == "NetResponseModel") {
        data <- model@datamatrix
        all.samples <- rownames(data)
    }
    
    # samples x features
    if (is.vector(data)) {
        data2 <- matrix(data)
        rownames(data2) <- names(data)
        data <- data2
        all.samples <- rownames(data)
    }
    
    if (is(model) == "NetResponseModel") {
        models <- get.subnets(model, min.size = min.size)
    } else {
        models <- model
    }
    
    associations <- enrichment.list(models, annotation.vector)
    
    associations
    
}

# associations <- continuous.responses.single(model, annotation.vector, method =
# 't.test')
continuous.responses.single <- function(model, annotation.vector, method = "t.test") {
    
    annotated.samples <- names(which(!is.na(annotation.vector)))
    annotation.data <- annotation.vector[annotated.samples]
    names(annotation.data) <- annotated.samples
    
    
    r2s <- response2sample(model)
    
    if (length(r2s) <= 1) 
        {
            return(NA)
        }  # No multimodality -> no enrichments
    
    pvals <- c()
    fold.change <- c()
    
    for (mo in seq_len(length(r2s))) {
        
        # annotated samples in the mode
        s <- intersect(r2s[[mo]], annotated.samples)
        
        # annotated samples in other modes
        sc <- intersect(unlist(r2s[-mo]), annotated.samples)
        
        if (length(na.omit(s)) > 1 && length(na.omit(sc)) > 1) {
            if (method == "t.test") {
                pval <- t.test(annotation.data[s], annotation.data[sc])$p.value
            }
            
            pvals[[mo]] <- pval
            
            fold.change[[mo]] <- mean(annotation.data[s]) -
            mean(annotation.data[sc])
        } else {
            
            warning(paste("Not enough annotated observations 
                to calculate p-values", mo))
        
            pvals[[mo]] <- NA
        
            fold.change[[mo]] <- NA
        }
    }
    
    associations <- data.frame(list(mode = seq_len(length(r2s)),
        pvalue = pvals, fold.change = fold.change))
    
    associations
    
}

#' @title Association strength between category labels and responses.
#' @description Plot association strength between user-defined category
#' labels and responses in a selected subnetwork. Associations are showm
#' in terms -log10(p) enrichment values for the annotation categories for the
#' responses within the specified subnetwork. No correction for multiple
#' testing. 
#' @param x NetResponseModel object
#' @param subnet.id Subnetwork.
#' @param labels Factor. Labels for the data samples. Name by samples, or
#' provide in the same order as in the original data.
#' @param method Method to calculate association strength.
#' @param mode group.by.responses or group.by.classes: indicate barplot
#' grouping type.
#' @param ... Other arguments to be passed for plot_
#' @return Used for side effect (plotting).
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @seealso plot_responses
#' @references See citation('netresponse').
#' @keywords utilities
#' @examples #
plot_associations <- function(x, subnet.id, labels, method = "hypergeometric", mode = "group.by.classes", 
    ...) {
    
    # assumes that labels are in same order as data if names not given
    names(labels) <- rownames(x@datamatrix)
    
    # Number of responses for this subnet
    N.responses <- length(x@models[[subnet.id]]$w)
    
    association.tab <- matrix(NA, nrow = length(unique(labels)), ncol = N.responses)
    rownames(association.tab) <- as.character(unique(labels))
    colnames(association.tab) <- names(x@models[[subnet.id]]$w)
    
    for (lab in unique(labels)) {
        # Pick samples associated with this annotation group
        samples <- names(labels)[which(labels == lab)]
        # Get enrichment information o <- order.responses(x, samples, method =
        # 'hypergeometric')$ordered.responses # for all responses
        
        enr <- list()
        for (response in seq_len(N.responses)) {
            
            enr[[response]] <- response.enrichment(x[[subnet.id]]$qofz,
                samples, 
                response, method)
        }
        neg.log.p <- -log10(sapply(enr, function(tab) {
            tab$info["pvalue"]
        }))
        association.tab[as.character(lab), ] <- unname(neg.log.p)
        
    }
    
    if (mode == "group.by.responses") {
        barplot(association.tab, beside = TRUE, legend = TRUE,
        las = 1, cex.names = 0.8, 
            main = "Label/response associations", ylab = "-log10(p)", ...)
    } else if (mode == "group.by.classes") {
        barplot(t(association.tab), beside = TRUE, legend = TRUE,
            las = 1, cex.names = 0.8, 
            main = "Label/response associations", ylab = "-log10(p)", ...)
    }
    
}




#' @title plotPCA
#' @description Visualize data, centroids and response confidence intervals
#' for a given
#' subnetwork with PCA. Optionally, color the samples according to annotations
#' labels.
#' @param x NetResponseModel object. Output from the detect.responses function.
#' @param subnet.id Subnetwork id. Either character as 'Subnetwork-2' or
#' numeric as 2, which is then converted to character.
#' @param labels Optional: sample class labels to be indicated in colors.
#' @param confidence Confidence interval for the responses based on the
#' covariances of each response. If NULL, no plotting.
#' @param npoints Argument to the ellipse function
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @examples #plotPCA(x, subnet.id)
plotPCA <- function(x, subnet.id, labels = NULL, confidence = 0.95, npoints=NULL, ...) {
    
    if (!is.null(labels)) {
        if (is.null(names(labels))) {
            names(labels) <- rownames(x@datamatrix)
        }
    }
    
    if (is.numeric(subnet.id)) {
        subnet.id <- paste("Subnet", subnet.id, sep = "-")
        warning("subnet.id given as numeric; converting to character: ",
        "Subnet-", 
            subnet.id, sep = "")
    }
    
    # model parameters
    m <- get.model.parameters(x, subnet.id)
    
    # pick data (subnet only)
    dat <- t(get.dat(x, subnet.id))
    
    # Add cluster centroids
    dat2 <- rbind(dat, m$mu)
    
    # center the data
    dat.mean <- colMeans(dat)
    dat.centered <- t(t(dat2) - dat.mean)
    
    # PCA, two principal components
    pca <- princomp(dat.centered)
    
    # projection plane
    v <- as.matrix(pca$loadings[, seq_len(2)])
    
    # Projected centroids (in PC space) for the detected components
    dat.pca <- dat.centered %*% v
    
    nlab <- length(unique(labels))
    if (nlab > 1) {
        my.palette <- palette(rainbow(nlab))
        cols <- my.palette[labels]
    } else {
        cols <- "black"
    }
    
    plot(dat.pca[seq_len(nrow(dat)), ], main = paste("PCA plot: subnetwork ",
        subnet.id, 
        sep = ""), xaxt = "n", yaxt = "n", xlab = "", ylab = "",
        col = cols, pch = 19)
    
    if (nlab > 1) {
        legend("topleft", legend = as.character(unique(labels)),
        fill = unique(cols))
    }
    
    for (ri in seq_len(nrow(m$mu))) {
        
        # Estimated covariance matrix for the response
        cmat <- diag(m$sd[ri, ]^2)
        
        # Projection of the covariance matrix in the PCA projection space
        # force it
        # diagonal as it should be
        cmat.projection <- diag(diag(t(v) %*% cmat %*% v))
        
        # Indicate estimated responses by ellipses
        if (!is.null(confidence)) {
            #add.ellipse(centroid = dat.pca[ri, ],
            # covmat = cmat.projection, confidence = confidence)
            centroid <- dat.pca[ri, ]
            covmat <- cmat.projection
            # add ellipse to a plot
            el <- ellipse(centroid, covmat, confidence, npoints)
            points(el, type = "l", col = col, ...)
        }
    }
}



#' @title PlotMixtureBivariate
#' @description Visualize data, centroids and response confidence intervals
#' for a given
#' Gaussian mixture model in two-dimensional (bivariate) case. Optionally, 
#' color the samples according to annotations labels.
#' @param x data matrix (samples x features)
#' @param means mode centroids (modes x features)
#' @param sds mode standard deviations, assuming diagonal covariance 
#'    matrices (modes x features, each row giving the sqrt of covariance 
#'    diagonal for the corresponding mode)
#' @param ws weight for each mode
#' @param labels Optional: sample class labels to be indicated in colors.
#' @param confidence Confidence interval for the responses based on the
#' covariances of each response. If NULL, no plotting.
#' @param main title text
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @examples #plotMixture(dat, means, sds, ws)
PlotMixtureBivariate <- function(x, means, sds, ws, labels = NULL, confidence = 0.95, 
    main = "", ...) {
    
    if (!is.null(labels)) {
        if (is.null(names(labels))) {
            names(labels) <- rownames(x)
        }
    }
    
    # Add cluster centroids
    dat.proj <- rbind(x, means)
    
    nlab <- length(unique(labels))
    if (nlab > 1) {
        my.palette <- palette(rainbow(nlab))
        cols <- my.palette[labels]
    } else {
        cols <- "black"
    }
    
    plot(dat.proj[seq_len(nrow(x)), ], main = main, xaxt = "n", yaxt = "n",
        xlab = "", 
        ylab = "", col = cols, pch = 19)
    
    if (nlab > 1) {
        legend("topleft", legend = as.character(unique(labels)),
        fill = unique(cols))
    }
    
    for (ri in seq_len(nrow(means))) {
        
        # Estimated covariance matrix for the response
        cmat <- diag(sds[ri, ]^2)
        
        # Indicate estimated responses by ellipses
        #if (!is.null(confidence)) {
        #    add.ellipse(centroid = dat.proj[ri, ], covmat = cmat, confidence = confidence)
        #}
    }
}

#' @title PlotMixtureMultivariate
#' @description Visualize data, centroids and response confidence intervals for a given
#' Gaussian mixture model with PCA. Optionally, color the samples according 
#' to annotations labels.
#' @param x data matrix (samples x features)
#' @param means mode centroids (modes x features)
#' @param sds mode standard deviations, assuming diagonal covariance matrices 
#'        (modes x features, each row giving the sqrt of covariance diagonal 
#'   for the corresponding mode)
#' @param ws weight for each mode
#' @param labels Optional: sample class labels to be indicated in colors.
#' @param title title
#' @param modes Optional: provide sample modes for visualization already in 
#'   the input
#' @param pca The data is projected on PCA plane by default (pca = TRUE). 
#'   By setting this off (pca = FALSE) it is possible to visualize 
#'   two-dimensional data in the original domain.
#' @param qofz Sample-response probabilistic assignments matrix 
#'   (samples x responses)
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @examples #plotMixture(dat, means, sds, ws)
PlotMixtureMultivariate <- function(x, means, sds, ws, labels = NULL, title = NULL, 
    modes = NULL, pca = FALSE, qofz = NULL, ...) {
    
    # Circumvent warnings
    Comp.1 <- Comp.2 <- dat2 <- NULL
    
    if (!is.null(labels) && is.null(names(labels))) {
        names(labels) <- rownames(x)
    }
    
    if (pca || ncol(x) > 2) {
        
        # center the data
        dat.mean <- colMeans(x)
        dat.centered <- t(t(x) - dat.mean)
        
        # PCA, two principal components
        pca <- princomp(dat.centered)
        
        # projection plane
        v <- as.matrix(pca$loadings[, seq_len(2)])
        
        # Projected centroids (in PC space) for the detected components
        dat.pca <- dat.centered %*% v
        
        pld <- dat.pca
        
        xtitle <- "PCA1"
        ytitle <- "PCA2"
        
        if (is.null(title)) {
            title <- paste("PCA (", ncol(x), " features)", sep = "")
        }
        
    } else {
        
        pld <- x
        xtitle <- colnames(x)[[1]]
        ytitle <- colnames(x)[[2]]
        if (is.null(title)) {
            title <- "Cross-plot"
        }
        
    }
    
    nlab <- length(unique(labels))
    if (nlab > 1) {
        my.palette <- palette(rainbow(nlab))
        cols <- my.palette[labels]
    } else {
        cols <- "black"
    }
    
    
    if (is.null(modes)) {
        
        # Determine the most likely cluster for each sample (-> hard clusters)
        if (is.null(qofz)) {
            qofz <- P.r.s(t(x), list(mu = means, sd = sds, w = ws), log = TRUE)
            rownames(qofz) <- rownames(x)
            colnames(qofz) <- paste("Mode", seq_len(ncol(qofz)), sep = "-")
        }
        modes <- paste("Mode", apply(qofz, 1, which.max), sep = "-")
        
    }
    
    # Form data.fframe
    df <- data.frame(list(Comp.1 = pld[, 1], Comp.2 = pld[, 2]))
    df$mode <- factor(modes)
    
    theme_set(theme_bw(15))
    p <- ggplot(df, aes(x = Comp.1, y = Comp.2, colour = mode)) + geom_point()
    p <- p + ggtitle(title) + xlab(xtitle) + ylab(ytitle)
    print(p)
    
    p
    
}

#' @title Plot observed data. 
#' @description Plotting tool for measurement data.
#' Produces boxplot for each feature in each annotation category for the
#' selected subnetwork.
#' @param x NetResponseModel object.
#' @param subnet.id Specify the subnetwork.
#' @param labels Annotation categories.
#' @param ... Further arguments for plot function.
#' @return ggplot2 plot object
#' @author Leo Lahti <leo.lahti@@iki.fi>
#' @seealso plot_responses
#' @references See citation('netresponse')
#' @keywords utilities
#' @examples #
plot_data <- function(x, subnet.id, labels, ...) {
    
    # ggplot2 boxplots for each user-defined sample category (listed in labels)
    dat <- t(get.dat(x, subnet.id))  # samples x nodes
    df <- data.frame(list(labels = labels, dat))
    dfm <- melt(df, id = "labels")
    p <- ggplot(dfm) + aes(x = labels, y = dfm$value) + facet_wrap(~variable)
    p <- p + geom_boxplot()
    p <- p + ggtitle(paste(subnet.id, ": annotation boxplot", sep = ""))
    print(p)
    p
    
}



#' @title plot_expression
#' @description Plot expression matrix in color scale. For one-channel data;
#' plot expression
#' of each gene relative to its mean expression level over all samples. Blue
#' indicates decreased expression and red indicates increased expression.
#' Brightness of the color indicates magnitude of the change. Black denotes no
#' change.
#' @param x samples x features matrix
#' @param maintext main title
#' @param ... optional arguments
#' @return Used for its side effects.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @seealso \code{\link{plot_scale}}
#' @references See citation('netresponse').
#' @keywords utilities
#' @examples #plot_expression(x)
plot_expression <- function(x, maintext, ...) {
    # was: plot_matrix
    
    # set color breakpoints and palette
    mybreaks <- set.breaks(1, interval = 0.02)
    mypalette <- colorRampPalette(c("blue", "black", "red"), space = "rgb")
    
    # compute differential expression in nodes with respect to the mean expression
    # level for each gene
    ctrl.state <- colMeans(x)
    dmat <- t(t(x) - ctrl.state)
    
    # Color plot of the whole expression matrix, ordered by responses
    tmp <- plot_matrix(dmat, mybreaks = mybreaks, maintext = maintext,
        cexlab = 1, 
        mypalette = mypalette)
}


#' @title plot_subnet
#' @description Plot the given subnetwork.
#' @param x Result from NetResponse (detect.responses function).
#' @param subnet.id Subnet id.
#' @param network Original network used in the modelling.
#' @param plot_names Plot node names (TRUE) or indices (FALSE).
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects. Returns a matrix that describes the
#' investigated subnetwork.
#' @author Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao. Maintainer:
#' Leo Lahti <leo.lahti@@iki.fi>
#' @references L. Lahti et al.: Global modeling of transcriptional responses in
#' interaction networks. Submitted.
#' @keywords utilities
#' @export
#' @examples #
#' # res <- detect.responses(D, netw, verbose = FALSE)
#' # net <- plot_subnet(res, subnet.idx = 1)
plot_subnet <- function(x, subnet.id, network, plot_names = TRUE, ...) {
    
    if (is.numeric(subnet.id)) {
        subnet.id <- paste("Subnet", subnet.id, sep = "-")
        warning("subnet.id given as numeric; converting to character: ",
        "Subnet-", 
            subnet.id, sep = "")
    }
    
    subnet.nodes <- get.subnets(x)[[subnet.id]]
    mynet <- network[subnet.nodes, subnet.nodes]
    
    tmp <- plot_response(x = NULL, mynet, mybreaks = NULL, mypalette = NULL,
        colors = FALSE, 
        maintext = subnet.id)
    
    mynet
    
}




#' @title plot_response
#' @description Plot a specific transcriptional response for a given subnetwork.
#' TRUE, colors = TRUE, plot_type = 'twopi', ...)
#' @param x A numerical vector, or NULL.
#' @param mynet Binary matrix specifying the interactions between nodes.
#' @param mybreaks Specify breakpoints for color plot_
#' @param mypalette Specify palette for color plot_
#' @param plot_names Plot node names (TRUE) or indices (FALSE).
#' @param colors Plot colors. Logical.
#' @param plot_type Network plot mode. For instance, 'neato' or 'twopi'.
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects.
#' @author Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao. Maintainer:
#' Leo Lahti <leo.lahti@@iki.fi>
#' @references L. Lahti et al.: Global modeling of transcriptional responses in
#' interaction networks. Submitted.
#' @keywords utilities
#' @export
#' @examples 
#' #tmp <- plot_response(model, mynet, 
#' #  \tmaintext = paste('Subnetwork', subnet.id))
plot_response <- function(x, mynet, mybreaks, mypalette, plot_names = TRUE, colors = TRUE, 
    plot_type = "twopi", ...) {
    
    check.bins <- function(difexp, mybreaks) {
        
        # check color scale bin for each expression value
        bins <- c()
        for (i in seq_len(length(difexp))) {
            # which color bins are smaller than our difexp value (for probet: i, mode:mode)
            inds <- which(difexp[[i]] > mybreaks)
            if (length(inds) == 0) {
                bins[[i]] <- 1
            } else if (length(inds) > 0) {
                bins[[i]] <- max(inds) + 1
            }
        }
        
        bins
    }
    
    
    # Add node color for specific nodes
    nAttrs <- list()
    if (colors) {
        bins <- check.bins(x, mybreaks)
        nAttrs$fillcolor <- mypalette(length(mybreaks) + 1)[bins]
    } else {
        nAttrs$fillcolor <- rep("white", nrow(mynet))
    }
    names(nAttrs$fillcolor) <- rownames(mynet)
    
    # add node names for all nodes
    if (plot_names) {
        nodenames <- rownames(mynet)
    } else {
        nodenames <- rep("", nrow(mynet))
    }
    
    nAttrs$label <- nodenames
    names(nAttrs$label) <- rownames(mynet)
    
    myg <- as(new("graphAM", mynet, "undirected"), "graphNEL")
    
    plot(myg, y = plot_type, nodeAttrs = nAttrs, ...)
    
}





#' @title plot_responses
#' @description Plot the detected transcriptional responses for a given
#' subnetwork.
#' plot_mode = 'network', xaxis = TRUE, yaxis = TRUE, plot_type = 'twopi', mar
#' = c(5, 4, 4, 2), horiz = TRUE, datamatrix = NULL, scale = FALSE, ...)
#' @param x Result from NetResponse (detect.responses function).
#' @param subnet.id Subnet id.
#' @param nc Number of columns for an array of images.
#' @param plot_names Plot node names (TRUE) or indices (FALSE).
#' @param plot_mode network: plot responses as a subnetwork graph; matrix,
#' heatmap: plot subnetwork expression matrix. For both, expression of each
#' gene is shown relative to the mean expression level of the gene;
#' boxplot_data: feature-wise boxplots for hard sample-to-response assignments;
#' response.barplot: estimated response centroids as barplot including 95%
#' confidence intervals for the means; pca: PCA projection with estimated
#' centroids and 95% intervals. In 1-dimensional case a histogram is drawn. In
#' two-dimensional case the original coordinates are used.
#' @param xaxis,yaxis Logical. Plot row/column names.
#' @param plot_type Network plot mode. For instance, 'neato' or 'twopi'.
#' @param mar Figure margins.
#' @param horiz Logical. Horizontal barplot_
#' @param datamatrix datamatrix
#' @param scale scale the phylotypes to unit length (only implemented for
#' plot_mode = 'matrix'
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @seealso \code{\link{plot_scale}}
#' @references See citation('netresponse')
#' @keywords utilities
#' @export
#' @examples #
#' #res <- detect.responses(D, netw)
#' #vis <- plot_responses(res, subnet.id)
plot_responses <- function(x, subnet.id, nc = 3, plot_names = TRUE, plot_mode = "network", 
    xaxis = TRUE, yaxis = TRUE, plot_type = "twopi", mar = c(5, 4, 4, 2), horiz = TRUE, 
    datamatrix = NULL, scale = FALSE, ...) {
    
    responses <- NULL
    variable <- NULL
    p <- NULL  # return ggplot2 object if available
    
    if (is.null(datamatrix)) {
        datamatrix <- x@datamatrix
    }
    
    value <- tmp <- NULL
    
    if (is.numeric(subnet.id)) {
        subnet.id <- paste("Subnet", subnet.id, sep = "-")
    warning("subnet.id given as numeric; converting to character: ", "Subnet-", 
            subnet.id, sep = "")
    }
    
    pars <- get.model.parameters(x, subnet.id)
    subnet.nodes <- get.subnets(x)[[subnet.id]]
    datamatrix <- datamatrix[, subnet.nodes]
    
    if (class(x@network) %in% c("graphNEL", "igraph")) {
        # FIXME speedup by just converting the correct subset, not whole matrix
        mynet <- as(x@network, "graphAM")@adjMat
        rownames(mynet) <- colnames(mynet) <- nodes(x@network)
        mynet <- mynet[subnet.nodes, subnet.nodes]
    } else {
        mynet <- x@network[subnet.nodes, subnet.nodes]
    }
    
    # Set color breakpoints and palette
    mybreaks <- set.breaks(1, interval = 0.02)
    mypalette <- colorRampPalette(c("blue", "white", "red"), space = "rgb")
    
    # compute differential expression in nodes with respect to the mean
    # expression
    # level for each gene
    ctrl.state <- colMeans(datamatrix)
    centroids <- t(pars$mu)
    difexp <- apply(centroids, 2, function(x) {
        x - ctrl.state
    })
    rownames(difexp) <- rownames(centroids)
    
    # Determine the most likely mode for each sample (-> hard clusters)
    qofz <- x@models[[subnet.id]]$qofz
    modes <- apply(qofz, 1, which.max)
    
    if (plot_mode == "network") {
        par(mfrow = c(ceiling(length(pars$w)/nc), nc))
        for (comp in seq_len(length(pars$w))) {
            tmp <- plot_response(difexp[, comp], mynet, mybreaks,
                mypalette, plot_names, 
                main = paste(subnet.id, "/Response-", comp, sep = ""),
                plot_type = plot_type, 
                ...)
        }
    } else if (plot_mode == "matrix" || plot_mode == "heatmap") {
        
        # order samples according to responses
        s2r <- apply(x[[subnet.id]]$qofz, 1, which.max)
        ordered.samples <- order(s2r)
        
        dmat <- datamatrix[ordered.samples, subnet.nodes]
        dmat <- t(t(dmat) - ctrl.state)
        
        if (scale) {
            dmat <- scale(dmat, center = FALSE, scale = TRUE)
        }
        
        par(mfrow = c())
        
        # Color plot of the whole expression matrix, ordered by responses
        if (horiz) {
            tmp <- plot_matrix(t(dmat), mybreaks = mybreaks,
                maintext = subnet.id, 
                xlab = NULL, ylab = NULL, palette = mypalette,
                xaxis = yaxis, yaxis = xaxis, 
                mar = mar, ...)
        } else {
            tmp <- plot_matrix(dmat, mybreaks = mybreaks,
                maintext = subnet.id, xlab = NULL, 
                ylab = NULL, palette = mypalette,
                xaxis = xaxis, yaxis = yaxis, mar = mar, 
                ...)
        }
        
    } else if (plot_mode == "boxplot_data") {
        
        s2r <- apply(x[[subnet.id]]$qofz, 1, which.max)
        label <- factor(s2r)
        
        # Ggplot2 boxplot handy as determines the grid size
        # automatically List samples in
        # each response (hard assignments)
        dat <- t(get.dat(x, subnet.id))  # samples x nodes
        df <- data.frame(list(responses = label, dat))
        dfm <- melt(df, id = "responses")
        
        p <- ggplot(dfm) + aes(x = responses, y = value) + facet_wrap(~variable)
        p <- p + geom_boxplot()
        p <- p + ggtitle(paste(subnet.id, ": response boxplot", sep = ""))
        
        print(p)
        
    } else if (plot_mode == "response.barplot") {
        
        # FIXME: does not work
        
        # Plot cross-bars for estimated means and 95% intervals for each
        # response for
        # each node
        m <- get.model.parameters(x, subnet.id)
        
        # Use 1.96*std of the mean (95% quantile) for error limits
        # using the soft assignment sum as the sample size for each
        # cluster FIXME: use directly the parametric estimates from
        # the model?  NOTE: that wouldn't work with pca.basis version
        # directly
        
        # note: reverse the groups since also sds is reversed
        s2r <- apply(x[[subnet.id]]$qofz, 1, which.max)
        
        dat <- t(get.dat(x, subnet.id))  # samples x nodes
        response <- factor(s2r)
        # factor(apply(sample2response(x, subnet.id), 1, which.max))
        df <- data.frame(dat)
        df$response <- response
        dfm <- melt(df, id.var = "response")
        
        ggplot(dfm) + aes(x = response, y = value, fill = response) +
            facet_wrap(~variable) + 
            geom_bar(stat = "identity")
        
        # mean and std of mean
        df <- ddply(dfm, c("response", "variable"), function(dd) {
            c(mean = mean(dd$value),
                sd = 1.96 * sd(dd$value)/sqrt(sum(x[[subnet.id]]$qofz[, 
                dd$response])))
        })
        
        p <- qplot(response, mean, fill = variable,
            data = df, geom = "bar", position = "dodge")
        p <- p + geom_errorbar(aes(ymax = mean + sd, ymin = mean - sd),
            position = "dodge") + 
            theme_bw()
        
        print(p)
        
        
    } else if (plot_mode == "pca") {
        
        dmat <- datamatrix[, subnet.nodes]
        
        if (length(subnet.nodes) == 1) {
            
            p <- PlotMixtureUnivariate(dmat, modes = modes,
            title.text = subnet.id, 
            xlab.text = "Component score",
            ylab.text = "Frequency", binwidth = 0.1)
            
            
        } else {
            
            if (ncol(dmat) > 2) {
                pca <- TRUE
            } else {
                pca <- FALSE
            }
            
            p <- PlotMixtureMultivariate(dmat, modes = modes,
            title = subnet.id, pca = pca)
            
        }
        
    }
    
    list(breaks = mybreaks, palette = mypalette, info = tmp, p = p)
    
}



#' @title plot_scale
#' @description Plot the color scale used in visualization.
#' @param x Breakpoints for the plot_
#' @param y Color palette.
#' @param m Breakpoints' upper limit.
#' @param cex.axis Axis scale.
#' @param label.step Density of the labels.
#' @param interval Interval.
#' @param two.sided Plot two-sided (TRUE) or one-sided (FALSE) visualization.
#' @param label.start Label starting point.
#' @param Nlab Number of labels to plot_
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects.
#' @author Leo Lahti <leo.lahti@@iki.fi>
#' @references See citation('netresponse')
#' @keywords utilities
#' @examples #
#'   #res <- detect.responses(D, netw, verbose = FALSE)
#'   #vis <- plot_responses(res, subnet.idx)
#'   #plot_scale(vis$breaks, vis$palette)
plot_scale <- function(x, y, m = NULL, cex.axis = 1.5, label.step = 2, interval = 0.1, 
    two.sided = TRUE, label.start = NULL, Nlab = 3, ...) {
    
    if (two.sided) {
        
        if (length(m) > 0) {
            x <- set.breaks(m, interval)
        } else {
            mm <- max(x[-c(1, length(x))])
            m <- mm - interval/2
        }
        
        image(t(as.matrix(seq(-mm, mm, length = 100))),
            col = y(length(x) - 1), xaxt = "n", 
            yaxt = "n", zlim = range(x), breaks = x)
        
        ndigits <- nchar(unlist(strsplit(as.character(mm), "\\."))[[2]])
        digit.step <- 10^(-ndigits)
        labs <- seq(-mm, mm, by = digit.step)
        label.start <- -max(abs(round(labs, ndigits)))
        start.position <- match(-label.start, round(labs, ndigits))
        end.position <- match(label.start, round(labs, ndigits))
        inds <- seq(start.position, end.position, length = Nlab)
        
        axis(2, at = inds/length(labs),
            labels = labs[inds], cex.axis = cex.axis, 
            las = 2)
    }
    
    if (!two.sided) {
        
        mm <- max(x) + 1e+06  # infty
        m <- max(x)
        
        labs <- seq(0, m, label.step)
        inds <- sapply(labs, function(lab) {
            min(which(lab <= x))
        })
        
        image(t(as.matrix(seq(0, m, length = 100))),
        col = y(length(x) - 1), xaxt = "n", 
            yaxt = "n", zlim = range(x), breaks = x)
        
        int <- 1/(length(x) - 1)
        axis(2, at = seq(0, 1, by = int)[inds],
        labels = labs, cex.axis = cex.axis, 
            las = 2)
    }
    
}

# 1. vdp.mixt: moodien loytyminen eri dimensiolla, naytemaarilla ja komponenteilla
#   -> ainakin nopea check

#######################################################################

# Generate random data from five Gaussians. 
# Detect modes with vdp-gm. 
# Plot data points and detected clusters with variance ellipses

#######################################################################

library(netresponse)
#source("~/Rpackages/netresponse/netresponse/R/detect.responses.R")
#source("~/Rpackages/netresponse/netresponse/R/internals.R")
#source("~/Rpackages/netresponse/netresponse/R/vdp.mixt.R")
#dyn.load("/home/tuli/Rpackages/netresponse/netresponse/src/netresponse.so")

#########  Generate DATA #############################################

# Generate Nc components from normal-inverseGamma prior

set.seed(12346)

dd <- 3   # Dimensionality of data
Nc <- 5   # Number of components
Ns <- 200 # Number of data points
sd0 <- 3  # component spread
rgam.shape = 2 # parameters for Gamma distribution 
rgam.scale = 2 # parameters for Gamma distribution to define precisions


# Generate means and variances (covariance diagonals) for the components 
component.means <- matrix(rnorm(Nc*dd, mean = 0, sd = sd0), nrow = Nc, ncol = dd)
component.vars <- matrix(1/rgamma(Nc*dd, shape = rgam.shape, scale = rgam.scale), 
	                 nrow = Nc, ncol = dd)
component.sds <- sqrt(component.vars)


# Size for each component -> sample randomly for each data point from uniform distr.
# i.e. cluster assignments
sample2comp <- sample.int(Nc, Ns, replace = TRUE)

D <- array(NA, dim = c(Ns, dd))
for (i in 1:Ns)  {
    # component identity of this sample
    ci <- sample2comp[[i]]
    cm <- component.means[ci,]
    csd <- component.sds[ci,]
    D[i,] <- rnorm(dd, mean = cm, sd = csd)
}


######################################################################

# Fit mixture model
out <- mixture.model(D, mixture.method = "bic")

# FIXME rowmeans(qofz) is constant but not 1
#qofz <- P.r.s(t(D), list(mu = out$mu, sd = out$sd, w = out$w), log = FALSE)

############################################################

# Compare input data and results

ord.out <- order(out$mu[,1])
ord.in <- order(component.means[,1])

means.out <- out$mu[ord.out,]
means.in <- component.means[ord.in,]

# Cluster stds and variances
sds.out <- out$sd[ord.out,]
sds.in  <- sqrt(component.vars[ord.in,])

# -----------------------------------------------------------

vars.out <- sds.out^2
vars.in <- sds.in^2

# Check correspondence between input and output
if (length(means.in) == length(means.out)) {
   cm <- cor(as.vector(means.in), as.vector(means.out))
   csd <- cor(as.vector(sds.in), as.vector(sds.out))
}

# Plot results (assuming 2D)

ran <- range(c(as.vector(means.in - 2*vars.in), 
               as.vector(means.in + 2*vars.in), 
	       as.vector(means.out + 2*vars.out), 
	       as.vector(means.out - 2*vars.out)))

plot(D, pch = 20, main = paste("Cor.means:", round(cm,3), "/ Cor.sds:", round(csd,3)), xlim = ran, ylim = ran) 
#for (ci in 1:nrow(means.out))  { add.ellipse(centroid = means.out[ci,], covmat = diag(vars.out[ci,]), col = "red") }
#for (ci in 1:nrow(means.in))  { add.ellipse(centroid = means.in[ci,], covmat = diag(vars.in[ci,]), col = "blue") }

######################################################

#for (ci in 1:nrow(means.out))  {
#    points(means.out[ci,1], means.out[ci,2], col = "red", pch = 19)
#    el <- ellipse(matrix(c(vars.out[ci,1],0,0,vars.out[ci,2]),2), centre = means.out[ci,])
#    lines(el, col = "red") 						  
#}

#for (ci in 1:nrow(means.in))  {
#    points(means.in[ci,1], means.in[ci,2], col = "blue", pch = 19)
#    el <- ellipse(matrix(c(vars.in[ci,1],0,0,vars.in[ci,2]),2), centre = means.in[ci,])
#    lines(el, col = "blue") 						  
#}

setMethod("get.P.r", "NetResponseModel", function(model, subnet.id, log = TRUE) {
    
    # Prior probability for each response (mixture weights) Pr(model, subnet.id)
    # output: a vector
    
    pars <- get.model.parameters(model, subnet.id)
    
    if (log) {
        log(pars$w)
    } else {
        pars$w
    }
})



setMethod("get.P.Sr", "NetResponseModel", function(sample, model, subnet.id, log = TRUE) {
    
    pars <- get.model.parameters(model, subnet.id)
    
    # Log density of a given sample group P(S|r) for each response; length of output
    # equals to number of responses i.e. logsum of the individual sample densities
    
    nodes <- model@subnets[[subnet.id]]
    
    # pick sample data for the response and ensure this is a matrix also when a
    # single sample is given
    dat <- t(matrix(model@datamatrix[sample, nodes], ncol = length(nodes)))
    colnames(dat) <- sample
    rownames(dat) <- nodes
    
    P.Sr(dat, pars, log = log)
    
})


setMethod("get.P.rs.joint", "NetResponseModel", function(sample, model, subnet.id, 
    log = TRUE) {
    
    # Joint probability P(r,s) where s can be a single point or set of samples: P(r,
    # s) = P(s | r) * P( r )
    pars <- get.model.parameters(model, subnet.id)
    nodes <- model@subnets[[subnet.id]]
    # pick sample data for the response and ensure this is a matrix also when a
    # single sample is given
    dat <- t(matrix(model@datamatrix[sample, nodes], ncol = length(nodes)))
    colnames(dat) <- sample
    rownames(dat) <- nodes
    
    P.rs.joint(dat, pars, log = TRUE)
    
})




setMethod("get.P.rS", "NetResponseModel", function(model, subnet.id, log = TRUE) {
    
    # Probability of a response, given sample (group) P(r|S) = P(S|r)P(r)/P(S) = P(S,
    # r)/(sum_r P(S, r))
    
    # Joint probability P(r,s) where s can be a single point or set of samples: P(r,
    # s) = P(s | r) * P( r )
    pars <- get.model.parameters(model, subnet.id)
    dat <- get.dat(model, subnet.id)
    
    P.rS(dat, pars, log = TRUE)
    
})


setMethod("get.P.rs.joint.individual", "NetResponseModel", function(sample, model, 
    subnet.id, log = TRUE) {
    
    # Joint probability P(r,s) for individual samples samples: P(r, s) = P(s | r) *
    # P( r )
    pars <- get.model.parameters(model, subnet.id)
    dat <- get.dat(model, subnet.id, sample)  # dat is now features x samples matrix
    
    prs <- P.rs.joint.individual(dat, pars, log = log)
    colnames(prs) <- sample
    
    prs
    
})




setMethod("get.P.s.individual", "NetResponseModel", function(sample, model, subnet.id, 
    log = TRUE) {
    
    # Overall probability of sample s, given the model.  individually for each sample
    
    # responses x samples for each sample (column), density mass is the sum over
    # joint densities on individual responses P(s) = sum_r P(s, r) = sum_r P(s,r) =
    # sum_r P(s|r)P(r)
    pars <- get.model.parameters(model, subnet.id)
    dat <- get.dat(model, subnet.id, sample)  # dat is now features x samples matrix
    
    ps <- P.s.individual(dat, pars, log = log)
    names(ps) <- sample
    
    ps
})




setMethod("sample.densities", "NetResponseModel", function(sample, model, subnet.id, 
    log = TRUE, summarize = FALSE) {
    
    # Calculate conditional density P(s|r) for each sample in each response r and
    # then in the complete model if summarize = TRUE then give overall density of the
    # sample, otherwise densities for individual samples
    pars <- get.model.parameters(model, subnet.id)
    dat <- get.dat(model, subnet.id, sample)  # dat is now features x samples matrix
    
    psr <- P.s.r(dat, pars, log = TRUE)
    
    # Density for the sample given the overall model
    ps <- P.s.individual(dat, pars, log = TRUE)
    names(ps) <- sample
    # dtot <- get.P.s.individual(sample, model, subnet.id, log = TRUE) this should be
    # same as colSums(psr * pars$w)
    
    rnam <- rownames(psr)
    psr <- rbind(psr, ps)
    rownames(psr) <- c(rnam, "total.density")
    
    # psr is a responses x samples matrix
    if (summarize) {
        psr <- rowSums(psr)
    }
    
    if (!log) {
        psr <- exp(psr)
    }
    
    psr
    
})



setMethod("get.P.s", "NetResponseModel", function(sample, model, subnet.id, log = TRUE) {
    
    # Overall probability of sample s, given the Gaussian mixture model P(s) = sum_r
    # P(s, r) ps <- log(sum(get.P.rs.joint(sample, model, subnet.id, log = FALSE)))
    
    # FIXME: numerically does not hold tightly that P(S) = sumr P(S, r) = sumr
    # P(S|r)P(r) the latter equality holds, P(S) is problematic. Differences are not
    # big for examples I checked, but they are still notable. Check in more detail
    # this one. sum(get.P.rs.joint(s, model, pars = NULL, subnet.id, log = FALSE))
    # sum(get.P.Sr(s, model, pars = NULL, subnet.id, log = FALSE) * pars$w)
    
    # P(s) separately for each individual sample
    
    # product over individual sample densities (i.e. log sum) psi <-
    # get.P.s.individual(sample, model, subnet.id, log = TRUE)
    pars <- get.model.parameters(model, subnet.id)
    dat <- get.dat(model, subnet.id, sample)  # dat is now features x samples matrix
    
    P.S(dat, pars, log = log)
    
})



setMethod("get.P.rs", "NetResponseModel", function(model, subnet.id, log = FALSE) {
    
    # Probability of each response, given sample samples x responses matrix, each row
    # sums to unity
    prs <- sample2response(model, subnet.id)
    
    if (log) {
        log(prs)
    } else {
        prs
    }
})


#' Sample-to-response matrix of probabilities P(r|s).
#' 
#' Retrieve P(r|s) from NetResponseModel model.
#' 
#' Calculates probability density for each response on a given sample based on
#' the estimated Gaussian mixture model.
#' 
#' @aliases getqofz getqofz,NetResponseModel-method
#' @usage getqofz(model, subnet.id, log = FALSE)
#' @param model NetResponseModel object.
#' @param subnet.id Subnetwork to investigate.
#' @param log Output in log probabilities.
#' @return Samples x responses matrix. Each entry is a probability P(r|s).
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse').
#' @keywords utilities internal
#' @examples
#' 
#' # qofz <- getqofz(model, subnet.id, log = FALSE)
#' 
setMethod("getqofz", "NetResponseModel", function(model, subnet.id, log = FALSE) {
    
    # Retrieve P(r|s) from the model, given data and model parameters
    pars <- get.model.parameters(model, subnet.id)
    dat <- get.dat(model, subnet.id)  # Dat is now features x samples matrix
    qofz <- P.r.s(dat, pars, log = log)
    rownames(qofz) <- rownames(model@datamatrix)  #model@samples
    colnames(qofz) <- paste("Mode", seq_len(ncol(qofz)), sep = "-")
    
    qofz
    
})


#' Get subnetwork data
#' 
#' @inheritParams sample2response
#' @param sample Define the retrieved samples
#' @aliases get.dat get.dat,NetResponseModel-method
#' @return Subnet data matrix
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords utilities
#' @examples
#' ## Load a pre-calculated netresponse model obtained with 
#' # model <- detect.responses(toydata$emat, toydata$netw, verbose = FALSE)
#' # data( toydata ); get.dat(toydata$model) 
setMethod("get.dat", "NetResponseModel", function(model, subnet.id, sample = NULL) {
    
    # usage get.dat(model, subnet.id, sample = NULL)
    
    if (is.null(sample)) {
        sample <- seq_len(nrow(model@datamatrix))
    }
    nodes <- model@subnets[[subnet.id]]
    
    dat <- t(matrix(model@datamatrix[sample, nodes], length(sample)))
    rownames(dat) <- nodes
    colnames(dat) <- as.character(sample)
    
    dat
})


#' get.subnets
#' 
#' List the detected subnetworks (each is a list of nodes in the corresponding
#' subnetwork).
#' 
#' @aliases get.subnets get.subnets,NetResponseModel-method
#' @param model Output from the detect.responses function. An object of
#' NetResponseModel class.
#' @param get.names Logical. Indicate whether to return subnetwork nodes using
#' node names (TRUE) or node indices (FALSE).
#' @param min.size,max.size Numeric. Filter out subnetworks whose size is not
#' within the limits specified here.
#' @param min.responses Numeric. Filter out subnetworks with less responses
#' (mixture components) than specified here.
#' @return A list of subnetworks.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references Leo Lahti et al.: Global modeling of transcriptional responses
#' in interaction networks. Bioinformatics (2010).  See citation('netresponse')
#' for details.
#' @keywords utilities
#' @export
#' @examples
#' ## Load a pre-calculated netresponse model obtained with 
#' # model <- detect.responses(toydata$emat, toydata$netw, verbose = FALSE)
#' # data( toydata ); get.subnets(toydata$model) 
setMethod("get.subnets", "NetResponseModel", function(model, get.names = TRUE, min.size = 2, 
    max.size = Inf, min.responses = 2) {
    
    grouping <- model@last.grouping
    
    # Use feature names instead of indices
    if (get.names) {
        grouping <- lapply(grouping, function(x) {
            colnames(model@datamatrix)[unlist(x)]
        })
    }
    
    # If filters are given, apply them (stat needs to be specified)
    
    # SUBNET SIZE
    
    subnet.size <- sapply(grouping, length)
    df <- data.frame(sapply(grouping, length))
    colnames(df) <- c("subnet.size")
    inds <- rownames(subset(df, subnet.size >= min.size & subnet.size <= max.size))
    
    ## NUMBER OF RESPONSES
    
    if (min.responses > 1) {
        
        stat <- model.stats(model)
        
        # check which subnets pass the filter
        inds.size <- rownames(subset(stat, subnet.size >= min.size & subnet.size <= 
            max.size))  # & 
        
        inds.nresp <- rownames(stat)[which(stat[["responses"]] >= min.responses)]
        
        inds <- intersect(inds.size, inds.nresp)
        
    }
    
    # Get the filtered subnetwork list
    if (length(inds) == 0) {
        grouping <- NULL
    } else {
        grouping <- grouping[inds]
    }
    
    grouping
    
})


setMethod(f = "[[", signature("NetResponseModel"), definition = (function(x, i, j = "missing", 
    ..., exact = TRUE) {
    if (typeof(i) == "numeric") {
        i <- names(x)[[i]]
    }
    get.model.parameters(x, subnet.id = i)
}))

#' @title Plot univariate mixtures
#' @description Visualize data, centroids and stds for a given univariate Gaussian mixture model with PCA.
#' @param x data vector
#' @param means mode centroids
#' @param sds mode standard deviations
#' @param ws weight for each mode
#' @param title.text Plot title
#' @param xlab.text xlab.text
#' @param ylab.text ylab.text
#' @param binwidth binwidth for histogram
#' @param qofz Mode assignment probabilities for each sample. Samples x modes.
#' @param density.color Color for density lines
#' @param cluster.assignments Vector of cluster indices, indicating cluster for each data point
#' @param ... Further arguments for plot function.
#' @return Used for its side-effects
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @examples # plotMixtureUnivariate(dat, means, sds, ws)
PlotMixtureUnivariate <- function(x, means = NULL, sds = NULL, ws = NULL, title.text = NULL, 
    xlab.text = NULL, ylab.text = NULL, binwidth = 0.05, qofz = NULL, density.color = "darkgray", 
    cluster.assignments = NULL, ...) {
    
    # Circumvent warnings
    ..density.. <- NULL
    vals <- NULL
    varname <- NULL
    
    # Find cluster for each sample
    if (is.null(cluster.assignments)) {
        if (is.null(qofz)) {
            qofz <- P.r.s(t(matrix(x)), list(mu = means, sd = sds, w = ws), log = TRUE)
        }
        cluster.assignments <- apply(qofz, 1, which.max)
    }
    
    x <- unname(x)
    df <- data.frame(list(x = x))
    df$mode <- factor(cluster.assignments)
    
    # Histogram and density plot
    pg <- ggplot(df, aes(x = x))
    pg <- pg + geom_histogram(aes(fill = mode), binwidth = binwidth)
    pg <- pg + theme_bw() + xlab(xlab.text) + ylab(ylab.text)
    pg <- pg + ggtitle(title.text)
    
    # If normal mixture parameters are given, overlay Gaussians on top of the plot
    if (!is.null(means)) {
        
        # Estimated normal distributions from the mixture model
        h <- hist(x, seq(min(x) - binwidth - 1/binwidth, max(x) + binwidth + 1/binwidth, 
            binwidth), plot = FALSE)
        df$vals <- seq(min(df$x), max(df$x), length = nrow(df))  # estimation points for fitted Gaussians
        scal <- max(h$counts)/max(ws[[1]] * dnorm(df$vals, mean = means[[1]], sd = sds[[1]]))
        for (comp in seq_len(length(means))) {
            df2 <- data.frame(list(vals = df$vals, varname = scal * ws[[comp]] * 
                dnorm(df$vals, mean = means[[comp]], sd = sds[[comp]])))
            # Add in normal distribution
            pg <- pg + geom_line(aes(x = vals, y = varname), color = density.color, 
                data = df2)
        }
    }
    
    pg
    
}

#' @title Multivariate BIC mixture
#' @description Latent class analysis based on (infinite) Gaussian mixture
#' model.
#' If the input (dat) is data matrix, a multivariate model is fitted. 
#' @param x matrix (for multivariate analysis)
#' @param max.modes Maximum number of modes to be checked for mixture
#' model selection
#' @param bic.threshold BIC threshold which needs to be exceeded before a
#' new mode is added to the mixture.
#' @param min.modes Minimum number of modes to be checked for mixture
#' model selection
#' @param ... Further optional arguments to be passed
#' @return Fitted latent class model (parameters and free energy)
#' @references See citation('netresponse') 
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @keywords utilities
bic.mixture.multivariate <- function(x, max.modes, bic.threshold = 0, min.modes = 1, 
    ...) {

    # x <- mat; max.modes = params$max.responses; bic.threshold =
    # params$bic.threshold
    
    best.mode <- bic.select.best.mode(x, max.modes, bic.threshold, min.modes)
    
    mcl <- Mclust(x, G = best.mode)
    
    bic <- try(-mclustBIC(x, G = best.mode)[, "VVV"])
    if (is.na(bic)) 
        {
            bic <- Inf
        }  # infinitely bad = Inf
    
    means <- t(mcl$parameters$mean)
    vars <- t(apply(mcl$parameters$variance$sigma, 3, function(x) {
        diag(x)
    }))
    sds <- sqrt(vars)
    ws <- as.vector(mcl$parameters$pro)
    if (is.null(ws)) {
        ws <- 1
    }
    
    Nparams <- prod(dim(means)) + prod(dim(sds)) + length(ws)
    
    # Determine the most likely mode for each sample (-> hard clusters)
    qofz <- P.r.s(t(x), list(mu = means, sd = sds, w = ws), log = FALSE)
    rownames(qofz) <- rownames(x)
    colnames(qofz) <- paste("Mode", seq_len(ncol(qofz)), sep = "-")
    
    rownames(means) <- rownames(sds) <- names(ws) <-
        paste("Mode", seq_len(length(ws)), 
        sep = "-")
    colnames(means) <- colnames(sds) <- colnames(x)
    
    list(means = means, sds = sds, ws = ws, Nparams = Nparams,
        free.energy = -mcl$loglik, 
        qofz = qofz, bic = bic)
    
}

# Copyright (C) 2008-2012 Leo Lahti and Olli-Pekka Huovilainen Contact: Leo Lahti
# <leo.lahti@iki.fi> This program is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your option) any
# later version.  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.  Acknowledgements: This program is based on the AIVGA
# Agglomerative Independent Variable Group Analysis package (v. 1.0) Copyright
# (C) 2001-2007 Esa Alhoniemi, Antti Honkela, Krista Lagus, Jeremias Seppa, Harri
# Valpola, and Paul Wagner.


#' @title Pick model pairs
#' @description Mainly for internal use. Calculate joint model for each node pair
#' @param network network
#' @param network.nodes network.nodes
#' @param node.models node.models
#' @param datamatrix datamatrix
#' @param params parameters
#' @return \item{model.pairs}{joint models for each node pair}
#' \item{delta}{corresponding delta value for the cost function}
#' @author Maintainer: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords internal
#' @examples #
pick.model.pairs <- function(network, network.nodes, node.models, datamatrix, params) {
    
    # Storage list for calculated models
    model.pairs <- vector(length = ncol(network), mode = "list")
    delta <- rep(NA, ncol(network))
    
    for (edge in seq_len(ncol(network))) {
        
        tmp <- edge.delta(edge, network = network, network.nodes = network.nodes, 
            datamatrix = datamatrix, params = params, node.models = node.models)
        
        model.pairs[[edge]] <- tmp[["model"]]
        delta[[edge]] <- tmp[["delt"]]
        
    }
    
    gc()
    
    list(model.pairs = model.pairs, delta = delta)
    
}



edge.delta <- function(edge, network, network.nodes, datamatrix, params, node.models) {
    
    if (params$verbose) {
        message(paste("Computing delta values for edge ", edge, "/", ncol(network), 
            "\n"))
    }
    a <- network[1, edge]
    b <- network[2, edge]
    vars <- network.nodes[c(a, b)]
    
    mat <- matrix(datamatrix[, vars], nrow(datamatrix))
    rownames(mat) <- rownames(datamatrix)
    colnames(mat) <- vars
    
    model.params <- mixture.model(x = mat, mixture.method = params$mixture.method, 
        max.responses = params$max.responses, implicit.noise = params$implicit.noise, 
        prior.alpha = params$prior.alpha, prior.alphaKsi = params$prior.alphaKsi, 
        prior.betaKsi = params$prior.betaKsi, vdp.threshold = params$vdp.threshold, 
        initial.responses = params$initial.responses, ite = params$ite, speeup = params$speedup, 
        bic.threshold = params$bic.threshold, pca.basis = params$pca.basis)
    
    # Compute COST-value for two independent subnets vs. joint model Negative free
    # energy (-cost) is (variational) lower bound for P(D|H) Use it as an
    # approximation for P(D|H) Cost for the indpendent and joint models -cost is sum
    # of two independent models (cost: appr. log-likelihoods)
    costind <- info.criterion(node.models[[a]]$Nparams + node.models[[b]]$Nparams, 
        params$Nlog, -(node.models[[a]]$free.energy + node.models[[b]]$free.energy), 
        criterion = params$information.criterion)
    costjoint <- info.criterion(model.params$Nparams, params$Nlog, -model.params$free.energy, 
        criterion = params$information.criterion)
    
    # NOTE: COST is additive so summing is ok change (increase) of the total COST /
    # cost
    delt <- as.numeric(costjoint - costind)
    
    # Store these only if it would improve the cost; otherwise never needed again
    if (-delt > params$merging.threshold) {
        mod.pair <- model.params
    } else {
        mod.pair <- 0
    }
    
    return(list(model = mod.pair, delt = delt))
}

enrichment.score <- function(total.samples, subset.samples, annotated.samples, method = "hypergeometric") {
    
    if (method == "hypergeometric") {
        
        N <- length(total.samples)
        
        # number of white balls in the urn
        m <- length(annotated.samples)
        
        # number of black balls in the urn
        n <- N - m
        
        # number of balls drawn from the urn
        k <- length(subset.samples)
        
        # overlap between investigated sample group among response samples (using hard
        # assignments) number of white balls drawn without replacement
        q <- sum(annotated.samples %in% subset.samples)
        
        # hypergeometric enrichment (small p, high enrichment) take 1-p to indicate high
        # enrichment with high score use q-1 since lower.tail = FALSE indicates X > x
        # calculation, but we need X >=x enr <- 1 - phyper(q-1, m, n, k, lower.tail =
        # FALSE, log.p = FALSE)
        pval <- phyper(q - 1, m, n, k, lower.tail = FALSE, log.p = FALSE)
        
        temp <- c(sample.size.total = N, sample.size.subset = k, sample.size.annotated = m, 
            annotated.in.subset = q, fraction.in.data = m/N, fraction.in.subset = q/k, 
            pvalue = pval)
        
        enr <- list(score = 1 - pval, info = temp)
    }
    
    enr
    
}

#' @title filter.netw
#' 
#' @description Mostly for internal use. Prefilter edges if speedups required.
#' 
#' @details Include only edges with the highest mutual information,
#' calculated based on the first principal components.
#' 
#' @usage filter.netw(network, delta, datamatrix, params)
#' @param network network
#' @param delta associated cost function value changes for each node merge
#' @param datamatrix datamatrix
#' @param params parameters
#' @return Filtered network
#' @author Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao.  Maintainer:
#' Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords internal
filter.netw <- function(network, delta, datamatrix, params) {
    
    if (params$max.subnet.size > 1) {
        
        if (params$positive.edges) {
            tmp <- remove.negative.edges(network, delta, datamatrix)
            network <- tmp$network
            delta <- tmp$delta
        }
        
        # Filter out the least promising edges from the network based on mutual
        # information. For each variable, pick at most speedup.max.edges
        if (params$speedup && !is.null(params$speedup.max.edges)) {
            if (params$verbose) {
                message("Filter the network to only keep the edges with highest mutual information")
            }
            tmp <- filter.network(network, delta, datamatrix, params)
            network <- tmp$network
            delta <- tmp$delta
            
        }
        
    }
    
    list(network = network, delta = delta)
    
}




#' remove.negative.edges
#' 
#' Remove edges where the connected nodes correlate negatively
#' 
#' @param network network
#' @param delta associated cost function value changes for each node merge
#' @param datamatrix datamatrix
#' @return Filtered network
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords internal
#' @examples #

remove.negative.edges <- function(network, delta, datamatrix) {
    
    message("Filtering out negative edges..")
    
    # Correlation matrix between the nodes FIXME: parallelize
    
    for (edge in seq_len(ncol(network))) {
        
        # Pick nodes for this edge
        a <- network[1, edge]
        b <- network[2, edge]
        
        # Calculate correlation btw. the two nodes
        cc <- cor(datamatrix[, a], datamatrix[, b], method = "spearman")
        
        # Give infinite cost for edges with non-positive association
        if (cc <= 0) {
            delta[[edge]] <- Inf
        }
    }
    
    # Remove edges with infinite cost
    keep.edges <- !is.infinite(delta)
    network <- network[, keep.edges]
    delta <- delta[keep.edges]
    
    list(network = network, delta = delta)
}



#' filter.network
#' 
#' 
#' Include to the network only the edges with the highest mutual information,
#' calculated based on the first principal components.
#' 
#' @param network network
#' @param delta associated cost function value changes for each node merge
#' @param datamatrix datamatrix
#' @param params parameters
#' @return Filtered network
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords internal
#' @examples #
filter.network <- function(network, delta, datamatrix, params) {
    
    # Include at maximum speedup.max.edges for each node in the network based on
    # mutual information return list of edges to keep
    
    remove.edges <- c()
    uniq.edges <- unique(network[1, ])
    
    for (idx in seq_len(length(uniq.edges))) {
        if (params$verbose) {
            message(paste(idx, "/", length(uniq.edges)))
        }
        a <- uniq.edges[[idx]]
        
        # Pick edges for this node
        eds <- which(network[1, ] == a)
        
        # Calculate MI scores
        mis <- c()
        mi.cnt <- 0
        # FIXME: could be parallelized which(is.na(delta))
        for (edge in eds) {
            mi.cnt <- mi.cnt + 1
            
            # Pick node indices
            i <- network[2, edge]
            
            # For singletons
            dat <- cbind(datamatrix[, a], datamatrix[, i])
            
            mis[[mi.cnt]] <- build.mim(dat, estimator = "spearman")[1, 2]
        }
        
        # Edges to remove

        mis <- as.data.frame(lapply(mis, unlist))

        remove.edges <- eds[na.omit(order(mis, decreasing = TRUE)[-seq(params$speedup.max.edges)])]
        keep.edges <- setdiff(seq_len(ncol(network)), remove.edges)
        
        # Filter out remove.edges
        if (length(remove.edges) > 0) {
            network <- network[, keep.edges]
            delta <- delta[keep.edges]
        }
        
    }
    
    list(network = network, delta = delta)
    
}

#' @importFrom rmarkdown render

setGeneric("get.subnets", function(model, get.names = TRUE,
    min.size = 2, max.size = Inf, 
    min.responses = 2) {
    standardGeneric("get.subnets")
})
setGeneric("getqofz", function(model, subnet.id, log = FALSE) {
    standardGeneric("getqofz")
})
setGeneric("get.dat", function(model, subnet.id, sample = NULL) {
    standardGeneric("get.dat")
})
setGeneric("get.P.r", function(model, subnet.id, log = TRUE) {
    standardGeneric("get.P.r")
})
setGeneric("get.P.Sr", function(sample, model, subnet.id, log = TRUE) {
    standardGeneric("get.P.Sr")
})
setGeneric("get.P.rs.joint", function(sample, model, subnet.id, log = TRUE) {
    standardGeneric("get.P.rs.joint")
})
setGeneric("get.P.rS", function(model, subnet.id, log = TRUE) {
    standardGeneric("get.P.rS")
})
setGeneric("get.P.rs.joint.individual", function(sample, model,
    subnet.id, log = TRUE) {
    standardGeneric("get.P.rs.joint.individual")
})
setGeneric("get.P.s.individual", function(sample, model,
    subnet.id, log = TRUE) {
    standardGeneric("get.P.s.individual")
})
setGeneric("sample.densities", function(sample, model, subnet.id,
    log = TRUE, summarize = FALSE) {
    standardGeneric("sample.densities")
})
setGeneric("get.P.s", function(sample, model, subnet.id, log = TRUE) {
    standardGeneric("get.P.s")
})
setGeneric("get.P.rs", function(model, subnet.id, log = FALSE) {
    standardGeneric("get.P.rs")
})

#' @import methods
#' @importFrom BiocStyle pkg_ver
#' @import graph
#' @importFrom graphics axis
#' @importFrom graphics barplot
#' @importFrom graphics hist
#' @importFrom graphics image
#' @importFrom graphics legend
#' @importFrom graphics par
#' @importFrom graphics points
#' @importFrom grDevices colorRampPalette
#' @importFrom grDevices gray
#' @importFrom grDevices palette
#' @importFrom grDevices rainbow
#' @import parallel
#' @import RColorBrewer
#' @import Rgraphviz
#' @importFrom igraph igraph.to.graphNEL
#' @importFrom igraph igraph.from.graphNEL
#' @importFrom igraph subgraph
#' @importFrom igraph graph.data.frame
#' @importFrom igraph graph.edgelist
#' @import mclust
#' @importFrom plyr ddply
#' @importFrom minet build.mim
#' @importFrom reshape2 melt
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 qplot
#' @importFrom ggplot2 theme_set
#' @importFrom ggplot2 theme_bw
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 geom_point
#' @importFrom ggplot2 ggtitle
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 ylab
#' @importFrom ggplot2 labs
#' @importFrom ggplot2 geom_histogram
#' @importFrom ggplot2 geom_line
#' @importFrom ggplot2 geom_errorbar
#' @importFrom ggplot2 geom_bar
#' @importFrom ggplot2 geom_boxplot
#' @importFrom ggplot2 facet_wrap
#' @importFrom stats cor
#' @importFrom stats dnorm
#' @importFrom stats na.omit
#' @importFrom stats rgamma
#' @importFrom stats rnorm
#' @importFrom stats runif
#' @importFrom stats p.adjust
#' @importFrom stats phyper
#' @importFrom stats princomp
#' @importFrom stats prcomp
#' @importFrom stats qchisq
#' @importFrom stats sd
#' @importFrom stats t.test
#' @importFrom utils read.csv

.onAttach <- function(lib, pkg) {
    packageStartupMessage("\nnetresponse (C) 2008-2023 Leo Lahti et al.\n\nhttps://github.com/antagomir/netresponse")
}

skip <- FALSE

if (!skip) {

library(netresponse)

# SINGLE MODE

# Produce test data that has full covariance
# It is expected that
# pca.basis = FALSE splits Gaussian with full covariance into two modes
# pca.basis = TRUE should detect just a single mode

Ns <- 200
Nd <- 2
k <- 1.5

D2 <- matrix(rnorm(Ns*Nd), ncol = Nd) %*% rbind(c(1,k), c(k,1))

par(mfrow = c(2,2))
for (mm in c("vdp", "bic")) {
  for (pp in c(FALSE, TRUE)) {

    # Fit nonparametric Gaussian mixture model
    out <- mixture.model(D2, mixture.method = mm, pca.basis = pp)
    plot(D2, col = apply(out$qofz, 1, which.max), main = paste("mm:" , mm, "/ pp:",  pp)) 

  }
}

}

#' @title Plot mixtures
#' @description Plot mixtures.
#' @param x data vector
#' @param qofz Mode assignment probabilities for each sample. Samples x modes.
#' @param binwidth binwidth for histogram
#' @param xlab.text xlab.text
#' @param ylab.text ylab.text
#' @param title.text title.text
#' @return Used for its side-effects
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for citation details.
#' @keywords utilities
#' @examples # PlotMixture(x, qofz)
PlotMixture <- function(x, qofz, binwidth = 0.05, xlab.text = NULL, ylab.text = NULL, 
    title.text = NULL) {
    
    # hard sample clusters
    sms <- apply(qofz, 1, which.max)
    
    x <- unname(x)
    df <- data.frame(list(x = x))
    df$mode <- factor(sms)
    
    # Histogram and density plot
    pg <- ggplot(df, aes(x = x)) +
        geom_histogram(aes(fill = mode), binwidth = binwidth) +
        theme_bw() +
        labs(x = xlab.text, y = ylab.text, title = title.text)
    
    pg
    
}

#' @title check.network
#' @description Internal use to check input network and format detect.responses.
#' @param network Input network, see detect.responses
#' @param datamatrix Input datamatrix, see detect.responses
#' @param verbose Print intermediate messages
#' @return 
#'   \item{formatted }{Formatted network (self-links removed)}
#'   \item{original }{Original network (possible in another representation format)} 
#'   \item{delta }{Cost function changes corresponding to the 'formatted' network.} 
#'   \item{nodes }{Nodes corresponding to the 'formatted' network.}
#' @author Maintainer: Leo Lahti \email{leo.lahti@@iki.fi}
#' @seealso detect.responses
#' @references See citation('netresponse')
#' @examples # check.network(network, datamatrix, verbose = FALSE)
check.network <- function(network, datamatrix, verbose = FALSE) {
    
    
    # If no network is given, assume fully connected net
    if (is.null(network)) {
        if (verbose) {
            message("No network provided in function call: assuming fully connected nodes.")
        }
        network <- matrix(1, ncol(datamatrix), ncol(datamatrix))
        rownames(network) <- colnames(network) <- colnames(datamatrix)
    }
    
    # FIXME: later add other forms of sparse matrices from Matrix package
    accepted.formats.net <- c("matrix", "Matrix", "dgCMatrix", "dgeMatrix", "graphNEL", 
        "igraph", "graphAM")
    if (!is(network)[[1]] %in% accepted.formats.net) {
        stop(paste("network needs to be in one of the following formats:", paste(accepted.formats.net, 
            collapse = "; ")))
    }
    
    # Convert matrix into graphNEL if it is not already
    if (length(is(network)) > 1 || !("graphNEL" %in% is(network))) {
    
        if (any(is(network) %in% c("dgeMatrix", "dfCMatrix", "Matrix", "data.frame"))) {
            # Sparse matrix or data.frame needs to be converted first into matrix
            network <- as.matrix(network)
        }
        
        if (is.matrix(network)) {
            
            if (!nrow(network) == ncol(network)) {
                stop("Error: network nrow = ncol required.\n")
            }
            
            # Ensuring symmetric network
            if (any(!network == t(network))) {
                warning("Network is not symmetric. Removing link directions to force symmetric network.")
                network <- ((network + t(network)) > 0) - 0
            }
            
            # Filter non-connected nodes
            keep <- rowSums(network) > 1
            network <- network[keep, keep]
            
            # check that node names given in the data and correspond
            if (is.null(rownames(network)) || is.null(colnames(datamatrix))) {
                if (!nrow(network) == ncol(datamatrix)) {
                    stop("Error: Equal number of features required for the network and data matrix when feature names not given.\n")
                } else {
                    warning("Warning: network and/or data features are not named; matched by order.\n")
                    if (is.null(rownames(network)) && is.null(colnames(datamatrix))) {
                        rownames(network) <- colnames(network) <- as.character(seq_len(nrow(network)))
                        colnames(datamatrix) <- rownames(network)
                    } else if (is.null(rownames(network)) && !is.null(colnames(datamatrix))) {
                        rownames(network) <- colnames(network) <- colnames(datamatrix)
                    } else if (!is.null(rownames(network)) && is.null(colnames(datamatrix))) {
                        colnames(datamatrix) <- rownames(network)
                    }
                }
            }
            network <- as(new("graphAM", adjMat = network), "graphNEL")
        } else if (is(network) == "igraph") {
            network <- igraph.to.graphNEL(network)
        }
    }

    # Now the network is in graphNEL format
    
    # FIXME: adjust such that igraph does not need to be converted in graphNEL (which
    # is larger FIXME: add option to give this as input; seems to consume much less
    # memory than graphNEL

    # list network nodes that are not in datamatrix
    common.feats <- intersect(nodes(network), colnames(datamatrix))
    other.feats <- setdiff(nodes(network), common.feats)

    # remove features with no functional data
    if (length(other.feats) > 0) {
        if (verbose) {
            message(paste("removing network nodes that are not in datamatrix: ", 
                length(other.feats), " nodes removed;", length(common.feats), " nodes used for modeling."))
        }
        # message('converting to igraph')
        network <- igraph.from.graphNEL(network)
        # message('selecting nodes that have functional data')
        network <- subgraph(network, common.feats)
        # message('converting to graphNEL')
        network <- igraph.to.graphNEL(network)
        # network <- removeNode(other.feats, network)
    }
    
    message("convert the network into edge matrix")
    # store original network node list
    network.nodes <- nodes(network)
    network <- edgeMatrix(network, duplicates = FALSE)  # indices correspond to node list in network.nodes
    # order such that row1 < row2
    network <- apply(network, 2, sort)
    if (verbose) 
        message("removing self-links")
    network <- network[, !network[1, ] == network[2, ]]
    
    # Store the network in igraph format
    network.orig <- network  # store network used for modeling (preprocessed)
    # FIXME: igraph is more memory-efficient but could not be used as network is in
    # NetResponseModel definition for some reason. If possible, convert from graphNEL
    # to igraph later on.
    
    # Network rows: 1) nodes 2) nodes 3) merging cost function delta
    rownames(network) <- c("node1", "node2")
    
    # Delta corresponds to the columns of the 'network' object
    delta <- rep(NA, ncol(network))
    
    list(formatted = network, original = network.orig, delta = delta, nodes = network.nodes)
}

#' @title Select best mode with BIC
#' @description Select optimal number of mixture components by adding components until 
#' the increase in objective function is below threshold.
#' @param x  dat vector (for univariate analysis) or a matrix (for multivariate analysis)
#' @param max.modes Maximum number of modes to be checked for mixture model selection
#' @param bic.threshold BIC threshold which needs to be exceeded before a new mode is added to the mixture.
#' @param min.modes Optional. Minimum number of modes.
#' @return Fitted latent class model (parameters and free energy)
#' @author Contact: Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
bic.select.best.mode <- function(x, max.modes = 1, bic.threshold = 1, min.modes = 1) {


    # Cost for single mode BIC : smaller is better mclustBIC returns the value for
    # -BIC, to be exact
    nc <- min.modes
    if (is.vector(x)) {
        # univariate
        m <- -mclustBIC(x, G = nc)[, "V"]
    } else {
        # multivariate
        m <- -mclustBIC(x, G = nc)[, "VVV"]  # BIC : smaller is better
    }
    
    # ----------------------------------------------------------------
    
    add.component <- TRUE
    best.mode <- min.modes
    if (max.modes == min.modes) {
        add.component <- FALSE
    }
    
    while (add.component && nc < max.modes) {
        
        nc <- nc + 1
        
        # BIC : smaller is better univariate
        if (is.vector(x)) {
            m.new <- try(-mclustBIC(x, G = nc)[, "V"])
        } else {
            # multivariate
            m.new <- try(-mclustBIC(x, G = nc)[, "VVV"])
        }
        if (is.na(m.new)) 
            {
                m.new <- Inf
            }  # infinitely bad = Inf
        
        # FIXME: compressing data with PCA after dimensionality gets otherwise too high?
        # with around ncol(x) = 30 the mclustBIC is starting to produce NAs
        
        # FIXME: remove this when code works ok if (is.na(m.new)) {save(x, nc, file =
        # 'm.new.RData')}
        
        bic.delta <- m.new - m
        
        if (bic.delta < -bic.threshold) {
            best.mode <- nc
            m <- m.new
        } else {
            add.component <- FALSE
        }
    }
    
    best.mode
    
}

#' @title sample2response
#' @description Probabilistic sample-response assignments for given subnet.
#' @usage sample2response(model, subnet.id, mode = 'soft')
#' @param model Result from NetResponse (detect.responses function).
#' @param subnet.id Subnet identifier. A natural number which specifies one of
#' the subnetworks within the 'model' object.
#' @param mode soft: gives samples x responses probabilistic assignment matrix;
#' hard: gives the most likely response for each sample
#' @return A matrix of probabilities. Sample-response assignments for given
#' subnet, listing the probability of each response, given a sample.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references Leo Lahti et al.: Global modeling of transcriptional responses
#' in interaction networks. Bioinformatics (2010).  See citation('netresponse')
#' for citation details.
#' @keywords utilities
#' @export
#' @examples
#' data( toydata )        # Load toy data set
#' D    <- toydata$emat   # Response matrix (for example, gene expression)
#' netw <- toydata$netw   # Network
#' 
#' # Detect network responses
#' #model <- detect.responses(D, netw, verbose = FALSE)
#' 
#' # Assign samples to responses (soft, probabilistic assignments sum to 1)
#' #response.probabilities <- sample2response(model, subnet.id = 'Subnet-1')
sample2response <- function(model, subnet.id, mode = "soft") {
    
    if (is.numeric(subnet.id)) {
        subnet.id <- paste("Subnet", subnet.id, sep = "-")
        warning("subnet.id given as numeric; converting to character: ", subnet.id, 
            sep = "")
    }
    
    # P(response | sample) assignment.matrix <-
    # model@models[[subnet.id]]$posterior$qOFz
    assignment.matrix <- getqofz(model, subnet.id, log = FALSE)
    
    if (mode == "hard") {
        sample.names <- rownames(assignment.matrix)
        assignment.matrix <- colnames(assignment.matrix)[unname(apply(assignment.matrix, 
            1, which.max))]
        names(assignment.matrix) <- sample.names
    }
    
    assignment.matrix
    
}

#' @title pick.model.parameters
#' @description Pick model parameters
#' @param m vdp.mixt output
#' @param nodes node names for naming purposes 
#' @return Model parameters
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse')
#' @keywords internal
#' @export
#' @examples pick.model.parameters(m = NULL, nodes = NULL) 
pick.model.parameters <- function(m, nodes) {

    if (is.null(m)) {return(m)}

    # Pick parameters
    w <- m$posterior$weights  # component weights
    mu <- m$posterior$centroids  # component centroids
    sds <- m$posterior$sds  # component standard devs
    qofz <- m$posterior$qOFz  # soft mode assignmentd
    free.energy <- m$free.energy  # free energy
    Nparams <- m$posterior$Nparams
    
    # For mu and std, rows correspond to the mixture components, in w the elements
    list(mu = mu, sd = sds, w = w, free.energy = free.energy, Nparams = Nparams, 
        qofz = qofz)
    
}

skip <- TRUE

if (!skip) {

# 2. netresponse test
# test later with varying parameters

# Load the package
library(netresponse)
#load("../data/toydata.rda")
fs <- list.files("../R/", full.names = TRUE); for (f in fs) {source(f)};

data(toydata)

D <- toydata$emat
netw <- toydata$netw

# The toy data is random data with 10 features (genes). 
# The features 
rf <- c(4, 5, 6)
#form a subnetwork with coherent responses
# with means 
r1 <- c(0, 3, 0)
r2 <- c(-5, 0, 2)
r3 <- c(5, -3, -3)
mu.real <- rbind(r1, r2, r3)
# real weights
w.real <- c(70, 70, 60)/200
# and unit variances
rv <- 1

# Fit the model
#res <- detect.responses(D, netw, verbose = TRUE, mc.cores = 2)
#res <- detect.responses(D, netw, verbose = TRUE, max.responses = 4)

res <- detect.responses(D, netw, verbose = TRUE, max.responses = 3, mixture.method = "bic", information.criterion = "BIC", merging.threshold = 1, bic.threshold = 10, pca.basis = FALSE)

print("OK")

# Subnets (each is a list of nodes)
subnets <- get.subnets(res)

# the correct subnet is retrieved in subnet number 2:
#> subnet[[2]]
#[1] "feat4" "feat5" "feat6"

# how about responses
# Retrieve model for the subnetwork with lowest cost function value
# means, standard devations and weights for the components
if (!is.null(subnets)) {
m <- get.model.parameters(res, subnet.id = "Subnet-2")

# order retrieved and real response means by the first feature 
# (to ensure responses are listed in the same order)
# and compare deviation from correct solution
ord.obs <- order(m$mu[,1])
ord.real <- order(mu.real[,1])

print(paste("Correlation between real and observed responses:", cor(as.vector(m$mu[ord.obs,]), as.vector(mu.real[ord.real,]))))

# all real variances are 1, compare to observed ones
print(paste("Maximum deviation from real variances: ", max(abs(rv - range(m$sd))/rv)))

# weights deviate somewhat, this is likely due to relatively small sample size
#print("Maximum deviation from real weights: ")
#print( (w.real[ord.real] - m$w[ord.obs])/w.real[ord.real])

print("estimated and real mean matrices")
print(m$mu[ord.obs,])
print(mu.real[ord.real,])

}

}

# Copyright (C) 2008-2012 Olli-Pekka Huovilainen and Leo Lahti Contact: Leo Lahti
# <leo.lahti@iki.fi> This program is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your option) any
# later version.  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.

# The best academic advice I ever got was: 'Spend at least an hour every day on
# the manuscript closest to publication'.

#' check.matrix
#' 
#' Mostly for internal purposes. Check input matrix format.
#' 
#' 
#' @usage check.matrix(datamatrix)
#' @param datamatrix See detect.responses
#' @return The datamatrix, possibly added with necessary formatting for the
#' netresponse algorithm.
#' @author Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao.  Maintainer:
#' Leo Lahti \email{leo.lahti@@iki.fi}
#' @seealso detect.responses
#' @references See citation('netresponse')
#' @keywords internal
#' @examples # datamatrix <- check.matrix(datamatrix)
check.matrix <- function(datamatrix) {
    
    accepted.formats.emat <- c("matrix", "Matrix", "data.frame")
    
    # ensure datamatrix is a matrix
    if (!is.matrix(datamatrix)) {
        if (class(datamatrix) %in% accepted.formats.emat) {
            datamatrix <- as.matrix(datamatrix)
        } else {
            stop(paste("datamatrix needs to be in one of the following formats:", 
                paste(accepted.formats.emat, collapse = "; ")))
        }
    }
    if (is.null(colnames(datamatrix))) {
        colnames(datamatrix) <- as.character(seq_len(ncol(datamatrix)))
    }
    if (is.null(rownames(datamatrix))) {
        rownames(datamatrix) <- as.character(seq_len(nrow(datamatrix)))
    }
    
    datamatrix
}

#' 
#' Find similar features with a given subnetwork.
#' 
#' Given subnetwork, orders the remaining features (genes) in the input data
#' based on similarity with the subnetwork. Allows the identification of
#' similar features that are not directly connected in the input network.
#' 
#' The same similarity measure is used as when agglomerating the subnetworks:
#' the features are ordered by delta (change) in the cost function, assuming
#' that the feature would be merged in the subnetwork. The smaller the change,
#' the more similar the feature is (change would minimize the new cost function
#' value). Negative values of delta mean that the cost function would be
#' improved by merging the new feature in the subnetwork, indicating features
#' having coordinated response.
#' 
#' @usage find.similar.features(model, subnet.id, datamatrix = NULL, verbose =
#' FALSE, information.criterion = NULL)
#' @param model NetResponseModel object.
#' @param subnet.id Investigated subnetwork.
#' @param datamatrix Optional. Can be used to compare subnetwork similarity
#' with new data which was not used for learning the subnetworks.
#' @param verbose Logical indicating whether progress of the algorithm should
#' be indicated on the screen.
#' @param information.criterion Information criterion for model selection. By
#' default uses the same than in the 'model' object.
#' @return A data frame with elements feature.names (e.g. gene IDs) and delta,
#' which indicates similarity level. See details for details. The smaller, the
#' more similar. The data frame is ordered such that the features are listed by
#' decreasing similarity.
#' @author Leo Lahti \email{leo.lahti@@iki.fi}
#' @references See citation('netresponse') for reference details.
#' @keywords utilities
#' @export
#' @examples
#' 
#' data(toydata)
#' model <- toydata$model
#' subnet.id <- 'Subnet-1' 
#' # g <- find.similar.features(model, subnet.id)
#' # List features that are similar to this subnetwork (delta < 0)
#' # (ordered by decreasing similarity)
#' # subset(g, delta < 0)
#' 
find.similar.features <- function(model, subnet.id, datamatrix = NULL, verbose = FALSE, 
    information.criterion = NULL) {
    
    # Given subnetwork, order the remaining genes in the data by similarity with this
    # subnetwork (coordinated transcritional response) the same similarity score is
    # used here than in subnetwork agglomeration steps
    
    # By default, investigate the same data matrix which was used for modelling NOTE:
    # default parameters of the model are always used (model@params)
    
    if (is.null(information.criterion)) {
        information.criterion <- model@params$information.criterion
    }
    
    if (is.null(datamatrix)) {
        datamatrix <- model@datamatrix
    }
    
    if (is.numeric(subnet.id)) {
        subnet.id <- paste("Subnet", subnet.id, sep = "-")
        warning("subnet.id given as numeric; converting to character: ", subnet.id, 
            sep = "")
    }
    
    subnetwork.nodes <- model@subnets[[subnet.id]]
    other.nodes <- setdiff(colnames(datamatrix), subnetwork.nodes)
    
    # Check that subnetwork nodes are included in the data
    if (!all(subnetwork.nodes %in% colnames(datamatrix))) {
        stop("The complete subnetwork (features) must be included in the input datamatrix (samples x features)")
    }
    
    # Use indices of the nodes instead of names
    other.nodes.idx <- match(other.nodes, colnames(datamatrix))
    subnet.idx <- match(subnetwork.nodes, colnames(datamatrix))
    Nlog <- log(nrow(datamatrix))  # log of sample size
    
    ############################################################# 
    
    # FIXME: take straight from the model Retrieve model for the investigated
    # subnetwork
    m.subnet <- vdp.mixt(matrix(datamatrix[, subnetwork.nodes], nrow(datamatrix)), 
        implicit.noise = model@params$implicit.noise, prior.alpha = model@params$prior.alpha, 
        prior.alphaKsi = model@params$prior.alphaKsi, prior.betaKsi = model@params$prior.betaKsi, 
        threshold = model@params$vdp.threshold, initial.K = model@params$initial.responses, 
        ite = model@params$ite, c.max = model@params$max.responses - 1)
    
    cost.subnet <- info.criterion(m.subnet$posterior$Nparams, Nlog, -m.subnet$free.energy, 
        criterion = information.criterion)
    
    ############################################################ 
    
    # Go through data points (features i.e. nodes) and measure similarity for each
    # feature with the given subnetwork
    delta <- c()
    for (fi in other.nodes.idx) {
        fname <- colnames(datamatrix)[[fi]]  # pick also node name
        
        # Note: this calculation jumps the indices that are in the subnetwork
        if (verbose) {
            message(paste("Calculating similarity for feature ", fi, "/", max(other.nodes.idx)))
        }
        
        # print(' Retrieve models for the individual node fi')
        
        m.node <- vdp.mixt(matrix(datamatrix[, fi], nrow(datamatrix)), implicit.noise = model@params$implicit.noise, 
            prior.alpha = model@params$prior.alpha, prior.alphaKsi = model@params$prior.alphaKsi, 
            prior.betaKsi = model@params$prior.betaKsi, threshold = model@params$vdp.threshold, 
            initial.K = model@params$initial.responses, ite = model@params$ite, c.max = model@params$max.responses - 
                1)
        
        cost.node <- info.criterion(m.subnet$posterior$Nparams, Nlog, -m.node$free.energy, 
            criterion = information.criterion)
        
        ########################################################################################## 
        
        # print(' JOINT MODEL')
        
        # subnetwork + candidate feature
        vars <- sort(c(subnet.idx, fi))
        
        m.joint <- vdp.mixt(matrix(datamatrix[, vars], nrow(datamatrix)), implicit.noise = model@params$implicit.noise, 
            prior.alpha = model@params$prior.alpha, prior.alphaKsi = model@params$prior.alphaKsi, 
            prior.betaKsi = model@params$prior.betaKsi, threshold = model@params$vdp.threshold, 
            initial.K = model@params$initial.responses, ite = model@params$ite, c.max = model@params$max.responses - 
                1)
        
        cost.joint <- info.criterion(m.joint$posterior$Nparams, Nlog, -m.joint$free.energy, 
            criterion = information.criterion)
        # = combined: subnet + the new node
        
        ##################################################################################################### 
        
        # print(' TWO INDEPENDENT MODELS')
        cost.ind <- cost.subnet + cost.node
        
        ##################################################################################### 
        
        # print('COST-cost for two independent vs. joint model') change (increase) of the
        # total COST (cost)
        delta[[fname]] <- cost.joint - cost.ind
        
    }
    
    # Return the COST delta values. The smaller this is, the more similar the feature
    # is in regard to the subnetwork negative values mean that the subnetwork model
    # would be improved by merging the gene (= feature)
    df <- as.data.frame(delta)
    df[["feature.name"]] <- rownames(df)
    df <- df[c("feature.name", "delta")]
    df <- fsort(df, "delta")
    
    df
}

#' @title Visualize a matrix with one or two-way color scale. 
#' @description Fast investigation of matrix objects; standard visualization
#' choices are made automatically; fast and easy-to-use but does not
#' necessarily provide optimal visualization.
#' @param mat matrix
#' @param type String. Specifies visualization type. Options: 'oneway' 
#'                       (color scale ranges from white to dark red; 
#'               the color can be changed if needed); 
#'               'twoway' (color scale ranges from dark blue 
#'               through white to dark red; colors can be changed 
#'               if needed)
#' @param midpoint middle point for the color plot: smaller values are 
#'              shown with blue, larger are shown with red 
#'           in type = 'twoway'
#' @param palette Optional. Color palette.
#' @param colors Optional. Colors.
#' @param col.breaks breakpoints for the color palette
#' @param interval interval for palette color switches
#' @param plot_axes String. Indicates whether to plot 
#'               x-axis ('x'), y-axis ('y'), or both ('both').
#' @param row.tick interval for plotting row axis texts
#' @param col.tick interval for plotting column axis texts
#' @param cex.xlab use this to specify distinct font size for the x axis
#' @param cex.ylab use this to specify distinct font size for the y axis
#' @param xlab optional x axis labels
#' @param ylab optional y axis labels
#' @param limit.trunc color scale limit breakpoint
#' @param mar image margins
#' @param ... optional parameters to be passed to function 'image', see
#' help(image) for further details
#' @return A list with the color palette (colors), 
#'        color breakpoints (breaks), and palette function (palette.function)
#' @export
#' @references See citation('microbiome') 
#' @author Leo Lahti \email{microbiome-admin@@googlegroups.com}
#' @examples 
#'   mat <- rbind(c(1,2,3,4,5), c(1, 3, 1), c(4,2,2))
#'   plot_matrix(mat, 'twoway', midpoint = 3) 
#' @keywords utilities
plot_matrix <- function(mat, type = "twoway", midpoint = 0, palette = NULL, colors = NULL, 
    col.breaks = NULL, interval = 0.1, plot_axes = "both", row.tick = 1, col.tick = 1, 
    cex.xlab = 0.9, cex.ylab = 0.9, xlab = NULL, ylab = NULL, limit.trunc = 0, mar = c(5, 
        4, 4, 2), ...) {
    
    .Deprecated("plot_matrix", "microbiome", msg = "The plot_matrix function development has been moved to microbiome package.")
    
    # Center the data and color breakpoints around the specified midpoint
    mat <- mat - midpoint
    
    if (length(col.breaks) == 0) {
        m <- max(round(max(abs(mat)), limit.trunc) - interval, 0)
        
        mm <- m + interval/2
        
        vals <- seq(interval/2, mm, interval)
        
        # Set col.breaks evenly around zero
        col.breaks <- c(-(m + 1e+06), c(-rev(vals), vals), m + 1e+06)
    }
    
    my.palette <- palette
    if (is.null(palette)) {
        my.palette <- colorRampPalette(c("blue", "white", "red"), space = "rgb")
    } else if (!is(palette) == "function" && palette == "blue-black-red") {
        my.palette <- colorRampPalette(c("blue", "black", "red"), space = "rgb")
    } else if (!is(palette) == "function" && palette == "blue-white-red") {
        my.palette <- colorRampPalette(c("blue", "white", "red"), space = "rgb")
    } else if (!is(palette) == "function" && palette == "blue-white-yellow") {
        my.palette <- colorRampPalette(c("blue", "white", "yellow"), space = "rgb")
    } else if (!is(palette) == "function" && palette == "blue-black-yellow") {
        my.palette <- colorRampPalette(c("blue", "black", "yellow"), space = "rgb")
    } else if (!is(palette) == "function" && palette == "bw") {
        gray.palette <- function(int) {
            gray(seq(0, 1, length = int))
        }
        my.palette <- gray.palette
    }
    
    palette <- my.palette
    
    # if mycolors is provided it overrides palette
    if (is.null(colors)) {
        colors <- palette(length(col.breaks) - 1)
    }
    
    # transpose and revert row order to plot matrix in the same way it appears in its
    # numeric form
    par(mar = mar)
    
    matm <- matrix(mat[rev(seq(nrow(mat))), ], ncol = ncol(mat))
    dimnames(matm) <- dimnames(mat)
    mat <- matm
    
    nsamples <- ncol(mat)
    nfeats <- nrow(mat)
    if (nfeats == 1) {
        mat <- mat
    }
    image(t(mat), col = colors, xaxt = "n", yaxt = "n", zlim = range(col.breaks), 
        breaks = col.breaks, ...)
    
    if (plot_axes == "both" || plot_axes == TRUE) {
        
        if (is.null(xlab)) {
            v <- seq(1, nsamples, col.tick)  # take every nth index
            axis(1, at = seq(0, 1, length = nsamples)[v], labels = colnames(mat)[v], 
                cex.axis = cex.xlab, las = 2, ...)
        } else if (!xlab == "") {
            axis(1, at = seq(0, 1, length = length(xlab)), labels = xlab, cex.axis = cex.xlab, 
                las = 2, ...)
        }
        
        if (is.null(ylab)) {
            v <- seq(1, nfeats, row.tick)  # take every nth index
            axis(2, at = seq(0, 1, length = nfeats)[v], labels = rev(rownames(mat))[v], 
                cex.axis = cex.ylab, las = 2, ...)
        } else if (!ylab == "") {
            axis(2, at = seq(0, 1, length = nfeats), labels = rev(ylab), cex.axis = cex.ylab, 
                las = 2, ...)
        }
        
    } else if (plot_axes == "x") {
        
        if (is.null(xlab)) {
            v <- seq(1, nsamples, col.tick)  # take every nth index
            axis(1, at = seq(0, 1, length = nsamples)[v], labels = colnames(mat)[v], 
                cex.axis = cex.ylab, las = 2)
        } else {
            axis(1, at = seq(0, 1, length = nsamples), labels = ylab, cex.axis = cex.ylab, 
                las = 2)
        }
        
    } else if (plot_axes == "y") {
        
        if (is.null(ylab)) {
            v <- seq(1, nfeats, row.tick)  # take every nth index
            axis(2, at = seq(0, 1, length = nfeats)[v], labels = rev(rownames(mat))[v], 
                cex.axis = cex.xlab, las = 2)
        } else {
            axis(2, at = seq(0, 1, length = nfeats), labels = ylab, cex.axis = cex.xlab, 
                las = 2)
        }
    }
    
    # Return default margins
    par(mar = c(5, 4, 4, 2) + 0.1)
    
    return(list(colors = colors, breaks = col.breaks + midpoint, palette.function = palette))
    
}

---
title: "Introduction to the netresponse R package"
author: "Leo Lahti et al."
date: "`r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc: true
    fig_caption: yes    
  rmarkdown::md_document:
    toc: true
  rmarkdown::pdf_document:
    toc: true    
vignette: >
  %\VignetteIndexEntry{microbiome R package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include=FALSE}
library(knitr)

# Global options
opts_chunk$set(fig.path="fig/", eval=FALSE)
```

# netresponse - probabilistic tools for functional network analysis

For bug reports and maintainer contact details, see the [README](../README.md) file

## Background 

Condition-specific network activation is characteristic for cellular
systems and other real-world interaction networks. If measurements of
network states are available across a versatile set of conditions or
time points, it becomes possible to construct a global view of network
activation patterns. Different parts of the network respond to
different conditions, and in different ways. Systematic, data-driven
identification of these responses will help to obtain a holistic view
of network activity
[[1](http://bioinformatics.oxfordjournals.org/content/26/21/2713.short)-[2](http://lib.tkk.fi/Diss/2010/isbn9789526033686/)]. This
package provides robust probabilistic algorithms for functional
network analysis
[[1](http://bioinformatics.oxfordjournals.org/content/26/21/2713.short),
[3](http://www.biomedcentral.com/1752-0509/4/4)].

The methods are based on nonparametric probabilistic modeling and
variational learning, and provide general exploratory tools to
investigate the structure

NetResponse detects and characterizes
subnetworks that exhibit context-specific activation patterns across
versatile collections of functional measurements, such as gene
expression data. The implementations are partially based on the
agglomerative independent variable group analysis
([AIVGA](http://www.sciencedirect.com/science/article/pii/S0925231208000659))
and variational Dirichlet process Gaussian mixture models (Kurihara
et al. 2007). The
tools are particularly useful for global exploratory analysis of
genome-wide interaction networks and versatile collections of gene
expression data, and in general to discover subnetworks with
alternative states.


## Usage examples

Examples on running NetResponse algorithm and visualizing the
results. The algorithm combines network and functional information to
detect coherent subnetworks that reveal distinct activation modes
across conditions. 


Generate simulated data.

```{r NetResponse1a, warning=FALSE, message=FALSE}
library(netresponse)
res <- generate.toydata(Dim = 3, Nc = 3, Ns = 200, sd0 = 3, rgam.shape = 1, rgam.scale = 1)

D <- res$data
component.means <- res$means
component.sds   <- res$sds
sample2comp     <- res$sample2comp

# Use fully connected network
network <- matrix(rep(1, 9), nrow = 3) 
```

Fit NetResponse model.

```{r NetResponse1b, warning=FALSE, message=FALSE}
# Various network formats are supported, see help(detect.responses) for
# details. With large data sets, consider the 'speedup' option.
set.seed(4243)
res <- detect.responses(D, network, mixture.method = "vdp", pca.basis = TRUE)

# List subnets (each is a list of nodes)
subnet.id <- names(get.subnets(res))[[1]]
```

### PCA visualization

```{r NetResponse2, fig.width=6, fig.height=6, warning=FALSE, message=FALSE, fig.show="hide", eval=FALSE}
library(ggplot2)
vis <- plot_responses(res, subnet.id, plot_mode = "pca")
```

```{r NetResponse2b, fig.width=6, fig.height=5, warning=FALSE, message=FALSE, eval=FALSE}
# Modify the resulting ggplot2 object to enhance visualization
p <- vis$p # Pick the ggplot2 object from results
p <- p + geom_point(size = 3) # Modify point size
print(p) 
```


### Network visualization

```{r NetResponse3, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE}
vis <- plot_responses(res, subnet.id, plot_mode = "network")
```

### Heatmap visualization

```{r NetResponse4, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE}
vis <- plot_responses(res, subnet.id, plot_mode = "heatmap")
```

### Boxplot visualization

```{r NetResponse5, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE}
vis <- plot_responses(res, subnet.id, plot_mode = "boxplot_data")
```

See also mode = "response.barplot" 


### Color scale

```{r NetResponse7, fig.width=8, fig.height=8, warning=FALSE, message=FALSE, eval=FALSE}
plot_scale(vis$breaks, vis$palette, two.sided = TRUE)
```


### Cluster assignments

The sample-response assignments from the mixture model are soft
ie. defined as continuous probabilities. Retrieve the hard clustering
ie. list of samples for each response, response for each sample, based
the highest probability:

```{r NetResponse8, warning=FALSE, message=FALSE}
subnet.id <- 'Subnet-1'

# Sample - response probabilities (soft cluster assignment)
response.probs <- sample2response(res, subnet.id)
tail(round(response.probs, 6))

# Sample - response hard assignments
hard.clusters <- response2sample(res, subnet.id)
print(hard.clusters)
```

Retrieve model parameters for a given subnetwork (Gaussian mixture
means, covariance diagonal, and component weights; see
help(get.model.parameters) for details):

```{r NetResponse9, warning=FALSE, message=FALSE}
params <- get.model.parameters(res, subnet.id) 
names(params)
```

## Nonparametric Gaussian mixture models

Nonparametric Gaussian mixtures with variational Dirichlet processes
based on implementations by Kurihara et
al. (2007)
and [Honkela et
al.](http://www.sciencedirect.com/science/article/pii/S0925231208000659).

```{r vdp, warning=FALSE, message=FALSE}
# Generate 2-dimensional simulated data with 3 clusters
res <- generate.toydata(Dim = 2, Nc = 3, Ns = 200, sd0 = 3, rgam.shape = 1, rgam.scale = 1)

D <- res$data
real.means <- res$means
real.sds   <- res$sds
real.sample2comp     <- res$sample2comp

# Infinite Gaussian mixture model with       
# Variational Dirichlet Process approximation       
mixt <- vdp.mixt( D )
            
# Centroids of the detected Gaussian components       
estimated.means <- mixt$posterior$centroids

# The colors denote the known clusters
# The blue ball denotes the original (known) cluster centroids and
# the triangle denotes the estimated centroids
plot(D, col = real.sample2comp, pch = 1)
points(real.means, col = "blue", pch = 16, cex = 2)
points(estimated.means, col = "blue", pch = 17, cex = 2)
            
# Hard mixture component assignment for each sample
estimated.sample2comp <- apply(mixt$posterior$qOFz, 1, which.max)

# Compare known and estimated mixture components
# (note that cluster indices may have switched due to unidentifiability)
# nearly all samples have one-to-one match between the real and estimated 
# clusters
head(table(estimated.sample2comp, real.sample2comp))
```     

### Citing NetResponse

Please cite [Lahti et al. (2010)](http://bioinformatics.oxfordjournals.org/content/26/21/2713) with the package. 

```{r cite, warning=FALSE, message=FALSE}
citation("netresponse")
```

### Version information

This document was written using:

```{r version, warning=FALSE, message=FALSE}
sessionInfo()
```

# FIXME: something does not work here !!!

# Compare the real clusters and estimated clusters
real.clusters <- sample2comp
estimated.clusters <- sample2response(res, subnet.id, "hard")
table(real.clusters, estimated.clusters)


## Extending the subnetworks

After identifying the locally connected subnetworks, it is possible to
search for features (genes) that are similar to a given subnetwork but
not directly interacting with it. To order the remaining features
in the input data based on similarity with the subnetwork, type

```{r NetResponse10, warning=FALSE, message=FALSE}
g <- find.similar.features(res, subnet.id = "Subnet-1")
subset(g, delta < 0)
```

This gives a data frame which indicates similarity level with the
subnetwork for each feature. The smaller, the more similar. Negative
values of delta indicate the presence of coordinated responses,
positive values of delta indicate independent responses. The data
frame is ordered such that the features are listed by decreasing
similarity.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample2response.R
\name{sample2response}
\alias{sample2response}
\title{sample2response}
\usage{
sample2response(model, subnet.id, mode = 'soft')
}
\arguments{
\item{model}{Result from NetResponse (detect.responses function).}

\item{subnet.id}{Subnet identifier. A natural number which specifies one of
the subnetworks within the 'model' object.}

\item{mode}{soft: gives samples x responses probabilistic assignment matrix;
hard: gives the most likely response for each sample}
}
\value{
A matrix of probabilities. Sample-response assignments for given
subnet, listing the probability of each response, given a sample.
}
\description{
Probabilistic sample-response assignments for given subnet.
}
\examples{
data( toydata )        # Load toy data set
D    <- toydata$emat   # Response matrix (for example, gene expression)
netw <- toydata$netw   # Network

# Detect network responses
#model <- detect.responses(D, netw, verbose = FALSE)

# Assign samples to responses (soft, probabilistic assignments sum to 1)
#response.probabilities <- sample2response(model, subnet.id = 'Subnet-1')
}
\references{
Leo Lahti et al.: Global modeling of transcriptional responses
in interaction networks. Bioinformatics (2010).  See citation('netresponse')
for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factor.responses.R
\name{factor.responses}
\alias{factor.responses}
\title{Factor responses}
\usage{
factor.responses(
  annotation.vector,
  groupings,
  method = "hypergeometric",
  min.size = 2,
  data = NULL
)
}
\arguments{
\item{annotation.vector}{annotation vector with discrete factor levels, and named by the samples}

\item{groupings}{List of groupings. Each model should have a sample-cluster assignment matrix qofz, or a vector of cluster indices named by the samples.}

\item{method}{method for enrichment calculation}

\item{min.size}{minimum sample size for a response}

\item{data}{data (samples x features; or a vector in univariate case)}
}
\value{
List with each element corresponding to one factor level and listing the responses according to association strength
}
\description{
List responses for each level of the given factor
}
\examples{
res <- factor.responses(annotation.vector = NULL, groupings = NULL)
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add.ellipse.R
\name{add.ellipse}
\alias{add.ellipse}
\title{Add ellipse to an existing plot}
\usage{
add.ellipse(
  centroid,
  covmat,
  confidence = 0.95,
  npoints = 100,
  col = "black",
  ...
)
}
\arguments{
\item{centroid}{Vector with two elements defining the ellipse centroid.}

\item{covmat}{Covariance matrix for the investigated data. Only diagonal
covariances supported.}

\item{confidence}{Confidence level determining the ellipse borders based on
the covariance matrix.}

\item{npoints}{Number of plotting points.}

\item{col}{Color.}

\item{...}{Other arguments to be passed.}
}
\value{
Used for plotting side effects.
}
\description{
Calculates and plots ellipse corresponding to specified
confidence interval in 2-dimensional plot
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.network.R
\name{read.sif}
\alias{read.sif}
\title{Reading network files}
\usage{
read.sif(sif.file, format = 'graphNEL', directed = FALSE, header =
TRUE, sep = '\t', ...)
}
\arguments{
\item{sif.file}{Name of network file in SIF format.}

\item{format}{Output format: igraph or graphNEL}

\item{directed}{Logical. Directed/undirected graph. Not used in the current
model.}

\item{header}{Logical. Indicate whether the SIF file has header or not.}

\item{sep}{Field separator.}

\item{...}{Further optional arguments to be passed for file reading.}
}
\value{
R graph object in igraph or graphNEL format.
}
\description{
Function to read network files.
}
\details{
Read in SIF network file, return R graph object in igraph or graphNEL
format.
}
\examples{

#net <- read.sif('network.sif')

}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{PlotMixtureMultivariate}
\alias{PlotMixtureMultivariate}
\title{PlotMixtureMultivariate}
\usage{
PlotMixtureMultivariate(
  x,
  means,
  sds,
  ws,
  labels = NULL,
  title = NULL,
  modes = NULL,
  pca = FALSE,
  qofz = NULL,
  ...
)
}
\arguments{
\item{x}{data matrix (samples x features)}

\item{means}{mode centroids (modes x features)}

\item{sds}{mode standard deviations, assuming diagonal covariance matrices 
     (modes x features, each row giving the sqrt of covariance diagonal 
for the corresponding mode)}

\item{ws}{weight for each mode}

\item{labels}{Optional: sample class labels to be indicated in colors.}

\item{title}{title}

\item{modes}{Optional: provide sample modes for visualization already in 
the input}

\item{pca}{The data is projected on PCA plane by default (pca = TRUE). 
By setting this off (pca = FALSE) it is possible to visualize 
two-dimensional data in the original domain.}

\item{qofz}{Sample-response probabilistic assignments matrix 
(samples x responses)}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects.
}
\description{
Visualize data, centroids and response confidence intervals for a given
Gaussian mixture model with PCA. Optionally, color the samples according 
to annotations labels.
}
\examples{
#plotMixture(dat, means, sds, ws)
}
\references{
See citation('netresponse') for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect.responses.R
\name{detect.responses}
\alias{detect.responses}
\title{detect.responses}
\usage{
detect.responses(
  datamatrix,
  network = NULL,
  initial.responses = 1,
  max.responses = 10,
  max.subnet.size = 10,
  verbose = TRUE,
  prior.alpha = 1,
  prior.alphaKsi = 0.01,
  prior.betaKsi = 0.01,
  update.hyperparams = 0,
  implicit.noise = 0,
  vdp.threshold = 1e-05,
  merging.threshold = 0,
  ite = Inf,
  information.criterion = "BIC",
  speedup = TRUE,
  speedup.max.edges = 10,
  positive.edges = FALSE,
  mc.cores = 1,
  mixture.method = "vdp",
  bic.threshold = 0,
  pca.basis = FALSE,
  ...
)
}
\arguments{
\item{datamatrix}{Matrix of samples x features. For example, gene expression
matrix with conditions on the rows, and genes on the columns. The matrix
contains same features than the 'network' object, characterizing the network
states across the different samples.}

\item{network}{Binary network describing undirected pairwise interactions between
features of 'datamatrix'. The following formats are supported: binary
matrix, graphNEL, igraph, graphAM, Matrix, dgCMatrix, dgeMatrix}

\item{initial.responses}{Initial number of components for each subnetwork
model. Used to initialize calculations.}

\item{max.responses}{Maximum number of responses for each subnetwork. Can be
used to limit the potential number of network states.}

\item{max.subnet.size}{Numeric. Maximum allowed subnetwork size.}

\item{verbose}{Logical. Verbose parameter.}

\item{prior.alpha, prior.alphaKsi, prior.betaKsi}{Prior parameters for
Gaussian mixture model that is calculated for each subnetwork
(normal-inverse-Gamma prior). alpha tunes the mean; alphaKsi and betaKsi are
the shape and scale parameters of the inverse Gamma function, respectively.}

\item{update.hyperparams}{Logical. Indicate whether to update
hyperparameters during modeling.}

\item{implicit.noise}{Implicit noise parameter. Add implicit noise to vdp
mixture model. Can help to avoid overfitting to local optima, if this
appears to be a problem.}

\item{vdp.threshold}{Minimal free energy improvement after which the
variational Gaussian mixture algorithm is deemed converged.}

\item{merging.threshold}{Minimal cost value improvement required for merging
two subnetworks.}

\item{ite}{Defines maximum number of iterations on posterior update
(updatePosterior). Increasing this can potentially lead to more accurate
results, but computation may take longer.}

\item{information.criterion}{Information criterion for model selection.
Default is BIC (Bayesian Information Criterion); other options include AIC
and AICc.}

\item{speedup}{Takes advantage of approximations to PCA, mutual information
etc in various places to speed up calculations. Particularly useful with
large and densely connected networks and/or large sample size.}

\item{speedup.max.edges}{Used if speedup = TRUE. Applies prefiltering of
edges for calculating new joint models between subnetwork pairs when
potential cost changes (delta) are updated for a newly merged subnetwork and
its neighborghs. Empirical mutual information between each such subnetwork
pair is calculated based on their first principal components, and joint
models will be calculated only for the top candidates up to the number
specified by speedup.max.edges. It is expected that the subnetwork pair that
will benefit most from joint modeling will be among the top mutual
infomation candidates. This way it is possible to avoid calculating
exhaustive many models on the network hubs.}

\item{positive.edges}{Consider only the edges with positive association. Currently measured with Spearman correlation.}

\item{mc.cores}{Number of cores to be used in parallelization. See
help(mclapply) for details.}

\item{mixture.method}{Specify the approach to use in mixture modeling.
Options. vdp (nonparametric Variational Dirichlet process mixture model);
bic (based on Gaussian mixture modeling with EM, using BIC to select the
optimal number of components)}

\item{bic.threshold}{BIC threshold which needs to be exceeded before a new mode is added to the mixture with mixture.method = "bic"}

\item{pca.basis}{Transform data first onto PCA basis to try to avoid problems with non-diagonal covariances.}

\item{...}{Further optional arguments to be passed.}
}
\value{
NetResponseModel object.
}
\description{
Main function of the NetResponse algorithm. 
Detect condition-specific network responses, given
network and a set of measurements of node activity in a set of
conditions. Returns a set of subnetworks and their estimated
context-specific responses.
}
\examples{
\dontrun{
  #data(toydata)        # Load toy data set
  #D    <- toydata$emat   # Response matrix (for example, gene expression)
  #netw <- toydata$netw   # Network

  # Run NetReponse algorithm
  # model <- detect.responses(D, netw, verbose = FALSE)
}
}
\references{
See citation("netresponse").
}
\author{
Maintainer: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{iteration}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkg-package.R
\docType{class}
\name{NetResponseModel-class}
\alias{NetResponseModel-class}
\alias{[[,NetResponseModel-method}
\alias{show,NetResponseModel-method}
\title{Class 'NetResponseModel'}
\description{
A NetResponse model.
}
\section{Objects from the Class}{
 Returned by \code{\link{detect.responses}}
function.
}

\examples{
showClass('NetResponseModel')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response.enrichment.R
\name{response.enrichment}
\alias{response.enrichment}
\title{Enrichment for a specified sample group in the given response.}
\usage{
response.enrichment(
  total.samples,
  response.samples,
  annotated.samples,
  method = "hypergeometric"
)
}
\arguments{
\item{total.samples}{All samples in the data}

\item{response.samples}{Samples in the investigated subset}

\item{annotated.samples}{Samples at the investigated annotation level for enrichment calculation}

\item{method}{Enrichment method.}
}
\value{
List with enrichment statistics, depending on enrichment method.
}
\description{
Calculate enrichment values for a specified sample group in the given
response.
}
\examples{
#enr <- response.enrichment(subnet.id, models, sample, response, method)
}
\references{
See citation('netresponse')
}
\seealso{
order.responses
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkg-package.R
\docType{data}
\name{dna}
\alias{dna}
\title{Dna damage data set (PPI and expression)}
\format{
List of following objects: \describe{ \item{ppi}{PPI data matrix}
\item{exp}{gene expression profiles data matrix} \item{gids}{Vector of gene
ids corresponding to indices used in data matrices} \item{obs}{Gene
expression observation details} \item{combined.links}{pooled matrix of PPI
and expression links} }
}
\source{
PPI data pooled from yeast data sets of [1] and [2]. Dna damage
expression set of [3].
}
\usage{
data(dna)
}
\description{
A combined yeast data set with protein-protein interactions and gene expression (dna damage). Gene expression profiles are transformed into links by computing a Pearson correlation for all pairs of genes and treating all correlations above 0.85 as additional links. Number of genes: 1823, number of interactions: 12382, number of gene expression observations: 52, number of total links with PPI and expression links: 15547.
}
\examples{
data(dna)
}
\references{
Ulitsky, I. and Shamir, R. \emph{Identification of functional
modules using network topology and high-throughput data.} BMC Systems
Biology 2007, 1:8.

Nariai, N., Kolaczyk, E. D. and Kasif, S. \emph{Probabilistic Protein
Function Predition from Heterogenous Genome-Wide Data}. PLoS ONE 2007,
2(3):e337.

Gasch, A., Huang, M., Metzner, S., Botstein, D. and Elledge, S.
\emph{Genomic expression responses to DNA-damaging agents and the regulatory
role of the yeast ATR homolog Mex1p.} Molecular Biology of the Cell 2001,
12:2987-3003.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vdp.mixt.R
\name{vdp.mixt}
\alias{vdp.mixt}
\title{vdp.mixt}
\usage{
vdp.mixt(
  dat,
  prior.alpha = 1,
  prior.alphaKsi = 0.01,
  prior.betaKsi = 0.01,
  do.sort = TRUE,
  threshold = 1e-05,
  initial.K = 1,
  ite = Inf,
  implicit.noise = 0,
  c.max = 10,
  speedup = TRUE,
  min.size = 5
)
}
\arguments{
\item{dat}{Data matrix (samples x features).}

\item{prior.alpha, prior.alphaKsi, prior.betaKsi}{Prior parameters for
Gaussian mixture model (normal-inverse-Gamma prior). alpha tunes the mean;
alphaKsi and betaKsi are the shape and scale parameters of the inverse Gamma
function, respectively.}

\item{do.sort}{When true, qOFz will be sorted in decreasing fashion by
component size, based on colSums(qOFz). The qOFz matrix describes the
sample-component assigments in the mixture model.}

\item{threshold}{Defines the minimal free energy improvement that stops the
algorithm: used to define convergence limit.}

\item{initial.K}{Initial number of mixture components.}

\item{ite}{Defines maximum number of iterations on posterior update
(updatePosterior). Increasing this can potentially lead to more accurate
results, but computation may take longer.}

\item{implicit.noise}{Adds implicit noise; used by vdp.mk.log.lambda.so and
vdp.mk.hp.posterior.so. By adding noise (positive values), one can avoid
overfitting to local optima in some cases, if this happens to be a problem.}

\item{c.max}{Maximum number of candidates to consider in
find.best.splitting. During mixture model calculations new mixture
components can be created until this upper limit has been reached. Defines
the level of truncation for a truncated stick-breaking process.}

\item{speedup}{When learning the number of components, each component is
splitted based on its first PCA component. To speed up, approximate by using
only subset of data to calculate PCA.}

\item{min.size}{Minimum size for a component required for potential
splitting during mixture estimation.}
}
\value{
\item{ prior }{Prior parameters of the vdp-gm model (qofz: priors on observation lables; Mu: centroids; S2: variance).} 
        \item{ posterior }{Posterior estimates for the model parameters and statistics.} 
        \item{ weights }{Mixture proportions, or weights, for the Gaussian mixture components.}
 \item{ centroids }{Centroids of the mixture components.} 
 \item{ sds }{ Standard deviations for the mixture model components (posterior modes of the covariance diagonals square root). Calculated as sqrt(invgam.scale/(invgam.shape + 1)). } 
 \item{ qOFz }{ Sample-to-cluster assigments (soft probabilistic associations).} 
\item{ Nc }{Component sizes}
 \item{ invgam.shape }{ Shape parameter (alpha) of the inverse Gamma distribution } 
 \item{ invgam.scale }{ Scale parameter (beta) of the inverse Gamma distribution } 
 \item{ Nparams }{ Number of model parameters }
\item{ K }{ Number of components in the mixture model } 
 \item{ opts }{Model parameters that were used.} 
 \item{ free.energy }{Free energy of the model.}
}
\description{
Accelerated variational Dirichlet process Gaussian mixture.
}
\details{
Implementation of the Accelerated variational Dirichlet process Gaussian
mixture model algorithm by Kenichi Kurihara et al., 2007.

ALGORITHM SUMMARY 
This code implements Gaussian mixture models with diagonal covariance matrices. 
The following greedy iterative approach is taken in order to obtain the number
of mixture models and their corresponding parameters:

1. Start from one cluster, $T = 1$.
2. Select a number of candidate clusters according to their values of 
   'Nc' = \\sum_{n=1}^N q_{z_n} (z_n = c) (larger is better).
3. For each of the candidate clusters, c: 
    3a. Split c into two clusters, c1 and c2, through the bisector of its 
        principal component. Initialise the responsibilities 
        q_{z_n}(z_n = c_1) and q_{z_n}(z_n = c_2). 
    3b. Update only the parameters of c1 and c2 using the observations that
        belonged to c, and determine the new value for the free energy, F{T+1}.
    3c. Reassign cluster labels so that cluster 1 corresponds to the largest 
        cluster, cluster 2 to the second largest, and so on.
4. Select the split that lead to the maximal reduction of free energy, F{T+1}.
5. Update the posterior using the newly split data.
6. If FT - F{T+1} < \\epsilon then halt, else set T := T +1 and go to step 2.

The loop is implemented in the function greedy(...)
}
\note{
This implementation is based on the Variational Dirichlet Process
  Gaussian Mixture Model implementation, Copyright (C) 2007 Kenichi Kurihara
  (all rights reserved) and the Agglomerative Independent Variable Group
  Analysis package (in Matlab): Copyright (C) 2001-2007 Esa Alhoniemi, Antti
  Honkela, Krista Lagus, Jeremias Seppa, Harri Valpola, and Paul Wagner.
}
\examples{

  set.seed(123)

  # Generate toy data with two Gaussian components
  dat <- rbind(array(rnorm(400), dim = c(200,2)) + 5,
               array(rnorm(400), dim = c(200,2)))

  # Infinite Gaussian mixture model with 
  # Variational Dirichlet Process approximation
  mixt <- vdp.mixt( dat )

  # Centroids of the detected Gaussian components
  mixt$posterior$centroids

  # Hard mixture component assignments for the samples
  apply(mixt$posterior$qOFz, 1, which.max)


}
\references{
Kenichi Kurihara, Max Welling and Nikos Vlassis: Accelerated
  Variational Dirichlet Process Mixtures.  In B. Sch\'olkopf and J. Platt and
  T. Hoffman (eds.), Advances in Neural Information Processing Systems 19,
  761--768. MIT Press, Cambridge, MA 2007.
}
\author{
Maintainer: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{iteration}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response2sample.R
\name{response2sample}
\alias{response2sample}
\title{response2sample}
\usage{
response2sample(
  model,
  subnet.id = NULL,
  component.list = TRUE,
  verbose = FALSE,
  data = NULL
)
}
\arguments{
\item{model}{A NetResponseModel object or list.}

\item{subnet.id}{Subnet id. A natural number which specifies one of the
subnetworks within the 'model' object.}

\item{component.list}{List samples separately for each mixture component
(TRUE). Else list the most strongly associated component for each sample
(FALSE).}

\item{verbose}{Follow progress by intermediate messages.}

\item{data}{Data (features x samples; or a vector for univariate case) to predict response for given data points (currently implemented only for mixture.model output)

Return:}
}
\value{
A list. Each element corresponds to one subnetwork response, and
contains a list of samples that are associated with the response (samples
for which this response has the highest probability P(response | sample)).
}
\description{
List the most strongly associated response of a given subnetwork for each sample.
}
\examples{
# Load example data
data( toydata )         # Load toy data set
D    <- toydata$emat    # Response matrix (for example, gene expression)
model <- toydata$model  # Pre-calculated model

# Find the samples for each response (for a given subnetwork)
response2sample(model, subnet.id = 1)

}
\references{
Leo Lahti et al.: Global modeling of transcriptional responses
in interaction networks. Bioinformatics (2010).  See citation('netresponse')
for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centerData.R
\name{centerData}
\alias{centerData}
\title{Center data matrix.}
\usage{
centerData(X, rm.na = TRUE, meanvalue = NULL)
}
\arguments{
\item{X}{The data set: samples x features. Each feature will be centered.}

\item{rm.na}{Ignore NAs.}

\item{meanvalue}{Can be used to set a desired center value. The default is 0.}
}
\value{
Centered data matrix.
}
\description{
Center data matrix to 0 for each variable by removing the means.
}
\note{
Note that the model assumes samples x features matrix, and centers
each feature.
}
\examples{
centerData(matrix(rnorm(100), 10, 10))
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{maths}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find.similar.features.R
\name{find.similar.features}
\alias{find.similar.features}
\title{Find similar features with a given subnetwork.}
\usage{
find.similar.features(model, subnet.id, datamatrix = NULL, verbose =
FALSE, information.criterion = NULL)
}
\arguments{
\item{model}{NetResponseModel object.}

\item{subnet.id}{Investigated subnetwork.}

\item{datamatrix}{Optional. Can be used to compare subnetwork similarity
with new data which was not used for learning the subnetworks.}

\item{verbose}{Logical indicating whether progress of the algorithm should
be indicated on the screen.}

\item{information.criterion}{Information criterion for model selection. By
default uses the same than in the 'model' object.}
}
\value{
A data frame with elements feature.names (e.g. gene IDs) and delta,
which indicates similarity level. See details for details. The smaller, the
more similar. The data frame is ordered such that the features are listed by
decreasing similarity.
}
\description{
Given subnetwork, orders the remaining features (genes) in the input data
based on similarity with the subnetwork. Allows the identification of
similar features that are not directly connected in the input network.
}
\details{
The same similarity measure is used as when agglomerating the subnetworks:
the features are ordered by delta (change) in the cost function, assuming
that the feature would be merged in the subnetwork. The smaller the change,
the more similar the feature is (change would minimize the new cost function
value). Negative values of delta mean that the cost function would be
improved by merging the new feature in the subnetwork, indicating features
having coordinated response.
}
\examples{

data(toydata)
model <- toydata$model
subnet.id <- 'Subnet-1' 
# g <- find.similar.features(model, subnet.id)
# List features that are similar to this subnetwork (delta < 0)
# (ordered by decreasing similarity)
# subset(g, delta < 0)

}
\references{
See citation('netresponse') for reference details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plot_associations}
\alias{plot_associations}
\title{Association strength between category labels and responses.}
\usage{
plot_associations(
  x,
  subnet.id,
  labels,
  method = "hypergeometric",
  mode = "group.by.classes",
  ...
)
}
\arguments{
\item{x}{NetResponseModel object}

\item{subnet.id}{Subnetwork.}

\item{labels}{Factor. Labels for the data samples. Name by samples, or
provide in the same order as in the original data.}

\item{method}{Method to calculate association strength.}

\item{mode}{group.by.responses or group.by.classes: indicate barplot
grouping type.}

\item{...}{Other arguments to be passed for plot_}
}
\value{
Used for side effect (plotting).
}
\description{
Plot association strength between user-defined category
labels and responses in a selected subnetwork. Associations are showm
in terms -log10(p) enrichment values for the annotation categories for the
responses within the specified subnetwork. No correction for multiple
testing.
}
\examples{
#
}
\references{
See citation('netresponse').
}
\seealso{
plot_responses
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.responses.factor.minimal.R
\name{list.responses.factor.minimal}
\alias{list.responses.factor.minimal}
\title{List factor responses (minimal)}
\usage{
list.responses.factor.minimal(
  annotation.df,
  groupings,
  method = "hypergeometric",
  min.size = 2,
  pth = Inf,
  verbose = TRUE,
  data = NULL,
  rounding = NULL
)
}
\arguments{
\item{annotation.df}{annotation data.frame with discrete factor levels, rows
named by the samples}

\item{groupings}{List of groupings. Each model should have a sample-cluster assignment matrix qofz, or a vector of cluster indices named by the samples.}

\item{method}{method for enrichment calculation}

\item{min.size}{minimum sample size for a response}

\item{pth}{p-value threshold; applied to adjusted p-value}

\item{verbose}{verbose}

\item{data}{data (samples x features; or a vector in univariate case)}

\item{rounding}{rounding digits}
}
\value{
A list with two elements: Table listing all associations between the factor levels and responses; multiple p-value adjustment method
}
\description{
List significantly associated responses for all factors and levels in the given annotation matrix
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkg-package.R
\docType{data}
\name{osmo}
\alias{osmo}
\title{Osmoshock data set (PPI and expression)}
\format{
List of following objects: \describe{ \item{ppi}{PPI data matrix}
\item{exp}{gene expression profiles data matrix} \item{gids}{Vector of gene
ids corresponding to indices used in data matrices} \item{obs}{Gene
expression observation details} \item{combined.links}{pooled matrix of PPI
and expression links} }
}
\source{
PPI data pooled from yeast data sets of [1] and [2]. Dna damage
expression set of [3].
}
\usage{
data(osmo)
}
\description{
A combined yeast data set with protein-protein interactions and gene expression (osmotick shock response). Gene expression profiles are transformed into links by computing a Pearson correlation for all pairs of genes and treating all correlations above 0.85 as additional links. Number of genes: 1711, number of interactions: 10250, number of gene expression observations: 133, number of total links with PPI and expression links: 14256.
}
\examples{
data(osmo)
}
\references{
Ulitsky, I. and Shamir, R. \emph{Identification of functional
modules using network topology and high-throughput data.} BMC Systems
Biology 2007, 1:8.

Nariai, N., Kolaczyk, E. D. and Kasif, S. \emph{Probabilistic Protein
Function Predition from Heterogenous Genome-Wide Data}. PLoS ONE 2007,
2(3):e337.

O'Rourke, S. and Herskowitz, I. \emph{Unique and redundant roles for Hog
MAPK pathway components as revealed by whole-genome expression analysis.}
Molecular Biology of the Cell 2004, 15:532-42.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.responses.factor.R
\name{list.responses.factor}
\alias{list.responses.factor}
\title{List significant responses}
\usage{
list.responses.factor(
  annotation.df,
  models,
  method = "hypergeometric",
  min.size = 2,
  qth = Inf,
  verbose = TRUE,
  data = NULL,
  rounding = NULL
)
}
\arguments{
\item{annotation.df}{annotation data.frame with discrete factor levels, rows named by the samples}

\item{models}{List of models. Each model should have a sample-cluster assignment matrix qofz, or a vector of cluster indices named by the samples.}

\item{method}{method for enrichment calculation}

\item{min.size}{minimum sample size for a response}

\item{qth}{q-value threshold}

\item{verbose}{verbose}

\item{data}{data (samples x features; or a vector in univariate case)}

\item{rounding}{rounding digits}
}
\value{
Table listing all associations between the factor levels and responses
}
\description{
List significantly associated responses for all factors and levels in the given annotation matrix
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{write.netresponse.results}
\alias{write.netresponse.results}
\title{Write NetResponse results summary into a file.}
\usage{
write.netresponse.results(x, subnet.ids = NULL, filename)
}
\arguments{
\item{x}{NetResponseModel}

\item{subnet.ids}{List of subnet ids to consider. By default, all subnets.}

\item{filename}{Output file name.}
}
\value{
Used for side effects.
}
\description{
Experimental version.
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixture.model.R
\name{mixture.model}
\alias{mixture.model}
\title{Mixture model}
\usage{
mixture.model(
  x,
  mixture.method = "vdp",
  max.responses = 10,
  implicit.noise = 0,
  prior.alpha = 1,
  prior.alphaKsi = 0.01,
  prior.betaKsi = 0.01,
  vdp.threshold = 1e-05,
  initial.responses = 1,
  ite = Inf,
  speedup = TRUE,
  bic.threshold = 0,
  pca.basis = FALSE,
  min.responses = 1,
  ...
)
}
\arguments{
\item{x}{data matrix (samples x features, for multivariate analysis) or a vector (for univariate analysis)}

\item{mixture.method}{Specify the approach to use in mixture modeling. Options. vdp (nonparametric Variational Dirichlet process mixture model); bic (based on Gaussian mixture modeling with EM, using BIC to select the optimal number of components)}

\item{max.responses}{Maximum number of responses for each subnetwork. Can be used to limit the potential number of network states.}

\item{implicit.noise}{Implicit noise parameter. Add implicit noise to vdp
mixture model. Can help to avoid overfitting to local optima, if this
appears to be a problem.}

\item{prior.alpha, prior.alphaKsi, prior.betaKsi}{Prior parameters for
Gaussian mixture model that is calculated for each subnetwork
(normal-inverse-Gamma prior). alpha tunes the mean; alphaKsi and betaKsi are
the shape and scale parameters of the inverse Gamma function, respectively.}

\item{vdp.threshold}{Minimal free energy improvement after which the
variational Gaussian mixture algorithm is deemed converged.}

\item{initial.responses}{Initial number of components for each subnetwork
model. Used to initialize calculations.}

\item{ite}{Maximum number of iterations on posterior update
(updatePosterior). Increasing this can potentially lead to more accurate results, but computation may take longer.}

\item{speedup}{Takes advantage of approximations to PCA, mutual information
 etc in various places to speed up calculations. Particularly useful with
large and densely connected networks and/or large sample size.}

\item{bic.threshold}{BIC threshold which needs to be exceeded before a new mode is added to the mixture with mixture.method = "bic"}

\item{pca.basis}{pca.basis}

\item{min.responses}{minimum number of responses}

\item{...}{Further optional arguments to be passed.}
}
\value{
List with two elements: model: fitted mixture model (parameters and free energy); model.params: model parameters
}
\description{
Fit Gaussian mixture model
}
\examples{
res <- mixture.model(NULL)
}
\references{
See citation("netresponse")
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/order.responses.R
\name{list.significant.responses}
\alias{list.significant.responses}
\title{Listing significant responses}
\usage{
list.significant.responses(model, sample, qth = 1, method = "hypergeometric")
}
\arguments{
\item{model}{NetResponseModel object.}

\item{sample}{User-specified samples group for which the enrichments are
calculated. For instance, an annotation category.}

\item{qth}{q-value threshold for enrichments}

\item{method}{Enrichment method.}
}
\value{
Statistics of the significantly associated responses.
}
\description{
List responses with significant associations to a given sample group.
}
\examples{
# 
}
\references{
See citation('netresponse')
}
\seealso{
response.enrichment
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.stats.R
\name{model.stats}
\alias{model.stats}
\title{model.stats}
\usage{
model.stats(models)
}
\arguments{
\item{models}{NetResponse object or list of models}
}
\value{
A 'subnetworks x properties' data frame containing the following elements.
\item{subnet.size: }{ Vector of subnetwork sizes. } \item{subnet.responses:
}{ Vector giving the number of responses in each subnetwork. }
}
\description{
Subnetwork statistics: size and number of distinct responses for each subnet.
}
\examples{
# Load a pre-calculated netresponse model obtained with 
# model <- detect.responses(toydata$emat, toydata$netw, verbose = FALSE)
data(toydata)        
# Calculate summary statistics for the model
stat <- model.stats(toydata$model)


}
\references{
Leo Lahti et al.: Global modeling of transcriptional responses
in interaction networks. Bioinformatics (2010).  See citation('netresponse')
for reference details.
}
\author{
Leo Lahti <leo.lahti@iki.fi>
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{set.breaks}
\alias{set.breaks}
\title{Set breaks}
\usage{
set.breaks(mat, interval = 0.1)
}
\arguments{
\item{mat}{Matrix to visualize.}

\item{interval}{Density of color breakpoints.}
}
\value{
A vector listing the color breakpoints.
}
\description{
Set breakpoints for two-way color palette.
}
\examples{
 set.breaks(array(rnorm(100), dim = c(10, 10)), interval = .1) 
}
\references{
L. Lahti et al.: Global modeling of transcriptional responses in
interaction networks. Submitted.
}
\author{
Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao. Maintainer:
Leo Lahti <leo.lahti@iki.fi>
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.responses.continuous.multi.R
\name{list.responses.continuous.multi}
\alias{list.responses.continuous.multi}
\title{Investigate association of a continuous variable and the modes}
\usage{
list.responses.continuous.multi(
  annotation.df,
  groupings,
  method = "t-test",
  pth = Inf,
  verbose = TRUE,
  rounding = NULL
)
}
\arguments{
\item{annotation.df}{annotation data.frame with discrete factor levels, rows
named by the samples}

\item{groupings}{Sample mode information. Each element corresponds to one grouping; each grouping lists samples for the modes within that grouping.}

\item{method}{method for quantifying the association}

\item{pth}{p-value threshold applied to adjusted p-values}

\item{verbose}{verbose}

\item{rounding}{rounding digits}
}
\value{
Table listing all associations between the factor levels and responses
}
\description{
Investigate association of a continuous variable and the modes given a list of groupings
}
\examples{
res <- list.responses.continuous.multi(annotation.df = NULL, groupings = NULL)
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/order.responses.R
\name{order.responses}
\alias{order.responses}
\title{order.responses}
\usage{
order.responses(
  models,
  sample,
  method = "hypergeometric",
  min.size = 2,
  max.size = Inf,
  min.responses = 2,
  subnet.ids = NULL,
  verbose = FALSE,
  data = NULL
)
}
\arguments{
\item{models}{List of models. Each model should have a sample-cluster assignment matrix qofz.}

\item{sample}{Measure enrichment of this sample (set) across the observed
responses.}

\item{method}{'hypergeometric' measures enrichment of factor levels in this
response; 'precision' measures response purity for each factor level;
'dependency' measures logarithm of the joint density between response and
factor level vs. their marginal densities: log(P(r,s)/(P(r)P(s)))}

\item{min.size, max.size, min.responses}{Optional parameters to filter the
results based on subnet size and number of responses.}

\item{subnet.ids}{Specify subnets for which the responses shall be ordered.
By default, use all subnets.}

\item{verbose}{Follow progress by intermediate messages.}

\item{data}{data (samples x features; or a vector in univariate case)}
}
\value{
A data frame with elements 'ordered.responses' which gives a data
  frame of responses ordered by enrichment score for the investigated sample.
  The subnetwork, response id and enrichment score are shown. 
  The method field indicates the enrichment calculation method. 
  The sample field lists the samples et for which the enrichments 
  were calculated. 
  The info field lists additional information on enrichment statistics.
}
\description{
Orders the responses by association strength (enrichment score) to a given sample set. For instance, if the samples correspond to a particular experimental factor, this function can be used to prioritize the responses according to their association strength to this factor.
}
\note{
Tools for analyzing end results of the model.
}
\examples{
res <- order.responses(models = NULL, sample = NULL)
# - for given sample/s (factor level), 
#     order responses (across all subnets) by association strength 
#     (enrichment score); overrepresentation
# order.responses(model, sample, method  = 'hypergeometric') 
}
\references{
See citation('netresponse') for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pick.model.parameters.R
\name{pick.model.parameters}
\alias{pick.model.parameters}
\title{pick.model.parameters}
\usage{
pick.model.parameters(m, nodes)
}
\arguments{
\item{m}{vdp.mixt output}

\item{nodes}{node names for naming purposes}
}
\value{
Model parameters
}
\description{
Pick model parameters
}
\examples{
pick.model.parameters(m = NULL, nodes = NULL) 
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.matrix.R
\name{check.matrix}
\alias{check.matrix}
\title{check.matrix}
\usage{
check.matrix(datamatrix)
}
\arguments{
\item{datamatrix}{See detect.responses}
}
\value{
The datamatrix, possibly added with necessary formatting for the
netresponse algorithm.
}
\description{
Mostly for internal purposes. Check input matrix format.
}
\examples{
# datamatrix <- check.matrix(datamatrix)
}
\references{
See citation('netresponse')
}
\seealso{
detect.responses
}
\author{
Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao.  Maintainer:
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic.select.best.mode.R
\name{bic.select.best.mode}
\alias{bic.select.best.mode}
\title{Select best mode with BIC}
\usage{
bic.select.best.mode(x, max.modes = 1, bic.threshold = 1, min.modes = 1)
}
\arguments{
\item{x}{dat vector (for univariate analysis) or a matrix (for multivariate analysis)}

\item{max.modes}{Maximum number of modes to be checked for mixture model selection}

\item{bic.threshold}{BIC threshold which needs to be exceeded before a new mode is added to the mixture.}

\item{min.modes}{Optional. Minimum number of modes.}
}
\value{
Fitted latent class model (parameters and free energy)
}
\description{
Select optimal number of mixture components by adding components until 
the increase in objective function is below threshold.
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.netw.R
\name{remove.negative.edges}
\alias{remove.negative.edges}
\title{remove.negative.edges}
\usage{
remove.negative.edges(network, delta, datamatrix)
}
\arguments{
\item{network}{network}

\item{delta}{associated cost function value changes for each node merge}

\item{datamatrix}{datamatrix}
}
\value{
Filtered network
}
\description{
Remove edges where the connected nodes correlate negatively
}
\examples{
#
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NetResponseModel-accessors.R
\name{get.dat,NetResponseModel-method}
\alias{get.dat,NetResponseModel-method}
\alias{get.dat}
\title{Get subnetwork data}
\usage{
\S4method{get.dat}{NetResponseModel}(model, subnet.id, sample = NULL)
}
\arguments{
\item{model}{Result from NetResponse (detect.responses function).}

\item{subnet.id}{Subnet identifier. A natural number which specifies one of
the subnetworks within the 'model' object.}

\item{sample}{Define the retrieved samples}
}
\value{
Subnet data matrix
}
\description{
Get subnetwork data
}
\examples{
## Load a pre-calculated netresponse model obtained with 
# model <- detect.responses(toydata$emat, toydata$netw, verbose = FALSE)
# data( toydata ); get.dat(toydata$model) 
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.netw.R
\name{filter.network}
\alias{filter.network}
\title{filter.network}
\usage{
filter.network(network, delta, datamatrix, params)
}
\arguments{
\item{network}{network}

\item{delta}{associated cost function value changes for each node merge}

\item{datamatrix}{datamatrix}

\item{params}{parameters}
}
\value{
Filtered network
}
\description{
Include to the network only the edges with the highest mutual information,
calculated based on the first principal components.
}
\examples{
#
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listify.groupings.R
\name{listify.groupings}
\alias{listify.groupings}
\title{Convert grouping info into a list; each element corresponds to a
group and lists samples in that group.}
\usage{
listify.groupings(groupings, verbose = FALSE)
}
\arguments{
\item{groupings}{a list, a vector, or a samplesxmodes assignment matrix}

\item{verbose}{verbose}
}
\value{
Group list
}
\description{
Convert grouping info into a list; each element corresponds to a
group and lists samples in that group.
}
\examples{
res <- listify.groupings(groupings = NULL)
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plot_subnet}
\alias{plot_subnet}
\title{plot_subnet}
\usage{
plot_subnet(x, subnet.id, network, plot_names = TRUE, ...)
}
\arguments{
\item{x}{Result from NetResponse (detect.responses function).}

\item{subnet.id}{Subnet id.}

\item{network}{Original network used in the modelling.}

\item{plot_names}{Plot node names (TRUE) or indices (FALSE).}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects. Returns a matrix that describes the
investigated subnetwork.
}
\description{
Plot the given subnetwork.
}
\examples{
#
# res <- detect.responses(D, netw, verbose = FALSE)
# net <- plot_subnet(res, subnet.idx = 1)
}
\references{
L. Lahti et al.: Global modeling of transcriptional responses in
interaction networks. Submitted.
}
\author{
Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao. Maintainer:
Leo Lahti <leo.lahti@iki.fi>
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotMixture.R
\name{PlotMixture}
\alias{PlotMixture}
\title{Plot mixtures}
\usage{
PlotMixture(
  x,
  qofz,
  binwidth = 0.05,
  xlab.text = NULL,
  ylab.text = NULL,
  title.text = NULL
)
}
\arguments{
\item{x}{data vector}

\item{qofz}{Mode assignment probabilities for each sample. Samples x modes.}

\item{binwidth}{binwidth for histogram}

\item{xlab.text}{xlab.text}

\item{ylab.text}{ylab.text}

\item{title.text}{title.text}
}
\value{
Used for its side-effects
}
\description{
Plot mixtures.
}
\examples{
# PlotMixture(x, qofz)
}
\references{
See citation('netresponse') for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plot_expression}
\alias{plot_expression}
\title{plot_expression}
\usage{
plot_expression(x, maintext, ...)
}
\arguments{
\item{x}{samples x features matrix}

\item{maintext}{main title}

\item{...}{optional arguments}
}
\value{
Used for its side effects.
}
\description{
Plot expression matrix in color scale. For one-channel data;
plot expression
of each gene relative to its mean expression level over all samples. Blue
indicates decreased expression and red indicates increased expression.
Brightness of the color indicates magnitude of the change. Black denotes no
change.
}
\examples{
#plot_expression(x)
}
\references{
See citation('netresponse').
}
\seealso{
\code{\link{plot_scale}}
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plot_response}
\alias{plot_response}
\title{plot_response}
\usage{
plot_response(
  x,
  mynet,
  mybreaks,
  mypalette,
  plot_names = TRUE,
  colors = TRUE,
  plot_type = "twopi",
  ...
)
}
\arguments{
\item{x}{A numerical vector, or NULL.}

\item{mynet}{Binary matrix specifying the interactions between nodes.}

\item{mybreaks}{Specify breakpoints for color plot_}

\item{mypalette}{Specify palette for color plot_}

\item{plot_names}{Plot node names (TRUE) or indices (FALSE).}

\item{colors}{Plot colors. Logical.}

\item{plot_type}{Network plot mode. For instance, 'neato' or 'twopi'.}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects.
}
\description{
Plot a specific transcriptional response for a given subnetwork.
TRUE, colors = TRUE, plot_type = 'twopi', ...)
}
\examples{
#tmp <- plot_response(model, mynet, 
#  \tmaintext = paste('Subnetwork', subnet.id))
}
\references{
L. Lahti et al.: Global modeling of transcriptional responses in
interaction networks. Submitted.
}
\author{
Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao. Maintainer:
Leo Lahti <leo.lahti@iki.fi>
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment.list.factor.minimal.R
\name{enrichment.list.factor.minimal}
\alias{enrichment.list.factor.minimal}
\title{enrichment.list.factor}
\usage{
enrichment.list.factor.minimal(
  groupings,
  method,
  verbose = FALSE,
  annotation.vector,
  level
)
}
\arguments{
\item{groupings}{List of groupings. Each model should have a sample-cluster assignment matrix qofz.}

\item{method}{'hypergeometric' measures enrichment of factor levels in this
response; 'precision' measures response purity for each factor level;
'dependency' measures logarithm of the joint density between response and
factor level vs. their marginal densities: log(P(r,s)/(P(r)P(s)))}

\item{verbose}{Follow progress by intermediate messages.}

\item{annotation.vector}{annotation vector}

\item{level}{level}
}
\value{
A data frame which gives a data
  frame of responses ordered by enrichment score for the investigated sample.
  The model, response id and enrichment score are shown. The method field
  indicates the enrichment calculation method. The sample field lists the
  samples et for which the enrichments were calculated. The info field lists
  additional information on enrichment statistics.
}
\description{
Orders the responses by association strength (enrichment score) to a given sample set. For instance, if the samples correspond to a particular experimental factor, this function can be used to prioritize the responses according to their association strength to this factor.
}
\examples{
res <- enrichment.list.factor.minimal(groupings = NULL,
    method = NULL,
    annotation.vector = NULL,
    level = NULL)

}
\references{
See citation('netresponse') for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotMixtureUnivariate.R
\name{PlotMixtureUnivariate}
\alias{PlotMixtureUnivariate}
\title{Plot univariate mixtures}
\usage{
PlotMixtureUnivariate(
  x,
  means = NULL,
  sds = NULL,
  ws = NULL,
  title.text = NULL,
  xlab.text = NULL,
  ylab.text = NULL,
  binwidth = 0.05,
  qofz = NULL,
  density.color = "darkgray",
  cluster.assignments = NULL,
  ...
)
}
\arguments{
\item{x}{data vector}

\item{means}{mode centroids}

\item{sds}{mode standard deviations}

\item{ws}{weight for each mode}

\item{title.text}{Plot title}

\item{xlab.text}{xlab.text}

\item{ylab.text}{ylab.text}

\item{binwidth}{binwidth for histogram}

\item{qofz}{Mode assignment probabilities for each sample. Samples x modes.}

\item{density.color}{Color for density lines}

\item{cluster.assignments}{Vector of cluster indices, indicating cluster for each data point}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects
}
\description{
Visualize data, centroids and stds for a given univariate Gaussian mixture model with PCA.
}
\examples{
# plotMixtureUnivariate(dat, means, sds, ws)
}
\references{
See citation('netresponse') for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.model.pair.R
\name{update.model.pair}
\alias{update.model.pair}
\title{update.model.pair}
\usage{
update.model.pair(datamatrix, delta, network, edge, network.nodes, G,
params, model.nodes, model.pairs)
}
\arguments{
\item{datamatrix}{datamatrix}

\item{delta}{delta}

\item{network}{network}

\item{edge}{edge}

\item{network.nodes}{network.nodes}

\item{G}{G}

\item{params}{params}

\item{model.nodes}{model.nodes}

\item{model.pairs}{model.pairs

Return:}
}
\value{
\item{model.pairs }{model.pairs} \item{delta }{delta}
}
\description{
Mainly for internal use. Calculate joint model for given node pair and
update delta accordingly.
}
\examples{
#
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao.  Maintainer:
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/continuous.responses.R
\name{continuous.responses}
\alias{continuous.responses}
\title{Continuous responses}
\usage{
continuous.responses(
  annotation.vector,
  model,
  method = "t-test",
  min.size = 2,
  data = NULL
)
}
\arguments{
\item{annotation.vector}{annotation vector with discrete factor levels, and named by the samples}

\item{model}{NetResponse model object}

\item{method}{method for enrichment calculation}

\item{min.size}{minimum sample size for a response}

\item{data}{data matrix (samples x features)}
}
\value{
List with each element corresponding to one variable and listing the responses according to association strength
}
\description{
Quantify association between modes and continuous variable
}
\examples{
res <- continuous.responses(annotation.vector = NULL, model = NULL)
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plot_responses}
\alias{plot_responses}
\title{plot_responses}
\usage{
plot_responses(
  x,
  subnet.id,
  nc = 3,
  plot_names = TRUE,
  plot_mode = "network",
  xaxis = TRUE,
  yaxis = TRUE,
  plot_type = "twopi",
  mar = c(5, 4, 4, 2),
  horiz = TRUE,
  datamatrix = NULL,
  scale = FALSE,
  ...
)
}
\arguments{
\item{x}{Result from NetResponse (detect.responses function).}

\item{subnet.id}{Subnet id.}

\item{nc}{Number of columns for an array of images.}

\item{plot_names}{Plot node names (TRUE) or indices (FALSE).}

\item{plot_mode}{network: plot responses as a subnetwork graph; matrix,
heatmap: plot subnetwork expression matrix. For both, expression of each
gene is shown relative to the mean expression level of the gene;
boxplot_data: feature-wise boxplots for hard sample-to-response assignments;
response.barplot: estimated response centroids as barplot including 95%
confidence intervals for the means; pca: PCA projection with estimated
centroids and 95% intervals. In 1-dimensional case a histogram is drawn. In
two-dimensional case the original coordinates are used.}

\item{xaxis, yaxis}{Logical. Plot row/column names.}

\item{plot_type}{Network plot mode. For instance, 'neato' or 'twopi'.}

\item{mar}{Figure margins.}

\item{horiz}{Logical. Horizontal barplot_}

\item{datamatrix}{datamatrix}

\item{scale}{scale the phylotypes to unit length (only implemented for
plot_mode = 'matrix'}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects.
}
\description{
Plot the detected transcriptional responses for a given
subnetwork.
plot_mode = 'network', xaxis = TRUE, yaxis = TRUE, plot_type = 'twopi', mar
= c(5, 4, 4, 2), horiz = TRUE, datamatrix = NULL, scale = FALSE, ...)
}
\examples{
#
#res <- detect.responses(D, netw)
#vis <- plot_responses(res, subnet.id)
}
\references{
See citation('netresponse')
}
\seealso{
\code{\link{plot_scale}}
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{split.qofz}
\alias{split.qofz}
\title{split.qofz}
\usage{
\method{split}{qofz}(qOFz, c, new.c, dat, speedup = TRUE, min.size = 4)
}
\arguments{
\item{qOFz}{qOFz}

\item{c}{c}

\item{new.c}{new.c}

\item{dat}{dat}

\item{speedup}{speedup}

\item{min.size}{min.size}
}
\value{
object
Component must have at least min.size samples to be splitted.'
}
\description{
Split q of z.

Main function of the NetResponse algorithm. 
Detect condition-specific network responses, given
network and a set of measurements of node activity in a set of
conditions. Returns a set of subnetworks and their estimated
context-specific responses.
}
\details{
INPUT: data, qOFz, hp_posterior, hp_prior, opts OUTPUT: list(new.qOFz, new.c);
* new.qOFz: posterior over labels including the split clusters.  * new.c: index
 of the newly created cluster.  DESCRIPTION: Implements the VDP algorithm step
 3a.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixture.densities.R
\name{P.rS}
\alias{P.rS}
\title{Description: Probabiity of mode given multiple samples (ie. data matrix)
Mainly for internal use; documentation will be provided later. Tools for calculating densities with Gaussian mixture models.}
\usage{
P.rS(dat, pars, log = TRUE)
}
\description{
Arguments:
 @param dat features x samples data matrix for mixture modeling
 @param pars Gaussian mixture model parameters (diagonal covariances); list with elements mu (mean vectors), sd (covariance diagonals), w (weights). The mu and sd are component x features matrices, w is vector giving weight for each component.
 @param log Logical. Return densities in log domain.
}
\details{
Returns:
  @return Probability density
}
\references{
See citation("netresponse")
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NetResponseModel-accessors.R
\name{get.subnets,NetResponseModel-method}
\alias{get.subnets,NetResponseModel-method}
\alias{get.subnets}
\title{get.subnets}
\usage{
\S4method{get.subnets}{NetResponseModel}(
  model,
  get.names = TRUE,
  min.size = 2,
  max.size = Inf,
  min.responses = 2
)
}
\arguments{
\item{model}{Output from the detect.responses function. An object of
NetResponseModel class.}

\item{get.names}{Logical. Indicate whether to return subnetwork nodes using
node names (TRUE) or node indices (FALSE).}

\item{min.size, max.size}{Numeric. Filter out subnetworks whose size is not
within the limits specified here.}

\item{min.responses}{Numeric. Filter out subnetworks with less responses
(mixture components) than specified here.}
}
\value{
A list of subnetworks.
}
\description{
List the detected subnetworks (each is a list of nodes in the corresponding
subnetwork).
}
\examples{
## Load a pre-calculated netresponse model obtained with 
# model <- detect.responses(toydata$emat, toydata$netw, verbose = FALSE)
# data( toydata ); get.subnets(toydata$model) 
}
\references{
Leo Lahti et al.: Global modeling of transcriptional responses
in interaction networks. Bioinformatics (2010).  See citation('netresponse')
for details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic.mixture.univariate.R
\name{bic.mixture.univariate}
\alias{bic.mixture.univariate}
\title{Univariate BIC mixture}
\usage{
bic.mixture.univariate(x, max.modes, bic.threshold = 0, min.modes = 1, ...)
}
\arguments{
\item{x}{dat vector (for univariate analysis) or a matrix (for multivariate analysis)}

\item{max.modes}{Maximum number of modes to be checked for mixture model selection}

\item{bic.threshold}{BIC threshold which needs to be exceeded before a new mode is added to the mixture.}

\item{min.modes}{minimum number of modes}

\item{...}{Further optional arguments to be passed}
}
\value{
Fitted latent class model (parameters and free energy)
}
\description{
Latent class analysis based on (infinite) Gaussian mixture
model. If the input (dat) is data matrix, a multivariate model is fitted. If
the input is a vector or a 1-dimensional matrix, a univariate model is
fitted.
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{PlotMixtureBivariate}
\alias{PlotMixtureBivariate}
\title{PlotMixtureBivariate}
\usage{
PlotMixtureBivariate(
  x,
  means,
  sds,
  ws,
  labels = NULL,
  confidence = 0.95,
  main = "",
  ...
)
}
\arguments{
\item{x}{data matrix (samples x features)}

\item{means}{mode centroids (modes x features)}

\item{sds}{mode standard deviations, assuming diagonal covariance 
matrices (modes x features, each row giving the sqrt of covariance 
diagonal for the corresponding mode)}

\item{ws}{weight for each mode}

\item{labels}{Optional: sample class labels to be indicated in colors.}

\item{confidence}{Confidence interval for the responses based on the
covariances of each response. If NULL, no plotting.}

\item{main}{title text}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects.
}
\description{
Visualize data, centroids and response confidence intervals
for a given
Gaussian mixture model in two-dimensional (bivariate) case. Optionally, 
color the samples according to annotations labels.
}
\examples{
#plotMixture(dat, means, sds, ws)
}
\references{
See citation('netresponse') for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bic.mixture.multivariate.R
\name{bic.mixture.multivariate}
\alias{bic.mixture.multivariate}
\title{Multivariate BIC mixture}
\usage{
bic.mixture.multivariate(x, max.modes, bic.threshold = 0, min.modes = 1, ...)
}
\arguments{
\item{x}{matrix (for multivariate analysis)}

\item{max.modes}{Maximum number of modes to be checked for mixture
model selection}

\item{bic.threshold}{BIC threshold which needs to be exceeded before a
new mode is added to the mixture.}

\item{min.modes}{Minimum number of modes to be checked for mixture
model selection}

\item{...}{Further optional arguments to be passed}
}
\value{
Fitted latent class model (parameters and free energy)
}
\description{
Latent class analysis based on (infinite) Gaussian mixture
model.
If the input (dat) is data matrix, a multivariate model is fitted.
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/independent.models.R
\name{independent.models}
\alias{independent.models}
\title{independent.models}
\usage{
independent.models(datamatrix, params)
}
\arguments{
\item{datamatrix}{datamatrix}

\item{params}{parameters}
}
\value{
\item{nodes }{Model for each node} 
  \item{C }{Costs for individual models}
}
\description{
Mainly for internal use. Provide independent models for each node.
}
\examples{
res <- independent.models(datamatrix = NULL, params = NULL)
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkg-package.R
\docType{data}
\name{toydata}
\alias{toydata}
\title{toydata}
\format{
Toy data: a list with three elements:

emat: Data matrix (samples x features). This contains the same features that
are provided in the network (toydata$netw). The matrix characterizes
measurements of network states across different conditions.

netw: Binary matrix that describes pairwise interactions between features.
This defines an undirected network over the features. A link between two
nodes is denoted by 1.

model: A pre-calculated model. Object of NetResponseModel class, resulting
from applying the netresponse algorithm on the toydata with model <-
detect.responses(D, netw).
}
\usage{
data(toydata)
}
\description{
Toy data for NetResponse examples.
}
\examples{
  data(toydata)
  D    <- toydata$emat   # Response matrix (samples x features)
  netw <- toydata$netw   # Network between the features
  model <- toydata$model # Pre-calculated NetResponseModel obtained with
                         # model <- detect.responses(D, netw)
}
\references{
Leo Lahti et al.: Global modeling of transcriptional responses
in interaction networks. Bioinformatics (2010).
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NetResponseModel-accessors.R
\name{getqofz,NetResponseModel-method}
\alias{getqofz,NetResponseModel-method}
\alias{getqofz}
\title{Sample-to-response matrix of probabilities P(r|s).}
\usage{
getqofz(model, subnet.id, log = FALSE)
}
\arguments{
\item{model}{NetResponseModel object.}

\item{subnet.id}{Subnetwork to investigate.}

\item{log}{Output in log probabilities.}
}
\value{
Samples x responses matrix. Each entry is a probability P(r|s).
}
\description{
Retrieve P(r|s) from NetResponseModel model.
}
\details{
Calculates probability density for each response on a given sample based on
the estimated Gaussian mixture model.
}
\examples{

# qofz <- getqofz(model, subnet.id, log = FALSE)

}
\references{
See citation('netresponse').
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkg-package.R
\docType{package}
\name{netresponse-package}
\alias{netresponse-package}
\alias{netresponse}
\title{NetResponse: Global modeling of transcriptional responses in interaction networks}
\description{
Global modeling of transcriptional responses in interaction networks.
\tabular{ll}{ Package: \tab netresponse\cr Type: \tab Package\cr Version:
\tab See sessionInfo() or DESCRIPTION file\cr Date: \tab 2011-02-03\cr
License: \tab GNU GPL >=2\cr LazyLoad: \tab yes\cr }
}
\examples{
\dontrun{
# Define parameters for toy data
Ns <- 200  # number of samples (conditions)
Nf <- 10   # number of features (nodes)
feature.names <- paste('feat', seq(Nf), sep='')
sample.names  <- paste('sample', seq(Ns), sep='') 
# random seed
set.seed( 123 )
# Random network
netw <- pmax(array(sign(rnorm(Nf^2)), dim = c(Nf, Nf)), 0)
# in pathway analysis nodes correspond to genes
rownames(netw) <- colnames(netw) <- feature.names
# Random responses of the nodes across conditions 
D <- array(rnorm(Ns*Nf), dim = c(Ns,Nf), dimnames = list(sample.names, feature.names))
D[1:100, 4:6]  <- t(sapply(1:(Ns/2),function(x){rnorm(3, mean = 1:3)}))
D[101:Ns, 4:6] <- t(sapply(1:(Ns/2),function(x){rnorm(3, mean = 7:9)}))
# Calculate the model
#model <- detect.responses(D, netw)
## Subnets (each is a list of nodes)
#get.subnets( model )
}
}
\references{
Leo Lahti et al.: Global modeling of transcriptional responses
in interaction networks. Bioinformatics (2010). See citation('netresponse')
for details.
}
\author{
Leo Lahti, Olli-Pekka Huovilainen, Antonio Gusmao and Juuso
Parkkinen. Maintainer: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.network.R
\name{check.network}
\alias{check.network}
\title{check.network}
\usage{
check.network(network, datamatrix, verbose = FALSE)
}
\arguments{
\item{network}{Input network, see detect.responses}

\item{datamatrix}{Input datamatrix, see detect.responses}

\item{verbose}{Print intermediate messages}
}
\value{
\item{formatted }{Formatted network (self-links removed)}
  \item{original }{Original network (possible in another representation format)} 
  \item{delta }{Cost function changes corresponding to the 'formatted' network.} 
  \item{nodes }{Nodes corresponding to the 'formatted' network.}
}
\description{
Internal use to check input network and format detect.responses.
}
\examples{
# check.network(network, datamatrix, verbose = FALSE)
}
\references{
See citation('netresponse')
}
\seealso{
detect.responses
}
\author{
Maintainer: Leo Lahti \email{leo.lahti@iki.fi}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.mis.R
\name{get.mis}
\alias{get.mis}
\title{get.mis}
\usage{
get.mis(datamatrix, network, delta, network.nodes, G, params)
}
\arguments{
\item{datamatrix}{datamatrix}

\item{network}{network}

\item{delta}{delta}

\item{network.nodes}{network.nodes}

\item{G}{G}

\item{params}{params}
}
\value{
mutual information matrix
}
\description{
Estimate mutual information for node pairs based on
the first principal components.
}
\references{
See citation('netresponse')
}
\author{
Maintainer: Leo Lahti \email{leo.lahti@iki.fi}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plot_scale}
\alias{plot_scale}
\title{plot_scale}
\usage{
plot_scale(
  x,
  y,
  m = NULL,
  cex.axis = 1.5,
  label.step = 2,
  interval = 0.1,
  two.sided = TRUE,
  label.start = NULL,
  Nlab = 3,
  ...
)
}
\arguments{
\item{x}{Breakpoints for the plot_}

\item{y}{Color palette.}

\item{m}{Breakpoints' upper limit.}

\item{cex.axis}{Axis scale.}

\item{label.step}{Density of the labels.}

\item{interval}{Interval.}

\item{two.sided}{Plot two-sided (TRUE) or one-sided (FALSE) visualization.}

\item{label.start}{Label starting point.}

\item{Nlab}{Number of labels to plot_}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects.
}
\description{
Plot the color scale used in visualization.
}
\examples{
#
  #res <- detect.responses(D, netw, verbose = FALSE)
  #vis <- plot_responses(res, subnet.idx)
  #plot_scale(vis$breaks, vis$palette)
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti <leo.lahti@iki.fi>
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_matrix.R
\name{plot_matrix}
\alias{plot_matrix}
\title{Visualize a matrix with one or two-way color scale.}
\usage{
plot_matrix(
  mat,
  type = "twoway",
  midpoint = 0,
  palette = NULL,
  colors = NULL,
  col.breaks = NULL,
  interval = 0.1,
  plot_axes = "both",
  row.tick = 1,
  col.tick = 1,
  cex.xlab = 0.9,
  cex.ylab = 0.9,
  xlab = NULL,
  ylab = NULL,
  limit.trunc = 0,
  mar = c(5, 4, 4, 2),
  ...
)
}
\arguments{
\item{mat}{matrix}

\item{type}{String. Specifies visualization type. Options: 'oneway' 
        (color scale ranges from white to dark red; 
the color can be changed if needed); 
'twoway' (color scale ranges from dark blue 
through white to dark red; colors can be changed 
if needed)}

\item{midpoint}{middle point for the color plot: smaller values are 
   shown with blue, larger are shown with red 
in type = 'twoway'}

\item{palette}{Optional. Color palette.}

\item{colors}{Optional. Colors.}

\item{col.breaks}{breakpoints for the color palette}

\item{interval}{interval for palette color switches}

\item{plot_axes}{String. Indicates whether to plot 
x-axis ('x'), y-axis ('y'), or both ('both').}

\item{row.tick}{interval for plotting row axis texts}

\item{col.tick}{interval for plotting column axis texts}

\item{cex.xlab}{use this to specify distinct font size for the x axis}

\item{cex.ylab}{use this to specify distinct font size for the y axis}

\item{xlab}{optional x axis labels}

\item{ylab}{optional y axis labels}

\item{limit.trunc}{color scale limit breakpoint}

\item{mar}{image margins}

\item{...}{optional parameters to be passed to function 'image', see
help(image) for further details}
}
\value{
A list with the color palette (colors), 
       color breakpoints (breaks), and palette function (palette.function)
}
\description{
Fast investigation of matrix objects; standard visualization
choices are made automatically; fast and easy-to-use but does not
necessarily provide optimal visualization.
}
\examples{
  mat <- rbind(c(1,2,3,4,5), c(1, 3, 1), c(4,2,2))
  plot_matrix(mat, 'twoway', midpoint = 3) 
}
\references{
See citation('microbiome')
}
\author{
Leo Lahti \email{microbiome-admin@googlegroups.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list.responses.continuous.R
\name{list.responses.continuous.single}
\alias{list.responses.continuous.single}
\title{Investigate association of a continuous variable and the modes}
\usage{
list.responses.continuous.single(
  annotation.df,
  groupings,
  method = "t-test",
  pth = Inf,
  verbose = TRUE,
  rounding = NULL,
  adjust.p = TRUE
)
}
\arguments{
\item{annotation.df}{annotation data.frame with discrete factor levels, rows
named by the samples}

\item{groupings}{Sample mode information. Each element corresponds to one of the modes and lists the samples assignment matrix qofz. Alternatively, a vector of mode indices named by the samples can be given.}

\item{method}{method for quantifying the association}

\item{pth}{p-value threshold (for adjusted p-values)}

\item{verbose}{verbose}

\item{rounding}{rounding digits}

\item{adjust.p}{Adjust p-values (this will add p.adj column and remove pvalue column in the output table)}
}
\value{
Table listing all associations between the factor levels and responses
}
\description{
Investigate association of a continuous variable and the modes.
}
\examples{
res <- list.responses.continuous.single(annotation.df = NULL, groupings = NULL)
}
\references{
See citation('netresponse')
}
\author{
Contact: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toydata.R
\name{generate.toydata}
\alias{generate.toydata}
\title{generate.toydata}
\usage{
D <- generate.toydata()
}
\arguments{
\item{Dim}{Dimensionality of data}

\item{Nc}{Number of modes}

\item{Ns}{Number of data points}

\item{sd0}{Component spread}

\item{rgam.shape}{Shape parameter for Gamma distribution}

\item{rgam.scale}{Scale parameter for Gamma distribution}
}
\value{
Simulated data matrix (samples x features)
}
\description{
Generate toy data.
}
\examples{
D <- generate.toydata()
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.netw.R
\name{filter.netw}
\alias{filter.netw}
\title{filter.netw}
\usage{
filter.netw(network, delta, datamatrix, params)
}
\arguments{
\item{network}{network}

\item{delta}{associated cost function value changes for each node merge}

\item{datamatrix}{datamatrix}

\item{params}{parameters}
}
\value{
Filtered network
}
\description{
Mostly for internal use. Prefilter edges if speedups required.
}
\details{
Include only edges with the highest mutual information,
calculated based on the first principal components.
}
\references{
See citation('netresponse')
}
\author{
Leo Lahti, Olli-Pekka Huovilainen and Antonio Gusmao.  Maintainer:
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plotPCA}
\alias{plotPCA}
\title{plotPCA}
\usage{
plotPCA(x, subnet.id, labels = NULL, confidence = 0.95, npoints = NULL, ...)
}
\arguments{
\item{x}{NetResponseModel object. Output from the detect.responses function.}

\item{subnet.id}{Subnetwork id. Either character as 'Subnetwork-2' or
numeric as 2, which is then converted to character.}

\item{labels}{Optional: sample class labels to be indicated in colors.}

\item{confidence}{Confidence interval for the responses based on the
covariances of each response. If NULL, no plotting.}

\item{npoints}{Argument to the ellipse function}

\item{...}{Further arguments for plot function.}
}
\value{
Used for its side-effects.
}
\description{
Visualize data, centroids and response confidence intervals
for a given
subnetwork with PCA. Optionally, color the samples according to annotations
labels.
}
\examples{
#plotPCA(x, subnet.id)
}
\references{
See citation('netresponse') for citation details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-methods.R
\name{plot_data}
\alias{plot_data}
\title{Plot observed data.}
\usage{
plot_data(x, subnet.id, labels, ...)
}
\arguments{
\item{x}{NetResponseModel object.}

\item{subnet.id}{Specify the subnetwork.}

\item{labels}{Annotation categories.}

\item{...}{Further arguments for plot function.}
}
\value{
ggplot2 plot object
}
\description{
Plotting tool for measurement data.
Produces boxplot for each feature in each annotation category for the
selected subnetwork.
}
\examples{
#
}
\references{
See citation('netresponse')
}
\seealso{
plot_responses
}
\author{
Leo Lahti <leo.lahti@iki.fi>
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{get.model.parameters}
\alias{get.model.parameters}
\title{get.model.parameters}
\usage{
get.model.parameters(model, subnet.id = NULL)
}
\arguments{
\item{model}{Result from NetResponse (detect.responses function).}

\item{subnet.id}{Subnet identifier. A natural number which specifies one of
the subnetworks within the 'model' object.}
}
\value{
A list with the following elements: \item{mu}{ Centroids for the
mixture components. Components x nodes.} \item{sd}{ Standard deviations for
the mixture components. A vector over the nodes for each component, implying
the diagonal covariance matrix of the model (i.e. diag(std^2)). Components x
nodes} \item{w}{Vector of component weights.} \item{nodes}{List of nodes in
the subnetwork.} \item{K}{Number of mixture components.}
}
\description{
Retrieve the mixture model parameters of the NetResponse algorithm for a
given subnetwork.
}
\details{
Only the non-empty components are returned. Note: the original data matrix
needs to be provided for function call separately.
}
\examples{

# Load toy data
data( toydata )          # Load toy data set
D     <- toydata$emat    # Response matrix (for example, gene expression)
model <- toydata$model   # Pre-calculated model

# Get model parameters for a given subnet
# (Gaussian mixture: mean, covariance diagonal, mixture proportions)
get.model.parameters(model, subnet.id = 1)

}
\references{
Leo Lahti et al.: Global modeling of transcriptional responses
in interaction networks. Bioinformatics (2010). See citation('netresponse')
for details.
}
\author{
Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pick.model.pairs.R
\name{pick.model.pairs}
\alias{pick.model.pairs}
\title{Pick model pairs}
\usage{
pick.model.pairs(network, network.nodes, node.models, datamatrix, params)
}
\arguments{
\item{network}{network}

\item{network.nodes}{network.nodes}

\item{node.models}{node.models}

\item{datamatrix}{datamatrix}

\item{params}{parameters}
}
\value{
\item{model.pairs}{joint models for each node pair}
\item{delta}{corresponding delta value for the cost function}
}
\description{
Mainly for internal use. Calculate joint model for each node pair
}
\examples{
#
}
\references{
See citation('netresponse')
}
\author{
Maintainer: Leo Lahti \email{leo.lahti@iki.fi}
}
\keyword{internal}
